!     Changed Nl_max from 20000 to 30000.
!     Changed Nb_max from 300 to 400 to fit Vazdekis+11 complete base.
!     Changed Npar_max from 303 to 403 to accommodate above change. 
!     In line 2961 I changed the output format to not screw up the output of Zs in log
!     Paula@Roma - 29/Dez/2018
!
!     ------------------------------------------------------------------
!     |                    StarlightChains_v05.for                     |
!     ------------------------------------------------------------------
!
!     !!Found a TRUE & +/- SERIOUS bug in StarlightChains_v04.for:( The
!     bug affects ONLY the single-component (SSP) fits carried out at
!     the end of each run. It MAY lead to wrong SSP-parameters,
!     particularly when they are already bad anyway. The bug was:
!
!         call mnbrak(ax,bx,cx,fa,fb,fc,F_SSP_chi2_AV)
!         chi2_AV_Gold = golden(ax,bx,cxf,F_SSP_chi2_AV,tol,AV_Gold)
!
!     where "cxf", instead of "cx", was passed to golden, leading to a
!     (not always) incorrect estimation of AV_Gold. Cid & Rosa noticed
!     this bug in the AV_SSP x SSP age curves while fiting star cluster
!     spectra. AV_SSP becomes constant at some values of age (but,
!     luckly, not in the range of best SSP fits). This version corrects
!     this bug.
!     
!     This version also reads a new i_SaveBestSingleCompFit flag from
!     the config file. If = 1, the best fit SSP spectrum is saved as a
!     5th column in arq_out, otherwise the output is as in v04.
!
!     Finally, I also take the chance to fix the harmless previously
!     knwon bug that when fobs_norm = 0 arq_out is saved in the running
!     dir, instead or in out_dir. Now such bugged output-files are saved
!     in out_dir.
!
!     All changes are marked with "!v05!".
!
!     Cid@Lagoa - 10/May/2008
!     ------------------------------------------------------------------


!     ------------------------------------------------------------------
!     |                    StarlightChains_v04.for                     |
!     ------------------------------------------------------------------

!     v04 = 1st public version of STARLIGHT!

!     This new version is still similar to StarlightChains_v02.for, but
!     with a large suite of new features, including some which imply
!     changes in the grid file info, base-file (arq_base), config-file
!     (arq_config) and output file (arq_out) too!
!
!     The ~ minor changes are (some imply changes in the grid, config,
!     base & arq_out files):
!
!     (01) Fixed the "infinite loop" bug plus the silly i<->j mistake in
!     arrays-reseting (found by William). Also, changed Nl_max from 7000
!     to 14000. (Paula's base has ~ 10400 pixels!).
!     
!     (02) Added some time() & etime() commands to figure how long a run
!     takes (useful to test configs).
!
!     (03) Added protections against non-increasing l_obs() & l_base(),
!     Mstars() <= 0 & idum_orig > 0.
!
!     (04) If idum_orig = 0 in the grid.in file, then idum_orig will be
!     kept the same for all runs in the grid file. Also added protection
!     against idum_orig > 0.
!
!     (05) Added a nice & simple feature: If the output file arq_out
!     already exists, then we SKIP THE FIT altogether and go to the next
!     one on the list! Very useful to run grids in "modo-burro"! At
!     William's request, I added a Y/N flag in arq_config which decides
!     whether this trick is to be used or not.
!
!     (06) In preparation for alpha enhanced bases, I have increased to
!     character*15 the size of the component nickname (base_name), and
!     introduced a alphaFe_base() arrays which MUST be read from
!     arq_base. alphaFe_base() is echoed in the output, along with x &
!     etc.
!
!     (07) Mask base edges! The first & last dl_cushion Angs in the base
!     spectra will be masked to prevent edge effects in the kinematical
!     filter! 
!
!     (08) dl_cushion used to be fixed at 50 Angs in previous
!     versions. Now we read it from arq_config! (whose format thus
!     changed!)
!
!     (09) After STARLIGHT is done (ie, just before output) we now do
!     "Single-Component-Fits", where each of the base spectra is
!     compares to the data individually. We fit AV and the
!     norm. factor. The results are stored along with the main fit pop
!     vector (see SSP_* arrays). This may be used to define a "best
!     single component" representing a spectrum (potentially usefull for
!     star-clusters and elliptical galaxies.)
!
!     (10) Now reading fEX0_MinBaseSize from arq_config. This is used
!     to define N_base_min = nint(fEX0_MinBaseSize * N_base) in EX0-fit.
!
!     (11) Removed unnecessary common/YAV_stuff/ in ConvertLight2Mass.
!
!     (12) Now the EX0s routine will NOT be called when NEX0s_loop = 0.
!     Part of the output was computed & printed with EX0s* parameters
!     (like x, av, v0, vd...), so I had to fix that to allow for the
!     NEX0s_loop = 0 case. (changes marked with "!Cid24Mar07-FIX!")
!
!     (13) Added "censored weights" (Weight-control-filter-stuff) info
!     in the output file just for control-purposes.
!
!     (14) Added a scheme to count number of columns in arq_obs to check
!     consistency of IsErrSpecAvailable & IsFlagSpecAvailable...
!
!     (15) Added protection againts using i_FastBC03_FLAG = 1 and non
!     BC03/STELIB models. If this happens, reset i_FastBC03_FLAG to 0.
!
!     (16) The grid file now hase obs_dir, mask_dir & out_dir, which are
!     dire directorieds whre arq_obs & arq_masks will be read from and
!     arq_out will be written to.
!     
!     (17) Format of l_obs on arq_out changed from i4 to f8.2 to cope
!     with IR data and high-res too.
!
!
!     The ~ major changes (implying changes in the grid file) are:
!
!     (01) grid-file "header" info is now vertical! Ie, the previoulsy
!     horribly long 3rd line input is now broken into one thing per
!     line. Similarly, the config file has been restructured and
!     reordered for clarity! 
!
!     (02) Introduced Gordon et al (2003) SMC & LMC extinction curves
!     (red_law_option = GD1, GD2 & GD3). They are still interpolated
!     from a table, but better and different from HyperZ-curves.
!     Also, introduced a general "from-a-file" reddening law read from a
!     file named ExtinctionLaw.red_law_option! 
!
!     (03) Now the initial estimates for v0 & vd are read FOR EACH
!     GALAXY from the grid file! The previous version all used v0 = 0 &
!     vd = 150 km/s as a starting point. This implied a change to the
!     format of grid files...
!
!     (04) Introduced a new "FXK" option for the FitOrSamplePDF_option
!     string, which keeps vd & v0 FIXED throught the fits. In other
!     words, kinematics is NOT fitted at all, and v0 & vd are kept = to
!     their initial values (now read from the grid file)! The base is
!     shifted by v0 & vd before any call to "Scubidu" routines and never
!     shifted nor smoothed again. This may be useful in a number of
!     cases. This was the major change. Search for "FXK" to find where
!     the changes were made.
!
!     (05) While writing the manual and checking the code I found an
!     invisible bug! In the Refit-after-clipping and Burn-In calls to
!     Fit_Scubidu the variable i_RestartChains was UNDEFINED! That means
!     it was 0, and neither in the Refit nor Burn-In stages the chains
!     were reset to the best parameters... I now set i_RestartChains =
!     i_RestartChains_FF in both calls. This was most lilkey harmless,
!     though (in general) we should now get slightly better fits (if
!     i_RestartChains_FF = 1, of course).
!
!     Later (01April07) I found another related bug! The new bug is that
!     by setting the last values equal to the best ones (in Fit_Scubi
!     with i_RestartChains = 1), if one of these best chain values = the
!     global best so far then the trick of resetting the chain with
!     worst energy = to the best global model may result in this
!     chain-values appearing TWICE!! Ie, 2 of the chains would start
!     from IDENTICAL positions.... To see how I fixed this search for
!     "Cid01April07-FIX".  I also introduce a new option:
!     i_RestartChains = 2, which will reset ONLY the worst chain to the
!     best model, but leave the others wherever they are.
!
!
!     Cid@Lagoa - 07/February/2007 --> 01/April/2007

!

!     **Disorganized list of things to improve**

!     - Better/analytical reddening laws for SMC & LMC! & fix minor
!     "step" in CAL law. Also, R_V parameter in calc_red_term is
!     obsolete.

!     - The i_FastBC03_FLAG for Fast-rebinning should be changed to cope
!     with other bases...

!     - Write robust resampling routine...

!     -Option to deal with rectified spectra??? Dificult to see how this
!     can be done...

!     -COMO FIXAR AV E NAO FITA-LO?? ISTO SERAH OUTRO PROG, COM
!     RETIFICACAO E O ESCAMBAU...  UMA possibilidade eh inventar
!     red_law_option = 'NONE' e setar red_term(lambda) = 0 ou 1 ...
!     desse modo o av/yav variaria mas sem efeito pratico (?) e talvez
!     as chains convirjam anyway...  neste caso teriamos que mudar
! 100     AV_test  = 0.3
!         YAV_test = 0.3
!     onde se testa os chi2 slow x fast... que acontece se setar NRC_AV=0?
!     ------------------------------------------------------------------


!     ------------------------------------------------------------------
!     |                      StarlightChains_v02.for                   |
!     ------------------------------------------------------------------
!     This version fixes a few minor things in StarlightChains_v01.for.
!     Changes are essentially cosmetic or safety-measures.
!
!     (1) No 'Running model # ...' screen output anymore.
!
!     (2) Introduced Is1stLineHeader flag in config.
!
!     (3) If fobs_norm <= 0 the run is aborted.
!
!     (4) Added safety-check in the computation of S/N's in the
!     llow_SN--lupp_SN window...
!
!     (5) Changed a bit the way we constrain initial chain fn, AV & YAV
!     values to allow them to be constrained by the low->upp values in
!     the config file.
!
!     (6) Introduced IsScaleNstepsInEX0sFit flag in config. If turned
!     on, the numbers N_sim & Nmax_steps, which control the length of
!     Metropolis runs in Scubidu, will be scaled according to the actual
!     number of relevant parameters in the EX0s fits. This ONLY changes
!     things in the EX0s fits.

!     (7) Introduced a IsNoKin4LargeBaseInEX0sFits flag & a
!     frac_NoKin4LargeBaseInEX0sFits variable, which, multiplied by
!     N_base, defines what is the minimum size of a EX0s-condensed base
!     for which we turn-off kinematical fits. (In v01 this was fixed at
!     50 elements). This too ONLY changes things in the EX0s fits.
!
!     (8) Introduced the 5 reddening laws of HYPERZ ('HZ1' ... 'HZ5'),
!     of which HZ4 is the SMC (which I'd expect to be most relevant)
!
!     Only (6) & (7) may have some relevant effect wrt v01, particular
!     (6). Setting IsScaleNstepsInEX0sFit = 0 should yield nearly
!     identical results, but setting it to 1 will result in shorter
!     (faster) EX0s-fits, which may be slightly worse in chi2.
!
!     Cid@Lagoa - 09/October/2005
!     ------------------------------------------------------------------



!     This is my "1st-final" version of "ScubiStar" (built upon
!     ScubiStar7.for version produced at IAA in May/2005). The main
!     difference wrt previous versions of Starliht is that this version
!     uses N_chains parallel Markov chains, which run untill a Gelman &
!     Rubin-like Burn-In convergence criterion is satisfied. Full
!     documentation is not ready yet...
!
!     A few other cosmetic changes were made. Base files are now stored
!     in a user-defined directory (read from the input file), avoiding
!     multiple copies of base-spectra throughout the disk.  Another
!     change is that now the input file specifies two flags
!     (IsErrSpecAvailable & IsFlagSpecAvailable) which define whether
!     error & flag-spectra are read from the input file (as for SDSS
!     data) or are computed online with the old rms-in-a-window
!     method. Yet another change is that now the base, mask and config
!     files & extinction curve-option are specified for each galaxy!
!     These changes (and others) were made in the sense of having a
!     single version of the code useful for different purposes (eg, SDSS
!     and my LLAGN data). More details will be added later.
!
!     I have not run a full set of tests/simulations of this code...
!     However, quick & dirty tests of different configurations show
!     that results are very similar for all configurations I
!     tried. Hence, I'll distribute the code and see how it
!     performs. Changes will be made "on-the-fly", ie, as problems appear.
!
!     Sent to Abilio on 30/May/2005
!
!     Cid@IAA - 30/May/2005

c     voltei a mexer no codigo - ... abstracts-totalmente desatualizados....
c     Cid@IAA - 3/May/2005
c
c     Tentando botar Scubidu no Starlight05_Fast_v01.for...
c     melhor seguir testes sem YAV e depois resolver este bug novo...
c     Cid - 17-2-05


c     @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c     This is my latest update of the Starlight spectral synthesis code.

c     BRIEF HISTORY of Starlight: So far there have been 2 main
c     versions.

c     (1) Starligh03_v02.for (September/2003) was the original code,
c     used in the Seyfert 2 paper (CF, Gu, Melnick ... et al 2004;
c     MNRAS). The same code was used in my LLAGN paper III (CF, Gonzalez
c     Delgado ... et al 2004; MNRAS), with a base of template
c     galaxies. Initial experiments with SDSS spectra were also
c     performed using a slightly modified version of the code
c     (Starlight03_v04SDSS.for; which dealt with errors and flags).

c     (2) In Starlight03_v04SDSS_Fast2.for I've introduce the trick of
c     expanding the extinction term in a truncated series (the
c     "RapidChi2" or RC-trick), which speeds things up by > 200 times!
c     That was used in SDSS-Paper-I (CF, Mateus ... et al MNRAS 2005). A
c     slightly modified version called Starlight03_v04_ind_Fast2.for was
c     produced to deal with individual spectra (which require galaxy by
c     galaxy input info); it also introduced the Calzetti reddening-law
c     option.

c     NEW FEATURES OF THIS VERSION: The main novelty in this 3rd version
c     is that we now allow for 2 different extinctions: AV & YAV. Each
c     population j (j=1...N_base) is reddened by AV_tot(j) = AV + YAV,
c     and only some of them are allowed to have the extra extinction
c     YAV. The array ind_YAV(j) says who can have this extra
c     extinction. The idea is to do something like a Charlot & Fall
c     (2000) extinction law, which has one (higher) AV for young
c     populations (<= 10 Myr) and a global AV affecting both young and
c     old stars. However, the SAME reddening law is assumed for both
c     populations, otherwise heavy changes would be required in the
c     RC-chi2 routines... The code was also re-organized, with
c     improvement structure and comments. Finally, I now read the base
c     DIRECTORY (string base_dir) from the input/grid file (2nd
c     line). This avoids duplication of base spectral files. Use "./"
c     for base_dir if the base files are in the current directory!
c     Note that symbolic links work:)

c     Other changes: Allowance for returned-mass in the light-to-mass
c     fractions conversion was introduced. The conjugate-gradient
c     routines were removed altogether (they never proved worthy). Both
c     config & base files now have different formats. The output file
c     too is now slightly different (so auxiliary table-making &
c     plotting codes will have to be updated accordingly).

c     FUTURE CHANGES: There is a major change in progress to make the
c     whole algorithm more efficient, using the estimates of the
c     covariance matrix to identify principal directions for movement in
c     the parameter space. The burn-in phase should also be mapped, and
c     full error estimates of the parameters will be done. Jean, William
c     & myself are working on it.

c     The code is heavily commented, but not yet fully comprehenssible
c     by people other then myself! Full documentation & tips will be
c     provided elsewhere, one day...
c
c     Cid@Lagoa - 23/January/2005
c     @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@




c ###########################################################################
c #                         DRIVER FOR MAIN CODE                            #
c ###########################################################################
      program DRIVER_4_Starlightv05
      implicit real (a-h,j-m,o-z)
      implicit integer (i,n)
      character*3 FitOrSamplePDF_option
!Cid29Mar07-FIX! Defining base, obs, mask & out directories strings!
      character*100 base_dir , obs_dir , mask_dir , out_dir

c     1st line in input/grid-file is the number of fits to be computed
      read (*,*) N_models

!Cid20Jan07-FIX! Now read grid-file stuff sequentially!
c     Input some global grid-parameters here & pass them on to main code
      read (*,'(a)') base_dir
      read (*,'(a)') obs_dir
      read (*,'(a)') mask_dir
      read (*,'(a)') out_dir
      read (*,*) idum_orig
      read (*,*) llow_SN
      read (*,*) lupp_SN
      read (*,*) Olsyn_ini
      read (*,*) Olsyn_fin
      read (*,*) Odl_syn
      read (*,*) fscale_chi2
      read (*,*) FitOrSamplePDF_option
      read (*,*) IsErrSpecAvailable
      read (*,*) IsFlagSpecAvailable

!Cid14Jan07-FIX! idum-trick: Sometimes I may want to repeat the run with 
!     the same idum, but different config, or base, or red-law. To allow
!     for this possibility using a single grid-file, just set idum = 0
!     in the grid.in file and all runs will have the same idum:)
!     Also, added a protection against idum_orig > 0.
      i_ChangeIdum = 1
      if (idum_orig.EQ.0) then
         i_ChangeIdum = 0
         idum_orig = -123456
      end if
      if (idum_orig.GT.0) idum_orig = -idum_orig

!Cid14Jan07-FXK! Added FXK = Fixed kinematics option!!
c     Safety-check on some of the input info...
      if ((FitOrSamplePDF_option.NE.'PDF').AND.
     $     (FitOrSamplePDF_option.NE.'FIT').AND.
     $     (FitOrSamplePDF_option.NE.'FXK')) then
         write (*,'(a)') ' Problem with your grid file! '
         write (*,'(a,a,a)') ' What are you doing? ' ,
     $        FitOrSamplePDF_option ,' should be = FIT or FXK ...'
         stop
      end if 

      if ((IsErrSpecAvailable.NE.0).AND.(IsErrSpecAvailable.NE.1)) then
         write (*,'(a)') ' Problem with your grid file! '
         write (*,'(a,a,i3)') ' What are you doing? ' ,
     $        ' IsErrSpecAvailable = ' , IsErrSpecAvailable
         stop
      end if 

      if ((IsFlagSpecAvailable.NE.0).AND.(IsFlagSpecAvailable.NE.1))then
         write (*,'(a)') ' Problem with your grid file! '
         write (*,'(a,a,i3)') ' What are you doing? ' ,
     $        ' IsFlagSpecAvailable = ' , IsFlagSpecAvailable
         stop
      end if 

      if ((IsErrSpecAvailable.EQ.0).AND.(IsFlagSpecAvailable.EQ.1)) then
         write (*,'(a)') ' Problem with your grid file! '
         write (*,'(a,a,i3,a,i3)') ' What are you doing? ' ,
     $        ' IsFlagSpecAvailable = ' , IsFlagSpecAvailable ,
     $        ' but IsErrSpecAvailable = ' , IsErrSpecAvailable
         stop
      end if 


c     Run grid of N_models spectral fits
      do i_model=1,N_models

         call FitSpectrum(idum_orig,llow_SN,lupp_SN,Olsyn_ini,Olsyn_fin
     $        ,Odl_syn,fscale_chi2,base_dir,obs_dir,mask_dir,out_dir
     $        ,FitOrSamplePDF_option,IsErrSpecAvailable
     $        ,IsFlagSpecAvailable)

c     changes idum-seed just to avoid doing always the same sequence of
c     random numbers
!Cid30Dec06-FIX! idum-trick: only changes if i_ChangeIdum.NE.0!!
         if (i_ChangeIdum.NE.0) idum_orig = idum_orig - 1

      end do


      end
c ###########################################################################



c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c @                                MAIN CODE                                @
c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!     ATT: This is an incomplete & rather disorganized ABSTRACT of the
!     code. Full documentation will be produced one day... (yeah, right...)

c     This code synthesizes a galaxy spectrum f_obs(l_obs) with a base
c     of N_base spectra f_base(l_base). The base can be made up of
c     observed template galaxies or theoretical spectra (eg, BC03,
c     Starburst99, etc). The ouput consists of the population vector
c     x(i); i=1...N_base, plus the V-band global extinction AV and the
c     extra-extinction YAV to selected components, plus kinematical
c     parameters v0 (residual velocity-shift) & vd (velocity
c     dispersion).

c     The code seeks to minimize the chi2:
c
c     chi2(x,AV,YAV,v0,vd) = SUM [(f_syn - f_obs) * weight_obs]**2
c
c     where the sum is over all lambdas, and f_syn =
c     f_syn(x,AV,YAV,v0,vd) is the model spectrum for the given
c     parameters:
c
c     f_syn(i) = Sum_j x(j) * f_base(j,i) * 10.**(AV_tot(j) * red_term(i))
c                CONVOLVED with G(v0,vd)
c
c     where f_base(j,i) is the base spectrum of component # j at lambda
c     l_obs(i), convolved with the Gaussian kinematical filter G(v0,vd).
c
c     Masked lambdas are given weight_obs = 0. For other lambdas,
c     weight_obs = 1 / error_spectrum, or mask_weight times larger if
c     specified on arq_masks.

c     * NORMALIZATION: The base spectra f_base are normalized by their
c     flux at l_norm, while the observed spectrum is normalized by
c     fobs_norm = the median flux between llow_norm & lupp_norm (which
c     must bracket l_norm).  We do NOT force the population vector to be
c     normalized to 100%! The returned value of x_j is the flux in the
c     model spectrum (without convolution by G!) in component j, in unit of
c     fobs_norm.

c     * ERROR & FLAG SPECTRA: The input spectrum MUST come with an
c     associated error spectrum (err_obs) and a flag-spectrum
c     (flag_obs). The coding for the flag spectrum is like this: flag =
c     0 or 1 is OK, while flag > 1 will ignore the points from the fit
c     (like the masks described below). Flags are useful to identify bad
c     pixels. If you do NOT have an error spectrum, then invent one, but
c     beware that the chi2's computed here depend on this error.

c     * MASKS: The synthesis is carried out in all Nl_obs observed
c     wavelengths l_obs, but some are "masked out". The user-supplied
c     file arq_masks indicates which intervals should be excluded from
c     the modeling (eg, those containing emission lines or bad
c     pixels). We further impose a limit f_obs >= f_cut (in units of the
c     normalization flux). Weaker fluxes are masked out assignig
c     weight_obs = 0 for f_obs <= f_cut. (Just set f_cut = 0 if you do
c     not want this safety feature.)  The file arq_masks can also
c     specify windows over which we want to give larger weight (for
c     instance, on key absorption features). The array mask_weight()
c     decides by how much the weight should be multiplied in a given
c     window. (Example: mask_weight = 0 ignores the region in the fit
c     and mask_weight = 2 gives is twice as much weight as it would have
c     otherwise.)

c     * SAMPLING: The input spectrum f_obs(l_obs) is resampled/rebinned in
c     wavelength to lsyn_ini ==> lsyn_fin, in steps of dl_syn
c     (subroutine RebinSpec). This is the range which will actually be
c     modeled (but note that we internaly extend the desired fitting
c     range by 50 Angs on the blue & red edges to circumvent problems in
c     the kinematical filter). The base spectra f_base are resampled to
c     the same wavelengths (ie, l_obs = lsyn_ini ==> lsyn_fin in dl_syn
c     steps). Note that vd depends on dl_syn, as well as on the actual
c     spectral resolution of the input spectra... To correct vd for the
c     different resolutions of observed & base spectra do a quadrature
c     correction. For instance, if your spectra have sigma_inst = 62
c     km/s and the base has 86 km/s resolution do:

c     vd_corrected = sqrt(vd_given_by_the_code**2 + 86.**2 - 62.**2)

c     ==> It is VERY STRONGLY recommended that you provide both observed
c     & base spectra already resampled to 1 Angs! (This makes things
c     easier.) One reason for this is that is that my Rebinning routine
c     does NOT handle the error spectrum properly. It just resamples
c     err_obs, whereas it should actually correct for the fact that
c     going from, say, 1 to 2 Angs sampling the error should be smaller...


c     * NUMERICAL-SCHEME: I've tried several schemes to estimate the
c     stellar population (x, AV & YAV) and kinematical parameters (v0 &
c     vd).
c
c     1 - Metropolis in the (x,AV,YAV,v0,vd) space - (slow method)
c     2 - Metropolis on x,AV followed by Metropolis on (x,AV,YAV,v0,vd).
c     3 - Annealing schemes
c     4 - etc (in previous versions I tried conjugate gradients, dicrete
c         grids, combinations of these,...)
c
c     I have settled for an adapted version of stimulated annealing,
c     which runs a series of Metropolis runs to find the best (x,AV,YAV)
c     for fixed (v0,vd) and, at each step, re-estimates (v0,vd) for
c     fixed (x,AV,YAV). [Also, after each Mertopolis run (in the
c     annealing loop) I do a quick search for better AV & YAV for fixed
c     x, v0 & vd.]
c
c     At each step in the annealing routine (Fit_xAVYAVv0vd) I multiply
c     the weights (= 1 / error) by a factor wf which runs from wf_low to
c     wf_upp. The step size (in x & AV) is concomitantly reduced from
c     eps_upp to eps_low, and the number of steps is adjusted so that a
c     long Metropolis trip can be made. The last loop is the slowest
c     one, with more points. The maximum number of steps at any given
c     annealing interaction is limited by Nmet_max. If you give a very
c     large number for this parameter then all steps that "should" be
c     made are actualy made, but it may take a long time to run...


c     [CLIPPIG & EX0s & etc] The code allows other tricks: 

c     (A) Sigma-Clipping can be made, removing all points with more
c     than, say, sig_clip_threshold = 3 sigma residuals, and re-doing
c     the fit. This can be done Nclip_loops_max times.

c     (B) Quick-Clipping: If Nclip_loops_max < 0 on input, then a "fast"
c     clipping routine is performed (only once) before going for the
c     actual fit. This works prettry well and saves time, so we
c     recommend it.

c     (C) Condensed-Base-Fitting: Setting NEX0s_llops >= 1 on arq_config
c     will re-fit the spectrum, taking as initial point the values
c     obtained in the main call to Fit_xAVYAVv0vd, and reducing the base
c     just to the N_non_zeros components which have x_j > 0. This speeds
c     up computations (because of the smaller base), and serves to
c     refine the fit.

c     (D) The rebinning of BC03 spectra is now much faster (and less
c     general, since we assume dlambda = 1). Use i_FastBC03_FLAG = 1 in
c     arq_config.

c     (E) WEIGHT-CONTROL-FILTER: I have implemented a "safety-filter" which
c     prevents any given point from having a weight more than 2 sigma
c     above the mean weight of unmasked points. I did this after
c     realizing that some SDSS spectra come with very low errors, which
c     produce large weight_obs(i) and are thus given too much weight in
c     the synthesis.

c     (F) fscale_chi2: I've introduced a "cooking factor", fscale_chi2,
c     which should be set to fscale_chi2 ~ actual-spectral-resolution /
c     Odl_syn. When you synthesize a spectrum of say, resolution 3 Angs
c     but sampled at every 1 Angs then only ~ 1 out of every 3 points is
c     actually independent. The chi2 is thus ~ 3 times larger than what
c     it should be, so dividing it by 3 would fix the chi2 scale. This
c     is NOT really relevant insofar as the value of the reduced chi2 is
c     concerned. In fact, the whole quest for a minimum chi2 through the
c     annealing scheme used here makes the actual absolute scale of chi2
c     rather irrelevant. The only noticeable effect of this re-scaling
c     of chi2 is in the METROPOLIS algorithm, where one uses a
c     likelyhood ratio to decide whether upwards steps are taken. This
c     ratio is exp(-(Enew - Eold)), where E = 0.5 * chi2, which IS
c     sensitive to the absolute scale of chi2. I therefore implement the
c     chi2 scaling ONLY in this part of the metropolis routine, which
c     becomes: exp(-(Enew - Eold) / fscale_chi2). The expected effect
c     for fscale_chi2 > 1 is that more updwards metropolis-steps will be
c     accepted, increasing the overall metropolis-efficiency. In
c     practice, since we fiddle with chi2 with the weight-factors in the
c     annealing loop, this extra-parameter probably has the same effect
c     as changing the weighting factors! The bottom line is: If you
c     believe your error spectrum (particularly its amplitude), just set
c     fscale_chi2 = 1 and forget this feature. (I myself never used it
c     seriously).


!     * RAPID-CHI2 TRICK: This used to be Starlight03_v04SDSS.for, but
!     is now VERY different. There were several minor changes and one
!     HUGE change. The huge modification is the use of a truncated
!     series to represent the extinction term in the model
!     spectrum. This simple trick, which works fine as long as AV is not
!     too large, allows a MUCH-MUCH faster calculation of the chi2 by
!     circumventing the need to sum over all lambdas every time. See
!     notes for details! The bottom line is that we can now let the
!     Metropolis routine run for MUCH-MUCH longer than before in less
!     time! (Indeed, the advantage is only noticeable when we do force
!     huge Metropolis runs.) The routines involved are Calc_RC_terms &
!     VeryFastF_chi2. The screen output register info on the difference
!     between approximate & exact chi2's, but nothing is stored in the
!     ouput files! Note that we have not eliminated the full-exact-chi2
!     routines altogether; some are still used (for output, to minimize
!     over kinematic parameters v0 & vd, etc). The faster routines are
!     used essentially only in the (x,AV,YAV) Metropolis chain...

c     The RC-scheme involves the computation of 3 terms: RC1, RC2(n,j)
c     and RC3(n,j,k). RC2 & RC3 involve the base spectra f_base(j,i). My
c     implementation of this scheme uses f_base(j,i) which are already
c     convolved with the kinematical filter G(v0,vd). This means that I
c     first convolve with G and only then apply the extinction
c     factor. This is slightly different than appliyng the extinction
c     factor and then convolving correction (the difference is small
c     because the extinction curve is ~ constant over scales of
c     vd). This RC-chi2 is inly used in the estimation of x, AV &
c     YAV. The kinematical fitting of v0 & vd uses a Golden Search
c     method with a chi2 function which included the extinction term
c     inside the convolution. This note is just to recall myself that
c     there are two slightly different chi2's in the code...

!     Still to be implemented: 

!     * A bias-correcting trick... (for the AV-expansion series)
!
!     * There might be a reasonable method to define an initial
!     annealing weight-factor (<=> Temperature) based on the typical
!     value of chi2 for a few random models and what factor
!     (Temperature) would it take to make it go to a chi2 ~ 1/N_dof in
!     not-too-many steps... Think about it!
!
!     * The "MCMC-thing", with ~ 12 paralell chains, mean & std dev
!     estimates (with burn-in convergence test)

c     ATT: This abstract is permanently under construction...
c
c     Cid@Lagoa - 27/Jan/2005


      SUBROUTINE FitSpectrum(idum_orig,llow_SN,lupp_SN,Olsyn_ini
     $     ,Olsyn_fin,Odl_syn,aux_fscale_chi2,base_dir,obs_dir,mask_dir
     $     ,out_dir,FitOrSamplePDF_option ,IsErrSpecAvailable
     $     ,IsFlagSpecAvailable)


c ***************************************************************************
c *				DEFINITIONS				    *
c ***************************************************************************
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      parameter (Nchains_max = 1000,Npar_max = 403)

c     Spectral quantities: observed, synthetic, mask & auxiliar stuff
      real l_obs(Nl_max)       , f_obs(Nl_max)
      real err_obs(Nl_max)     , weight_obs(Nl_max)
      real l_syn(Nl_max)       , f_syn(Nl_max)
      real llow_mask(Nl_max)   , lupp_mask(Nl_max)
      real mask_weight(Nl_max) , f_mask(Nl_max)
      real l_aux(Nl_max)
      real f_aux1(Nl_max)      , f_aux2(Nl_max)   , f_aux3(Nl_max)

c     Base spectra, age, Z & "absolute"-luminosity @ l_norm (= fbase_norm)
      real f_base(Nb_max,Nl_max) , fbase_norm(Nb_max) 
      real age_base(Nb_max) , Z_base(Nb_max) , alphaFe_base(Nb_max)
      character*15 base_name(Nb_max)

c     Population vector arrays in flux (x) & mass (mu_ini) fractions
      real x(Nb_max) , x_min(Nb_max) , mu_ini(Nb_max)

c     Auxiliar arrays
      real aux1(Nb_max) , aux2(Nb_max) , aux3(Nb_max)

c     Mstars = array which says what fraction of the INITIAL stellar
c     mass in population # j is still in stars at age(j). The name
c     "Mstars" is inspired by the bc2003 nomenclature. Mstars(j) is only
c     used in the flux-to-mass fractions conversions (ie, for output
c     purposes). mu_cor() are the corrected mass-fractions.
      real Mstars(Nb_max) , mu_cor(Nb_max)

c     AV_array: AV_tot(j) = population dependent extinction
      real AV_tot(Nb_max)

c     Arrays to store Original (hence the "O") observed & base spectra
      real Ol_obs(Nl_max)   , Of_obs(Nl_max)
      real Ol_base(Nl_max)  , Of_base(Nb_max,Nl_max)
      real Oerr_obs(Nl_max) , Oflag_obs(Nl_max)

c     Chain-arrays!
      real ChainPar(Npar_max,Nchains_max)
      real Gpar_min(Npar_max) , Gpar_ave(Npar_max)
      real Cpar_min(Npar_max,Nchains_max) , CEne_min(Nchains_max)
      real EX0sChainPar(Npar_max,Nchains_max)
      real EX0sGpar_min(Npar_max) , EX0sGpar_ave(Npar_max)
      real EX0sCpar_min(Npar_max,Nchains_max) ,EX0sCEne_min(Nchains_max)

c     file names
      character*100 arq_obs , arq_base , arq_masks , arq_config
      character*100 arq_out , arq_aux , arq_lix

c     Reddening-law option string 
      character*3 red_law_option

c     Option-String which defines which method will be used to define
c     who is relevant and who is not in the pop vector x_Test!
      character*5 EXOs_method_option
      character*3 EXOs_PopVector_option

!Cid29Mar07-FIX! Defining base, obs, mask & out directories strings!
      character*100 base_dir , obs_dir , mask_dir , out_dir

c     Clipping-Method-opting-string
      character*6 clip_method_option

c     Option-string which defines whether we'll fit or sample the PDF
      character*3 FitOrSamplePDF_option

c     commons...
      common/Red_factor/ red_term(Nl_max)
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/Base/ N_base , f_base
      common/Obase/ Of_base
      common/N_Gauss/ N_int_Gauss
      common/Age_and_Met/ age_base , Z_base
      common/CalcMass/ q_norm , fbase_norm , i_FitPowerLaw , fobs_norm
      common/ScaleChi2_Factor/ fscale_chi2
      common/RC_AV_Stuff/ NRC_AV , i_RC_CRASH_FLAG , NRC_AV_Default
      common/YAV_stuff/ ind_YAV(Nb_max) , N_YAV_components

c     Lower & upper limits on parameters (x,AV,YAV,fn)
      common/ParLimits/ par_low(Npar_max) , par_upp(Npar_max)

c     ScubiduCadeVoce Technical Parameters
      common/Scubi_TechnicalParameters/ alpha , Falpha , IsGRTestHard ,
     $     eff_IDEAL , i_UpdateEps , Feps , i_UpdateAlpha ,
     $     i_MoveOneParOnly , i_HelpParWithMaxR , prob_jRmax ,
     $     Npar4GRtest ,i_UpdateAVYAVStepSeparately ,
     $     ind_HelpParMove2Average(Npar_max) ,
     $     prob_HelpParMove2Average(Npar_max)

c     Opened file units, for output
      common/OutputFileUnits/ i_UnitScubi

c     Willy time functions
      real willy_tiempo(2)

!Cid19Jan07-FIX! Added chi2, AV & norm. factor for best SSP fits!
      real SSP_AV(Nb_max) , SSP_x(Nb_max) , SSP_chi2(Nb_max) ,
     $     SSP_adev(Nb_max)
      common/SSP_chi2_stuff/ x_SSP , f_SSP(Nl_max)
      external F_SSP_chi2_AV

!Cid14Jan07-FXK! Added FXK = Fixed kinematics option!!
      common/FXK_fits/ IsThisAnFXKFit


!v05! Array to store best-fit SSP spectrum
      real Best_f_SSP(Nl_max)
c ***************************************************************************



c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                       INPUT & PRE-PROCESSING STEPS                      +
c +                                                                         +
c +   In this part we reset variables, read input data & perform all        +
c + necessary pre-processing, like re-sampling to a common lambda-scale,    +
c + normalization, definition of weights, etc.                              +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c ***************************************************************************
c *			        RESET STUFF			            * 
c ***************************************************************************
!Cid01Apr07-FIX! Reseting all time counters (just in case)
      i_ini_time        = 0
      w_ini_TotTime     = 0
      w_ini_UsrTime     = 0
      w_ini_SysTime     = 0
      i_fin_time        = 0 
      w_fin_TotTime     = 0
      w_fin_UsrTime     = 0
      w_fin_SysTime     = 0

      i_ini_time_FF     = 0 
      w_ini_FF_TotTime  = 0 
      w_ini_FF_UsrTime  = 0 
      w_ini_FF_SysTime  = 0 
      i_fin_time_FF     = 0 
      w_fin_FF_TotTime  = 0 
      w_fin_FF_UsrTime  = 0 
      w_fin_FF_SysTime  = 0 

      i_ini_time_RF     = 0 
      w_ini_RF_TotTime  = 0 
      w_ini_RF_UsrTime  = 0 
      w_ini_RF_SysTime  = 0 
      i_fin_time_RF     = 0 
      w_fin_RF_TotTime  = 0 
      w_fin_RF_UsrTime  = 0 
      w_fin_RF_SysTime  = 0 

      i_ini_time_BI     = 0 
      w_ini_BI_TotTime  = 0 
      w_ini_BI_UsrTime  = 0 
      w_ini_BI_SysTime  = 0 
      i_fin_time_BI     = 0 
      w_fin_BI_TotTime  = 0 
      w_fin_BI_UsrTime  = 0 
      w_fin_BI_SysTime  = 0 

      i_ini_time_EX0    = 0 
      w_ini_EX0_TotTime = 0 
      w_ini_EX0_UsrTime = 0 
      w_ini_EX0_SysTime = 0 
      i_fin_time_EX0    = 0 
      w_fin_EX0_TotTime = 0 
      w_fin_EX0_UsrTime = 0 
      w_fin_EX0_SysTime = 0 


!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
      i_ini_time    = time()
      w_ini_TotTime = etime(willy_tiempo)
      w_ini_UsrTime = willy_tiempo(1)
      w_ini_SysTime = willy_tiempo(2)

c     Reset All arrays
      do i=1,Nl_max
         f_aux1(i)      = 0.
         f_aux2(i)      = 0.
         f_aux3(i)      = 0.
         l_obs(i)       = 0.
         f_obs(i)       = 0.
         l_syn(i)       = 0.
         f_syn(i)       = 0.
         err_obs(i)     = 0.
         weight_obs(i)  = 0.
         llow_mask(i)   = 0.
         lupp_mask(i)   = 0.
         mask_weight(i) = 0.
         f_mask(i)      = 0.
         Ol_obs(i)      = 0.
         Of_obs(i)      = 0.
         Oerr_obs(i)    = 0.
         Oflag_obs(i)   = 0.
         Ol_base(i)     = 0.
         do j=1,Nb_max
            f_base(j,i)  = 0.
            Of_base(j,i) = 0.
         end do
         Best_f_SSP(i)  = 0.       !v05!
         f_SSP(i)       = 0.       !v05!
      end do

!Cid29Dec06-FIX! Fixed silly i->j bug found by William on Dec/23/2006!
      do j=1,Nb_max
         age_base(j)     = 0.
         Z_base(j)       = 0.
         fbase_norm(j)   = 0.
         x(j)            = 0.
         x_min(j)        = 0.
         AV_tot(j)       = 0.
         mu_ini(j)       = 0.
         mu_cor(j)       = 0.
         Mstars(j)       = 0.
         base_name(j)    = 'none'
         SSP_AV(j)       = 0.
         SSP_x(j)        = 0.
         SSP_chi2(j)     = 0.
         SSP_adev(j)     = 0.
      end do

c     Reset x(), AV & YAV estimates to "-99" flag (Cid@Lagoa - Jan/14/2007)
      do j=1,Nb_max
         x(j)     = -0.99
         x_min(j) = -0.99
      end do
      AV_min  = -99.
      YAV_min = -99.

c     Reset clip-bug flag & CRASH flag for RC-scheme (RC = Rapid Chi2)
      iCLIPBUG_flag   = 0
      i_RC_CRASH_FLAG = 0

c     Reset sigma-clipping & global step counters
      i_clip_loop   = 0
      Ntot_cliped   = 0
      Nglobal_steps = 0

c     Set idum = seed for random-#-generator. idum_orig is preserved for output
      idum = idum_orig

c     Set chi2 scaling factor (used ONLY in Metropolis routine)
      fscale_chi2 = aux_fscale_chi2

c     Reset arrays ind_ & prob_HelpParMove2Average arrays
      do j=1,Npar_max
         ind_HelpParMove2Average(j)  = 0
         prob_HelpParMove2Average(j) = 0.0
      end do

c     Reset Chain-arrays!
      do j=1,Npar_max
         Gpar_min(j)     = 0
         Gpar_ave(j)     = 0
         EX0sGpar_min(j) = 0
         EX0sGpar_ave(j) = 0
         do i_chain=1,Nchains_max
            ChainPar(j,i_chain)     = 0
            Cpar_min(j,i_chain)     = 0
            EX0sCpar_min(j,i_chain) = 0
            CEne_min(i_chain)       = 0
            EX0sCEne_min(i_chain)   = 0
         end do 
      end do 

c     Reset SaveStep-flag = file unit!
      i_UnitScubi = 0

!v05!
      Best_SSP_chi2 = 1.e36        
c ***************************************************************************



c ***************************************************************************
c *	                          INPUT INFO                                * 
c ***************************************************************************
!Cid14Jan07-FXK! Added v0 & vd initial values to input!!
c     Input observed spectrum, base, masks, v0, vd & output filename on 
c     a galaxy by galaxy basis! 
!     Included AV_low & AV_upp in the grid file at Paula's request!
!     Cid@Lagoa - 11/Mar/2008
      read (*,*) arq_obs , arq_config , arq_base , arq_masks ,
     $     red_law_option , v0_ini , vd_ini , AV_low , AV_upp , arq_out
c ***************************************************************************



c ***************************************************************************
c *          READ "TECHNICAL-PARAMETERS" FROM CONFIGURATION FILE            *
c ***************************************************************************
      open (unit=1,file=arq_config,status='old')
      read (1,*)

c     Set normalization wavelength (for base & synthetic spectra) &
c     window limits for normalization of observed spectrum. Make sure
c     that llow_norm <= l_norm <= lupp_norm!
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*) l_norm
      read (1,*) llow_norm
      read (1,*) lupp_norm

      if ((l_norm.LT.llow_norm).OR.(l_norm.GT.lupp_norm)) then
         write (*,*) '[MAIN] l_norm outside llow_norm ==> lupp_norm!'
         write (*,*) llow_norm , l_norm , lupp_norm
         stop
      end if

c     Set a priori limits on extinction (AV), extra-extinction (YAV),
c     normalization factor (fn) and kinematical parameters (v0 & vd).
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
!     Included AV_low & AV_upp in the grid file at Paula's request! So
!     ignore the the config-values.
!     Cid@Lagoa - 11/Mar/2008
      read (1,*) RUBISH_AV_low
      read (1,*) RUBISH_AV_upp
      read (1,*) YAV_low
      read (1,*) YAV_upp
      read (1,*) fn_low
      read (1,*) fn_upp
      read (1,*) v0_low
      read (1,*) v0_upp
      read (1,*) vd_low
      read (1,*) vd_upp

c     Set clip_method_option-string & sigma-clipping threshold
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*) clip_method_option
      read (1,*) sig_clip_threshold
 
c     Set weight control filter ... Points with weights larger than
c     wei_ave + wei_nsig_threshold * wei_sig will be "censored".
c     wei_nsig_threshold <= 0 means that no weight filter will be
c     applied.
      read (1,*) wei_nsig_threshold

!Cid20Jan07-FIX! Set safety margin cushion (in Angs). Previoulsy = 50 Angs.
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*) dl_cushion

c     Set minimum flux (in units of fobs_norm) to be modeled. Points
c     with f_obs < f_cut are masked out!
      read (1,*) f_cut

c     Set # of points in the brute-force convolution in GaussSmooth.
c     (Smarter integrators did not prove worthy!)
      read (1,*) N_int_Gauss

c     Set verbose... (i_verbose = 0 gives silent output)
c     iMassAgeZ_verbose lists estimates of Mass & Z just just for control...
      read (1,*) i_verbose
      read (1,*) i_verbose_anneal
      iMassAgeZ_verbose = i_verbose_anneal

c     Sometimes the 1st line in a spectrum-file (arq_obs) is a
c     header. This flag (new in v02) says if this is so.
      read (1,*) Is1stLineHeader

c     Read fast-BC03-rebining option
      read (1,*) i_FastBC03_FLAG

c     Set if a POWER-LAW F_lambda ~ lambda**alpha_PowerLaw is to be
c     added to the base components from arq_base. Useful for AGN...  If
c     a power-law is added it will be the LAST component!
      read (1,*) i_FitPowerLaw
      read (1,*) alpha_PowerLaw
      if (i_FitPowerLaw.NE.1) alpha_PowerLaw = -9.99
!Cid07Fev07-FIX! Warn if old & ~ obsolte PL option is being used...
      if (i_FitPowerLaw.EQ.1) write (*,'(a,a)')
     $     ' @@> You are using and ~ obsolete buit-in PL component!' ,
     $     ' It is more elegant to include the PL in your base-files!'

!Cid08Fev07-FIX! Set if an existing arq_out file should be skiped (1) 
c     or if the fit should be redone and the file overwriten (0).
c     (Added this at William's request)
      read (1,*) i_SkipExistingOutFiles

!v05! Decide whether or not to save the best fit SSP spectrum on arq_out
      read (1,*) i_SaveBestSingleCompFit


c     Read # of chains & max(x_j) for initial random chain pop-vectors.
      read (1,*) 
      read (1,*) 
      read (1,*) 
      read (1,*) 
      read (1,*) N_chains
      read (1,*) xinit_max

c     Read a bunch of technical parameters...
      read (1,*) i_UpdateEps
      read (1,*) i_UpdateAlpha
      read (1,*) Falpha
      read (1,*) i_MoveOneParOnly
      read (1,*) i_UpdateAVYAVStepSeparately
      read (1,*) i_HelpParWithMaxR
      read (1,*) prob_jRmax
      read (1,*) i_HelpPopVectorMove2Average
      read (1,*) prob_HelpPopVectorMove2Average
      read (1,*) i_HelpAVYAVMove2Average
      read (1,*) prob_HelpAVYAVMove2Average

c     Set NRC_AV_Default = initial # of terms in the RC AV-series
      read (1,*) NRC_AV_Default

c     Read First-Fits (FF) technical parameters...
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*) Temp_ini_FF
      read (1,*) Temp_fin_FF
      read (1,*) fini_eps_FF
      read (1,*) ffin_eps_FF
      read (1,*) R_ini_FF
      read (1,*) R_fin_FF
      read (1,*) IsGRTestHard_FF
      read (1,*) N_loops_FF
      read (1,*) i_RestartChains_FF
      read (1,*) fN_sim_FF
      read (1,*) fNmax_steps_FF
      read (1,*) eff_IDEAL_FF

c     Read Burn-In stage technical parameters...
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*) R_BurnIn
      read (1,*) IsGRTestHard_BurnIn

c     Read EX0s-Fit (EX0s) technical parameters...
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*)
      read (1,*) EXOs_PopVector_option
      read (1,*) EXOs_method_option
      read (1,*) EX0s_Threshold
      read (1,*) Temp_ini_EX0s
      read (1,*) Temp_fin_EX0s
      read (1,*) fini_eps_EX0s
      read (1,*) ffin_eps_EX0s
      read (1,*) R_ini_EX0s
      read (1,*) R_fin_EX0s
      read (1,*) IsGRTestHard_EX0s
      read (1,*) N_loops_EX0s
      read (1,*) i_RestartChains_EX0s
      read (1,*) fN_sim_EX0s
      read (1,*) fNmax_steps_EX0s
      read (1,*) eff_IDEAL_EX0s
!     New stuff in v02!
      read (1,*) IsScaleNstepsInEX0sFits
      read (1,*) IsNoKin4LargeBaseInEX0sFits
      read (1,*) frac_NoKin4LargeBaseInEX0sFits

!Cid18Feb07-FIX! Define maximal base reduction factor for EX0-fits
!     such that at least NEX0s_base >= N_base * fEX0_MinBaseSize
!     (In previous versions this was fixed at 10%)
      read (1,*) fEX0_MinBaseSize
      close(1)

      if (i_verbose.GT.0) then 
         write (*,*)
         write (*,'(a,a)') '**> Welcome to STARLIGHT (v04) & ' ,
     $        'good luck with this run!'
         write (*,*)
         write (*,'(a,a)') '**> Read config file: ' , arq_config
      end if

!     Check some of the input quantities...
      if ((EXOs_method_option.NE.'CUMUL').AND.(EXOs_method_option.NE.
     $     'SMALL')) then
         write (*,'(a,a)') ' [MAIN] OOPS! Bad EXOs_method_option ' ,
     $        EXOs_method_option
         stop
      end if

      if ((i_HelpPopVectorMove2Average.NE.0).AND.
     $     (i_HelpPopVectorMove2Average.NE.1)) then
         write (*,*) '[MAIN] Bad i_HelpPopVectorMove2Average! ' ,
     $        i_HelpPopVectorMove2Average
         stop
      end if
         
      if ((i_HelpAVYAVMove2Average.NE.0).AND.
     $     (i_HelpAVYAVMove2Average.NE.1)) then
         write (*,*) '[MAIN] Bad i_HelpAVYAVMove2Average! ' ,
     $        i_HelpAVYAVMove2Average
         stop
      end if

      if ((EXOs_PopVector_option.NE.'AVE').AND.
     $     (EXOs_PopVector_option.NE.'MIN')) then 
         write (*,*) '[MAIN] Bad EXOs_PopVector_option! ' ,
     $        EXOs_PopVector_option
         stop
      end if

      if (i_MoveOneParOnly.NE.1) then
         write (*,*) '[MAIN] Bad i_MoveOneParOnly: ' , i_MoveOneParOnly
         stop
      end if

      if ((clip_method_option.NE.'NOCLIP').AND.
     $     (clip_method_option.NE.'NSIGMA').AND.
     $     (clip_method_option.NE.'RELRES').AND.
     $     (clip_method_option.NE.'ABSRES')) then
         write (*,*) '[MAIN] Bad clip_method_option ',clip_method_option
         stop
      end if

c     Initialize # of terms in RC-AV series (passed by common)
      NRC_AV = NRC_AV_Default

c     Reset alpha to 1.
      alpha  = 1.0


!Cid08Fev07-FIX! Skip runs already made!  New trick: Figure out if
!     arq_out already exists. If so, abort run & go to next one! Useful
!     to run grids in "modo-burro". If arq_out does NOT exit, then write
!     a tmp arq_out (to be overwritten at the end of the run) just to
!     avoid that another CPU takes this same job while the code is
!     running. 
!     OBS: Flag i_SkipExistingOutFile decides whether this test is done.
      if (i_SkipExistingOutFiles.EQ.1) then

!Cid29Mar07-FIX! Now write arq_out to out_dir, so figure what is the first
!     blank char in obs_dir & concatenate it: arq_out --> arq_aux
        i_last_non_blank_char = 0
        do i=1,99
           if ((out_dir(i:i+1).EQ.' ').AND.(i_last_non_blank_char.EQ.0))
     &          i_last_non_blank_char = i - 1
        end do
        arq_aux = out_dir(1:i_last_non_blank_char) // arq_out

        open (unit=2,file=arq_aux,status='old',err=500)
        close(2)
        write (*,'(a,a)')' OOPS! arq_out already exists!! Skipping ' ,
     $       arq_out
        return
 500    OkWeCanGoOnWithTheFit = 0.
        open (unit=2,file=arq_aux,status='unknown')
        write (2,'(a)')
     $       'This file will be overwritten in a few minutes...'
        close(2)
      end if
c ***************************************************************************



c ***************************************************************************
c *	                  READ OBSERVED SPECTRUM		            * 
c ***************************************************************************
c     OBS: The "O" arrays (where "O" stands for "Original") defined here
c     are renamed without the "O" later on, after pre-processing steps.

!Cid29Mar07-FIX! Now read arq_obs from obs_dir, so figure what is the first
!     blank char in obs_dir & concatenate it: arq_obs --> arq_aux
      i_last_non_blank_char = 0
      do i=1,99
         if ((obs_dir(i:i+1).EQ.' ').AND.(i_last_non_blank_char.EQ.0))
     &        i_last_non_blank_char = i - 1
      end do
      arq_aux = obs_dir(1:i_last_non_blank_char) // arq_obs

!Cid24Mar07-FIX! Try to figure how many columns are in arq_obs to check
!     consistency of IsErrSpecAvailable & IsFlagSpecAvailable info given...
!     If a problem is detected issue a warning, but do NOT stop!!
!     OBS: Does NOT work if columns are separated by tabs or commas!
      open (unit=1,file=arq_aux,status='old')
      read (1,*)
      read (1,'(a)') arq_lix
      close(1)
      icol = 0
      do i=1,99
         if ((arq_lix(i:i).EQ.' ').AND.(arq_lix(i+1:i+1).NE.' ')) 
     $        icol = icol + 1
      end do
      if (arq_lix(1:1).EQ.' ') icol = icol - 1

      if ((IsErrSpecAvailable.EQ.1).AND.(IsFlagSpecAvailable.EQ.1)
     $     .AND.(icol.NE.3)) then
 189  format(67('?'))
         write (*,189)
         write (*,'(a,a,a)') 'ATT: You said there are error & flag ' ,
     $        'columns in ' , arq_obs
         write (*,'(a,i3,a)') '     but I counted only ', icol + 1 ,
     $        ' columns!!'
         write (*,'(a)')      '     Hope you know what you are doing...'
         write (*,189)
      else if ((IsErrSpecAvailable.EQ.1).AND.(IsFlagSpecAvailable.EQ.0)
     $     .AND.(icol.LT.2)) then
         write (*,189)
         write (*,'(a,a,a)') 'ATT: You said there is an error ' ,
     $        'column in ' , arq_obs
         write (*,'(a,i3,a)') '     but I counted ', icol + 1 ,
     $        ' columns!!'
         write (*,'(a)')      '     Hope you know what you are doing...'
         write (*,189)
      end if


c     Read original/observed galaxy flux, error & flag spectra:
c     Of_obs(Ol_obs) , Oerr_obs(Ol_obs) & Oflag_obs(Ol_obs)
c     ATT: Implemented no error/flag-available possibilities...
c     Cid@IAA - 29/May/2005
      open (unit=1,file=arq_aux,status='old')

c     OBS: Skip 1st line (header) if Is1stLineHeade flag is on.
      if (Is1stLineHeader.NE.0) read (1,*)

      do i=1,Nl_max
      if ((IsErrSpecAvailable.EQ.1).AND.(IsFlagSpecAvailable.EQ.1)) then

         read (1,*,END=20) Ol_obs(i) , Of_obs(i) , Oerr_obs(i) ,
     $        Oflag_obs(i)

      else if ((IsErrSpecAvailable.EQ.1).AND.(IsFlagSpecAvailable.EQ.0))
     $        then

         read (1,*,END=20) Ol_obs(i) , Of_obs(i) , Oerr_obs(i)
         Oflag_obs(i) = 0

      else if ((IsErrSpecAvailable.EQ.0).AND.(IsFlagSpecAvailable.EQ.0))
     $        then

         read (1,*,END=20) Ol_obs(i) , Of_obs(i)
         Oflag_obs(i) = 0

      end if
      end do

      write (*,*) ' [MAIN] BUG reading input spectrum! N > Nl_max!! ' , 
     &     Nl_max
      stop
 20   NOl_obs = i - 1
      close(1)


!Cid07Feb07-FIX! Screen output 
      if (i_verbose.GT.0) then
         write (*,'(a,a30)') '**> Read data file: ',arq_obs
         write (*,'(a,a)') '    from dir = ' , 
     &        base_dir(1:i_last_non_blank_char)
         write (*,'(4x,a,2(2x,f6.1,a),i5,a)') 'Data goes from: ' , 
     $        Ol_obs(1) , '  --> ' , Ol_obs(NOl_obs) , 
     $        ' Angs with N = ' , NOl_obs , ' pixels'
         if (IsErrSpecAvailable.EQ.1) write (*,'(4x,a,a)')
     $        'STARLIGHT has read an ERROR (column 3) from ' , arq_obs
         if (IsFlagSpecAvailable.EQ.1) write (*,'(4x,a,a)')
     $        'STARLIGHT has read an FLAG  (column 4) from ' , arq_obs
      end if

!Cid14Jan07-FIX! Check if Ol_obs is given in increasing order.
      do i=1,NOl_obs-1
         if (Ol_obs(i+1).LE.Ol_obs(i)) then
            write (*,*) ' [MAIN] PROBLEM w/spectrum: ',
     $           'lambda should be given in increasing order! : ' , i ,
     $           Ol_obs(i) , Ol_obs(i+1)
            stop
         end if
      end do


c     In the absence of a proper error spectrum, define Oerr_obs() using
c     the old method of rms in a window... (f_aux1 is just a
c     tmp-fake-weight-array to fool Stats4SN...). 
      if (IsErrSpecAvailable.EQ.0) then
         do i=1,NOl_obs
            f_aux1(i) = 100.
            if (Of_obs(i).LE.0.) f_aux1(i) = -100.
         end do

         call Stats4SN(NOl_obs,Ol_obs,Of_obs,f_aux1,llow_SN,lupp_SN
     $        ,aux_ave,aux_sig)

c     ATT: For safety, if the rms in the llow_SN -> lupp_SN is 0
c     (god-knows-why...) then recompute it using ALL the spectrum!
         if (aux_sig.LE.0.) then
            llow_SN = Ol_obs(1)
            lupp_SN = Ol_obs(NOl_obs)
            call Stats4SN(NOl_obs,Ol_obs,Of_obs,f_aux1,llow_SN,lupp_SN
     $           ,aux_ave,aux_sig)
         end if

         do i=1,NOl_obs
            Oerr_obs(i) = aux_sig
         end do

         if (i_verbose.GT.0) then
            write (*,'(a,a,f9.3)')
     $           '**> ATT: Defined error & flag spectra the old way!',
     $           ' S/N =' , aux_ave / aux_sig
         end if
      end if
c ***************************************************************************



c ***************************************************************************
c *	              READ BASE MASTER-FILE & BASE SPECTRA                  * 
c ***************************************************************************
c     Read base spectra: Of_base(i_base,Ol_base) (later renamed to
c     f_base). Notice that I'm now also reading Mstars & ind_YAV &
c     alphaFe_base from the base-file!!
c     ATT: All base spectra are read from base_dir, so we first find the
c     last non empty bit in string base_dir (to concatenate names below)
      i_last_non_blank_char = 0
      do i=1,99
         if ((base_dir(i:i+1).EQ.' ').AND.(i_last_non_blank_char.EQ.0))
     &        i_last_non_blank_char = i - 1
      end do

      open (unit=3,file=arq_base,status='old')
      read (3,*) N_base

      if (N_base.GT.Nb_max) then
         write (*,*) ' [MAIN] OOPS! N_base > Nb_max: ' , N_base , Nb_max
         write (*,*) ' [MAIN] Run aborted!'
         stop
      end if

      do j=1,N_base

c     Read base-info: filename, age, Z, code-name, Mstars, YAV & alphaFe_base
         read (3,*) arq_aux , age_base(j) , Z_base(j) , base_name(j) , 
     &        Mstars(j) , ind_YAV(j) , alphaFe_base(j)

c     Concatenate base spectrum filename with base directory & read it!
         arq_aux = base_dir(1:i_last_non_blank_char) // arq_aux
         call ReadSpecWithHeader(arq_aux,Nl_max,NOl_base,Ol_base,f_aux1)

!Cid29Mar07-FIX! Introduced tests to check if i_FastBC03_FLAG = 1 is 
!     being used with the original BC03/STELIB models. If not, I issue a
!     warning and revert i_FastBC03_FLAG to 0.
         if (i_FastBC03_FLAG.EQ.1) then
            i_shit = 0
            if (NOl_base.NE.6900) i_shit = 1
            if (int(Ol_base(1)).NE.91) i_shit = 1
            if (int(Ol_base(1000)).NE.3974) i_shit = 1
            if (int(Ol_base(2000)).NE.4974) i_shit = 1
            if (int(Ol_base(3000)).NE.5974) i_shit = 1
            if (int(Ol_base(4738)).NE.7712) i_shit = 1
            if (int(Ol_base(6361)).NE.9990) i_shit = 1
            if (int(Ol_base(6900)).NE.1600000) i_shit = 1
            if (i_shit.EQ.1) then
 188           format(80('?'))
               write (*,188)
               write (*,'(a,a)') 'OOPS! You have specified ' , 
     $              'i_FastBC03_FLAG = 1 (in arq_config) but the file: '
               write (*,'(a)') arq_aux
               write (*,'(a,a)') 'does NOT match the lambdas of ',
     $              'BC03/STELIB!! Reverting to i_FastBC03_FLAG = 0!'
               i_FastBC03_FLAG = 0
               write (*,188)
            end if
         end if

         do i=1,NOl_base
            Of_base(j,i) = f_aux1(i)
         end do

!Cid14Jan07-FIX! Check if Ol_base is given in increasing order.
         do i=1,NOl_base-1
            if (Ol_base(i+1).LE.Ol_base(i)) then
               write (*,*) ' [MAIN] PROBLEM w/base spectrum: ',
     $              'lambda should be given in increasing order! : ' , i 
     $              , Ol_base(i) , Ol_base(i+1)
               write (*,*) ' [MAIN] base-file = ', arq_aux
               stop
            end if
         end do
      end do
      close(3)

!Cid14Jan07-FIX!  Warn if Mstars(j) <= 0, but only fixes it if ALL are <= 0.
      IsAtLeastOneMstarsOk = 0
      do j=1,N_base
         if ((Mstars(j).GT.0.).AND.(IsAtLeastOneMstarsOk.EQ.0))
     $        IsAtLeastOneMstarsOk = 1
         if (Mstars(j).LE.0.) then
            write (*,*) ' [MAIN] ATT: Mstars <= 0!! ',
     $           'Fix it or disregard mass fractions on output...' , j
         end if
      end do
      if (IsAtLeastOneMstarsOk.EQ.0) then
         write (*,*) ' [MAIN] ATT: ALL Mstars <= 0!! WILL SET IT TO 1!'
         do j=1,N_base
            Mstars(j) = 1.
         end do
      end if

!     Screen output 
      if (i_verbose.GT.0) then
         write (*,'(a,a45,a,i3)') '**> Read base files: ' , arq_base , 
     &        ' N_base  = ' , N_base
         write (*,'(a,a)') '    from dir = ' , 
     &        base_dir(1:i_last_non_blank_char)
      end if
c ***************************************************************************



c ***************************************************************************
c *	                      READ WEIGHT/MASK DATA		            * 
c ***************************************************************************
!Cid29Mar07-FIX! Now read arq_masks from mask_dir, so figure what is the first
!     blank char in obs_dir & concatenate it: arq_masks --> arq_aux
      i_last_non_blank_char = 0
      do i=1,99
         if ((mask_dir(i:i+1).EQ.' ').AND.(i_last_non_blank_char.EQ.0))
     &        i_last_non_blank_char = i - 1
      end do
      arq_aux = mask_dir(1:i_last_non_blank_char) // arq_masks


c     Read MASK intervals: The array mask_weight() is used like this:
c     * mask_weight <= 0 on unwanted windows (eg emission lines)
c     * mask_weight > 0  is the factor by which we multiply the weight of the
c                        points within llow_mask ==> lupp_mask. This may be
c                        used to give more weight to key absorption features.
c     OBS: If any of the intervals overlap, then the one further down
c     the list will take precedence over the other, so be careful when
c     defining the intervals in arq_masks!
      open (unit=1,file=arq_aux,status='old')
      read (1,*) N_masks
      do i=1,N_masks
         read (1,*) llow_mask(i) , lupp_mask(i) , mask_weight(i)
      end do
      close(1)


!     Screen output (Added FXK info - Cid@Lagoa - 14/Jan/2007)
      if (i_verbose.GT.0) then
         write (*,'(a,a45,a,i3)') '**> Read masks file: ' , arq_masks , 
     &        ' N_masks = ' , N_masks
         write (*,'(a,a)') '    from dir = ' , 
     &        mask_dir(1:i_last_non_blank_char)
         write (*,'(a,a30,a,a)') '**> Modeling ',arq_obs,' ==> ',arq_out
         write (*,'(4x,3(a,f6.1),a,a,1p,e9.3)') 'Sampling: ',Olsyn_ini, 
     &        ' ==> ' , Olsyn_fin , '  with dl =' , Odl_syn , ' Angs' ,
     &        '  & fscale_chi2 = ' , fscale_chi2
         write (*,'(a,f6.1)') '    dl_cushion (A) = ' , dl_cushion
         write (*,'(a,a)')    '    Extinction Law = ' , red_law_option
         write (*,'(4x,a,f6.1,a,f6.1,a))')
     $        'Initial kinematical parameters: v0 = ' , v0_ini ,
     $        ' & vd = ', vd_ini , ' km/s'

         if (FitOrSamplePDF_option.EQ.'PDF') then 
            write (*,'(a)')
     $           '    We are going to SAMPLE THE PDF for this galaxy'
         else if (FitOrSamplePDF_option.EQ.'FIT') then 
            write (*,'(a)')
     $           '    We are going to FIT the spectrum of this galaxy'
         else if (FitOrSamplePDF_option.EQ.'FXK') then 
            write (*,'(a,a)')
     $           '    We are going to FIT the spectrum of this galaxy' ,
     $           ' with FIXED KINEMATICAL PARAMETERS!'
         end if
      end if
c ***************************************************************************



c ***************************************************************************
c * PRE-PROCESSING I: RESAMPLING OBSERVED, BASE & WEIGHT/FLAG/MASK SPECTRA  *
c ***************************************************************************
c     Here I introduce the following trick: The wanted range is really
c     from Olsyn_ini ==> Olsyn_fin. I extend this range by ~ dl_cushion
c     Angstroms in both blue & red edges, but force the mask f_mask to
c     be = zero outside the Olsyn_ini ==> Olsyn_fin interval. Therefore
c     these extra points will NOT be synthesized. However, these
c     "safety" cushion will help minimizing edge effects in the
c     Gauss_smooth routine. In fact, these extra points are useful only
c     in the convolution of the synthetic spectrum with the kinematical
c     filter (in Gauss_smooth).

!Cid20Jan07-FIX! dl_cushion used to be fixed at 50 Angs. Now it is
!     read from arq_config!
c     Extend wavelength range by dl_cushion Angs to blue & red. 
      n_extra  = dl_cushion / Odl_syn
      lsyn_ini = Olsyn_ini - n_extra * Odl_syn
      lsyn_fin = Olsyn_fin + n_extra * Odl_syn
      dl_syn   = Odl_syn

c     Define lambda grid for spectral synthesis, l_syn (later renamed l_obs!)
      Nl_syn = (lsyn_fin - lsyn_ini) / dl_syn + 1

      if (Nl_syn.GT.Nl_max) then
         write (*,*) ' [MAIN] OOPS! Nl_syn > Nl_max: ' , Nl_syn , Nl_max
         stop
      end if

      do i=1,Nl_syn
         l_syn(i) = lsyn_ini + (i - 1) * dl_syn
      end do

c     Rebin observed spectra. I STRONGLY suggest you rebin your spectra
c     externally (eg, with IRAF's dispcor) and use Odl_syn = original
c     dl. The RebinSpec routine would then keep the spectrum untouched!
      call RebinSpec(0,NOl_obs,Ol_obs,Of_obs,Nl_syn,l_syn,f_aux1)

c     Rebin error & flag spectra too. 
c     **> ATT: THE REBINNING OF THE ERROR SPECTRUM SHOULD BE RE-THOUGHT!
c     (Binning adjacent pixels should reduce the noise, but my rebinning
c     routine simply "averages" it...). This is why I do NOT recommend
c     rebinning at all!
c     Rebinning the flag also changes the integer-flags to real
c     numbers... This, however, is not a problem: We simply flag-away
c     points with flag > 1.
      call RebinSpec(0,NOl_obs,Ol_obs,Oerr_obs,Nl_syn,l_syn,f_aux2)
      call RebinSpec(0,NOl_obs,Ol_obs,Oflag_obs,Nl_syn,l_syn,f_aux3)


c     OBS: This next step REDEFINES NOl_obs, Ol_obs(), Of_obs(),
c     Oerr_obs() & Oflag_obs()!!
      NOl_obs = Nl_syn
      do i=1,NOl_obs
!         write (*,'(i4,2x,4(1x,f8.3),10x,4(1x,f8.3))') i , 
!     &        Ol_obs(i) , Of_obs(i) , Oerr_obs(i) , Oflag_obs(i) ,
!     &        l_syn(i)  , f_aux1(i) , f_aux2(i)   , f_aux3(i)
         Ol_obs(i)    = l_syn(i)
         Of_obs(i)    = f_aux1(i)
         Oerr_obs(i)  = f_aux2(i)
         Oflag_obs(i) = f_aux3(i)

c     Mask/flag out points outside desired Olsyn_ini ==> Olsyn_fin range.
c     Also mask/flag out points with error <= 0! (Just a safety measure...)
         if ((Ol_obs(i).LT.Olsyn_ini).OR.
     &       (Ol_obs(i).GT.Olsyn_fin).OR.
     &       (Oerr_obs(i).LE.0.)) Oflag_obs(i) = 99.
      end do


c     Resample/Rebin base spectra to l_obs wavelength-scale.  I've
c     introduced a i_FastBC03_FLAG which greatly speeds up rebinning for
c     a BC03 base and dl = 1 and Ol_obs is within 3322-->9300 Angs.
c     i_FastBC03_FLAG is read from config file.
c     OBS: This step REDEFINES the Of_base() matrix!!
!
!     ATT: I've adapted the fast-rebin trick to work with Base-Rosa
!     spectra (br04*spec files), which cover the 3001 --> 6996 Angs
!     range. To use use fast-rebin with BR04 you must set
!     i_FastBC03_FLAG = 2!  This trick should be generalized in a more
!     elegant manner... (Cid@IAA - 30/May/2005)
!XXX! THIS SHOULD BE IMPROVED!
      do j=1,N_base
         do i=1,NOl_base
            f_aux1(i) = Of_base(j,i)
         end do

         call RebinSpec(i_FastBC03_FLAG,NOl_base,Ol_base,f_aux1,
     &        NOl_obs,Ol_obs,f_aux2)

         do i=1,NOl_obs
            Of_base(j,i) = f_aux2(i)
         end do
      end do


!Cid20Jan07-FIX! WARN if we will need to mask base edges to be safe
!     against edge effects in the kinematical filter! 
!     NOTE: I initially worried that masking my favorite norm-range
!     (4010-4060 A) in Pegase-HR models (which start at 4000 Ans) would
!     bring problems, but apparently not!
      if ((Ol_base(1).GT.lsyn_ini).OR.(Ol_base(NOl_base).LT.lsyn_fin))
     $     then
         write (*,'(a,a)') '@@@@@@@@@@@@@@@@@@@@@@@@@@@@> BASE ' , 
     $        'LIMITS WARNING <@@@@@@@@@@@@@@@@@@@@@@@@@@@@@'
         write (*,'(2(a,f9.1),a)') '@ Your base goes from lambda = ',
     $        Ol_base(1) ,' to ' , Ol_base(NOl_base) , ' Angs '
         write (*,'(2(a,f9.1),a)') '@ but the fit will run from    ' ,
     $        lsyn_ini , ' to ' , lsyn_fin ,
     $        ' Angs '
         write (*,'(a,f5.1,a)') '@ (including a +/-', dl_cushion ,
     $        ' Angs cushion for the kinematical filter)'
         write (*,'(2(a,f9.1),a)')
     $        '@ STARLIGHT will flag-out lambdas outside the ' ,
     $        Ol_base(1) + dl_cushion , ' to ' , 
     $        Ol_base(NOl_base) - dl_cushion , ' Angs range!'
         write (*,'(a,a)') '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@'
     $        ,'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@'
      end if


c     Define a MASK-SPECTRUM f_mask() in the Ol_obs wavelength-scale.
c     The mask_weight array should be = 0 on the N_mask llow_mask ==>
c     lupp_mask windows which we do NOT want to model. If mask_weight >
c     0, this sets different/special weight for the corresponding
c     llow_mask ==> lupp_mask interval.  We also force the mask to be
c     zero outside the Olsyn_ini ==> Olsyn_fin interval.  Finally, we
c     mask-out points for which the (SDSS-inspired) quality flag is > 1.
c     These points are marked with f_mask = -2.
      do i=1,NOl_obs

         f_mask(i) = 1.

         do j=1,N_masks
            if ((Ol_obs(i).GE.llow_mask(j)).AND.
     &           (Ol_obs(i).LE.lupp_mask(j))) f_mask(i) = mask_weight(j)
         end do

         if ((Ol_obs(i).LT.Olsyn_ini).OR.
     &        (Ol_obs(i).GT.Olsyn_fin)) f_mask(i) = 0.

!Cid20Jan07-FIX! Introduced safety against people trying to fit data
!     within dl_cushion Angs of the base edges. The if below will
!     mask the first and last dl_cushon Angs from the base edges.
!     OBS: The possible/likely overlap with the if above is harmless!
         if ((Ol_obs(i).LT.(Ol_base(1)+dl_cushion)).OR.
     $        (Ol_obs(i).GT.(Ol_base(NOl_base)-dl_cushion))) then
            f_mask(i) = 0.
         end if

c     Accept points with flag <= 1! This means that we INCLUDE things
c     SDSS people call "emission lines" (flag = 1), but we've verified
c     these are not always real emission lines!
c     Cid&Jean@UFSC - 20/Nov/2003
         if (Oflag_obs(i).GT.1.) f_mask(i) = -2.
      end do


c     OBS: After the steps above all arrays (Of_obs, Of_base & f_mask) 
c          are sampled at Ol_obs. (Ol_base is not used anymore).
c ***************************************************************************



c ***************************************************************************
c *     ADD A POWER-LAW COMPONENT TO BASE-MATRIX & SET N_YAV_components     *
c ***************************************************************************
c     If a PowerLaw (of fixed slope: F_lambda ~ lambda**alpha_PowerLaw)
c     component is wanted, then add it as component number N_base + 1 in
c     the base matrix Of_base & update N_base to N_base + 1. Note that
c     the PL has no "M/L" ratio! This is taken into consideration when I
c     compute mass-fractions.
c     Cid@lynx - 4/September/2003
      if (i_FitPowerLaw.GE.1) then
         N_base = N_base + 1
         do i=1,NOl_obs
            Of_base(N_base,i) = (Ol_obs(i) / l_norm)**alpha_PowerLaw
         end do
         base_name(N_base) = 'PowerLaw'
         age_base(N_base)  = 0.
         Z_base(N_base)    = 0.

c     Now that we allow for extra-extinction (YAV), we must decide
c     whether this extra-extinction applies to the power-law too. The
c     convention is that ind_YAV = YES for the PL-components only if
c     i_FitPowerLaw = 2.
c     Cid@Lagoa - 21/Jan/2005
         ind_YAV(N_base) = 0
         if (i_FitPowerLaw.EQ.2) ind_YAV(N_base) = 1

         if (i_verbose.GT.0) write (*,'(a,f7.4,a,i2)')
     $        '    Added Power-Law with index ' , alpha_PowerLaw ,
     $        ' to base, and with YAV = ' , ind_YAV(N_base)
      end if

c     Compute Number of components which will be allowed to have the
c     extra-extinction YAV (passed elsewhere by the common/YAV_stuff/).
      N_YAV_components = 0
      do j=1,N_base
        if (ind_YAV(j).NE.0) N_YAV_components = N_YAV_components + 1
      end do

      if (i_verbose.GT.0) write (*,'(a,i3,a,f5.2,a,f5.2,a)') 
     &     '    Number of components with different extinction = ' , 
     &     N_YAV_components, '  (',YAV_low,' < YAV < ',YAV_upp,')'
c ***************************************************************************



c ***************************************************************************
c *                     PRE-PROCESSING II: NORMALIZATION                    *
c ***************************************************************************
c     Renormalize the observed spectrum by the MEDIAN flux between
c     llow_norm & lupp_norm, but the base spectra are normalized just by
c     the flux at l_norm! Normalizing the base at a single wavelength is
c     better (eg, it preserves normalization when applying the reddening
c     term to synthetic spectra) and should be Ok since the base has "no
c     noise".  The observed spectrum, however, has noise, so it is
c     better to normalize it by the mean or median flux in a window.  

c     Normalize Of_obs to MEDIAN flux between llow_norm & lupp_norm
c     Re-Normalize Oerr_obs too.  ATT: the median is computed regardless
c     of whether points are flagged or masked!!
!     ATT: NEW-Safety: If fobs_norm <= 0 then produce a silly output
!     file & abort the run!! The user should fix this before running.
      call Stats_F(NOl_obs,Ol_obs,Of_obs,llow_norm,lupp_norm,
     &     Of_ave,Of_sig,Of_med)
      fobs_norm = Of_med

      if (fobs_norm.LE.0.) then
         write (*,'(a,1p,e12.3)') '## ATT: fobs_norm = ' , fobs_norm
         write (*,'(a,a)') '## arq_obs = ' , arq_obs
         write (*,'(a)') '## Run aborted & silly file produced :('

!v05! Store bugged-file in out_dir
         i_last_non_blank_char = 0
         do i=1,99
           if ((out_dir(i:i+1).EQ.' ').AND.(i_last_non_blank_char.EQ.0))
     &           i_last_non_blank_char = i - 1
        end do
        arq_aux = out_dir(1:i_last_non_blank_char) // arq_out
        open (unit=2,file=arq_aux,status='unknown')
        write (2,'(a,1p,e12.3)') '## ATT: fobs_norm = ' , fobs_norm
        write (2,'(a,a)') '## arq_obs = ' , arq_obs
        write (2,'(a)') '## Run aborted:('
        close(2)
        return
      end if

      do i=1,NOl_obs
         Of_obs(i)   = Of_obs(i) / fobs_norm
         Oerr_obs(i) = Oerr_obs(i) / fobs_norm
      end do


c     Normalize base spectra to their flux at l_norm.
c     OBS1: fbase_norm(j) is saved to compute M/L's & mass-fractions...
c     OBS2: fbase_norm(N_base) is meaningless if a power-law was added!
      do j=1,N_base
         do i=1,NOl_obs
            f_aux1(i) = Of_base(j,i)
         end do

         call calc_Fnorm(NOl_obs,Ol_obs,f_aux1,l_norm,fbase_norm(j))

         do i=1,NOl_obs
            Of_base(j,i) = f_aux1(i) / fbase_norm(j)
         end do
      end do
c ***************************************************************************



c ***************************************************************************
c *      PRE-PROCESSING III: DEFINITION OF ERRORS/WEIGHTS & S/N ratios      *
c ***************************************************************************
c     Define errors & weights! Weight = 0 if f_mask <= 0 OR f_obs <=
c     f_cut. These points will thus be excluded from the synthesis.  For
c     other points, use INPUT error flux!  Points with mask_weight > 0
c     are given extra weight (= f_mask).
c     Nl_eff = the number of non-zero-weight points, ie, the effective
c              number of lambdas synthesized.
      Nl_eff = 0
      do i=1,NOl_obs
         if ((f_mask(i).LE.0.).OR.(Of_obs(i).LE.f_cut).OR.
     &        (Oerr_obs(i).LE.0.)) then
            err_obs(i)    = -99.
            weight_obs(i) = 0.
         else
            err_obs(i)    = Oerr_obs(i) / f_mask(i)
            weight_obs(i) = 1. / err_obs(i)
            Nl_eff        = Nl_eff + 1
         end if
      end do


c     Store Nl_eff onto NOl_eff ("Original Nl_eff"), since Nl_eff will
c     be modified if clipping is turned on.
      NOl_eff = Nl_eff


c     Apply a "WEIGHT-CONTROL-FILTER"! Compute mean and std dev of
c     weight_obs for the NOl_eff unmasked lambdas, and reset all weights
c     2-sigma above mean to weigh_obs = mean + 2 sigma! I've decided to
c     apply this "filter" as some of the SDSS spectra have very-low
c     errors on some lambdas, and these are then given a
c     disproportionately large weight in the synthesis.
c     Cid@lynx - 17/March/2004
c
c     I later realized that it is desirable to have more control over
c     this option. For instance, when err_obs() = constant (as in Joao's
c     data) but we want (through the mask file) to give more weight to a
c     particular window (say, Ca II K), we'd have wei_sig(i) = 0, which
c     would result in weight_obs(i) = wei_ave all accross the
c     spectrum... Hence, I introduced the parameter wei_nsig_threshold
c     in arq_config. Setting wei_nsig_threshold <= 0 will turn OFF the
c     weight filter. Otherwise, we censor points whose weights are >=
c     wei_nsig_threshold times larger than wei_sig.
c     Cid@Lagoa - 24/March/2005

!Cid24Mar07-FIX! Added counter of censored weights just for output purposes.
      n_censored_weights = 0
      wei_limit          = 0
      if (wei_nsig_threshold.GT.0.) then
         N_aux = 0
         do i=1,NOl_obs
            if (weight_obs(i).GT.0.) then 
               N_aux         = N_aux + 1
               f_aux1(N_aux) = weight_obs(i)
            end if
         end do

         call avevar(f_aux1,N_aux,wei_ave,wei_var)
         wei_sig   = sqrt(wei_var)
         wei_limit = wei_ave + wei_nsig_threshold * wei_sig

         do i=1,NOl_obs
            if (weight_obs(i).GT.wei_limit) then
               weight_obs(i)      = wei_limit
               n_censored_weights = n_censored_weights + 1
            end if
         end do
      end if


c     Compute the S/N ratio in the SN & normalization windows from the
c     the average-flux/rms-flux ratios (excluding masked points). Then
c     re-estimate these quantities using the average-error instead of
c     the rm-flux in the denominator ("SNerr_*"). The S/N in the
c     normalization window is just 1 over the noise flux in the
c     SN-window (a sensible, albeit silly, definition).  
c     OBS: ALL these S/N's are IRRELEVANT since we use an input error
c     spectrum! They are ONLY used for output purposes!
      call Stats4SN(NOl_obs,Ol_obs,Of_obs,weight_obs,
     &     llow_SN,lupp_SN,ave1,sig1)

c     ATT: For safety, if the rms in the llow_SN -> lupp_SN is 0
c     (god-knows-why...) then recompute it using ALL the spectrum! This
c     is just to avoid stupid NaN's in the output file! 
c     Cid@Lagoa - 08/Oct/2005
      if (sig1.LE.0.) then
         llow_SN = Ol_obs(1)
         lupp_SN = Ol_obs(NOl_obs)
         call Stats4SN(NOl_obs,Ol_obs,Of_obs,weight_obs,llow_SN,lupp_SN
     $        ,ave1,sig1)
      end if

      call Stats4SN(NOl_obs,Ol_obs,Oerr_obs,weight_obs,
     &     llow_SN,lupp_SN,ave2,sig2)
      SN_snwin      = ave1 / sig1
      SN_normwin    = 1.   / sig1
      SNerr_snwin   = ave1 / ave2
      SNerr_normwin = 1.   / ave2

      if (i_verbose.GT.0) then
         write (*,'(a,f8.3,2x,f8.3,a)')'    S/N     = ', 
     &     SN_snwin , SN_normwin , '     in S/N & norm. windows'
         write (*,'(a,f8.3,2x,f8.3,a)')'    S/N_err = ', 
     &     SNerr_snwin , SNerr_normwin , '     in S/N & norm. windows'
      end if
c ***************************************************************************



c ***************************************************************************
c *        PRE-PROCESSING IV: SET EXTINCTION LAW & FINAL DETAILS            *
c ***************************************************************************
c     Define reddening term = -0.4 * (A(lambda)/AV - A(l_norm)/AV).
c     The array red_term is passed by the common/Red_factor/ to routines
c     which need it!
c     OBS1: The user should apply his own correction for Galactic
c           extinction to the input observed spectrum before-hand...
      R_V = 3.1
      call Calc_red_term(red_law_option,l_norm,R_V,
     &     NOl_obs,Ol_obs,red_term,q_norm)


c     Store O* variables onto proper names, passed by commons to other
c     routines.
      Nl_obs = NOl_obs
      do i=1,Nl_obs
         l_obs(i) = Ol_obs(i)
         f_obs(i) = Of_obs(i)
         do j=1,N_base
            f_base(j,i) = Of_base(j,i)
         end do
      end do
c ***************************************************************************
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                DONE WITH INPUT & PRE-PROCESSING STEPS:)                 +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                              SPECTRAL FIT                               +
c +                                                                         +
c +                                                                         +
c +   The pre-processing steps above produced the following main            +
c +   variables: l_obs() , f_obs() , weight_obs(), Of_base(,),              +
c +   f_base(,), age_base(), Z_base(), red_term(), N_base , Nl_obs,         +
c +   ind_YAV(), N_YAV_components. These are used in the spectral fit,      +
c +   which produces estimates of the following output variables: x(),      +
c +   AV_tot() [= AV + YAV], v0 & vd.                                       +
c +                                                                         +
c +   In this part we do the actual spectral fit. The steps are:            +
c +   (1) Initialize chains & set-up several details
c +   (2) First-Fits (FF), to make chains converge to a reasonable zone of  +
c +       parameter space. N_loops_FF annealing loops are done.
c +   (3) Identify & clip outliers.
c +   (4) Re-fit clipped spectrum. Only 1 loop.
c +   (5) Re-fit once more, this time with a more rigorous Burn-In 
c +       criterion. The chain & global-best parameters after this step 
c +       are stored for output...
c +   (6) Do a "Fine-tuning"-annealing-fit with a condense base.
c +   
c +   OBS: The code below also allows for sampling of the PDF, but this part
c +        part is not really ready ... (Jean should have used it...)
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+


c ***************************************************************************
c * Initialize Kinematical Parameters v0 & vd (Cid/FXK@Lagoa - 14/Jan/2007) *
c ***************************************************************************
c     Set initial values for kinematical parameters v0 & vd (in km/s).
c     These are now read from the input for each galaxy!!
c     (Life would be much easier if we knew these beforehand...)
      v0_min = v0_ini
      vd_min = vd_ini

c     IsKinFixed = 0 allows v0 & vd to be fitted. IsKinFixed = 1 keep
c     them fixed. IsThisAnFXKFit is an auxiliar flag (associated to
c     common/FXK_fits/ which indicates that this is an FXK fit, so v0 &
c     vd will NEVER be changed wrt the input values, so the base
c     smoothing can be done only once! (In this case, I smooth both
c     f_base & Of_base).
      IsThisAnFXKFit = 0
      IsKinFixed     = 0

      if (FitOrSamplePDF_option.EQ.'FXK') then
         IsKinFixed     = 1
         IsThisAnFXKFit = 1

         if (i_verbose.GT.0) write (*,'(a,2(a,f9.3),a)')
     $        '[MAIN] FXK-fit: Smoothing base to' ,' v0 = ' , v0_min ,
     $        '  & vd = ' , vd_min , ' km/s... once and for all!'

         do i=1,Nl_obs
            l_aux(i)  = l_obs(i)
         end do
         Nl_aux = Nl_obs
         do j=1,N_base
            do i=1,Nl_obs
               f_aux1(i) = f_base(j,i)
            end do
            call GaussSmooth(Nl_aux,l_aux,f_aux1,v0_min,vd_min,
     &           N_int_Gauss,Nl_obs,l_obs,f_aux2)
            do i=1,Nl_obs
               f_base(j,i)  = f_aux2(i)
               Of_base(j,i) = f_aux2(i)
            end do
         end do
         if (i_verbose.GT.0) write (*,'(a)') '[MAIN] FXK-fit: ... done!'

      end if

c     Sanity check: Warn if initial v0 & vd are not within low->upp bounds!
      if ((v0_min.LT.v0_low).OR.(v0_min.GT.v0_upp).OR.
     $     (vd_min.LT.vd_low).OR.(vd_min.GT.vd_upp)) then
         write (*,*) ' [MAIN] Your initial v0/vd values exceed the' ,
     $        '_low -> _upp  values in the config file!! '
         write (*,*) '        I hope you know what you are doing!'
      end if
c ***************************************************************************



c ***************************************************************************
c *                            Initialize Chains                            *
c ***************************************************************************
c     Define par_low & par_upp limits, where par =(x,AV,YAV,fn), in this
c     order!  Also define j_AV, j_YAV & j_fn indices used throughout!
      N_par = N_base + 3
      j_AV  = N_base + 1
      j_YAV = N_base + 2
      j_fn  = N_base + 3

      do j=1,N_base
         par_low(j) = 0.
         par_upp(j) = fn_upp
      end do
      par_low(j_AV)  = AV_low
      par_upp(j_AV)  = AV_upp 
      par_low(j_YAV) = YAV_low
      par_upp(j_YAV) = YAV_upp
      if (N_YAV_components.EQ.0) then
         par_low(j_YAV) = 0.0
         par_upp(j_YAV) = 0.0
      end if
      par_low(j_fn) = fn_low
      par_upp(j_fn) = fn_upp


c     Initialize N_chains with RANDOM initial parameters: (x,AV,YAV,fn).
c     Store them onto ChainPar-matrix.  Note that YAV is defined even if
c     not used! Note also that Generate_PopVector returns a pop-vector
c     normalized to unit sum, so we renormalize it to Sum{x_j} = fn a
c     posteriori.

!     ATT: To decrease initial chain energies, I'm using a more
!     constrained range for fn, AV & YAV so that chains do not start
!     from completely crazy positions allowed in the low ==> upp ranges!
!     Also, I force the 1st chain to start from x_j = 1/N_base, AV = YAV = 0.

!     ATT! Whereas in StarlightChains_v01.for the z*_low & *_upp values
!     for fn, AV & YAV were fixed, I now set them equal to the
!     corresponding input *_low & *_upp values and restrict to narrower
!     ranges them only if they exceed certain limits (0.9 -> 1.1 for fn;
!     0 -> 1 for AV & YAV). I did this because when Jean tryed to impose
!     -0.01 <= AV < 0.01 in the config file, the previsous scheme
!     allowed for larger initial chain AV's, which sometimes stayed
!     outside the AV_low --> AV_upp range all the way to the final
!     solution. Cid@Lagoa - 08/Oct/2005
      zfn_low = fn_low
      zfn_upp = fn_upp
      if (fn_low.LT.0.9) zfn_low = 0.9
      if (fn_upp.GT.1.1) zfn_upp = 1.1

      zAV_low = AV_low
      zAV_upp = AV_upp
      if (AV_low.LT.0.0) zAV_low = 0.0
      if (AV_upp.GT.1.0) zAV_upp = 1.0

      zYAV_low = YAV_low
      zYAV_upp = YAV_upp
      if (YAV_low.LT.0.0) zYAV_low = 0.0
      if (YAV_upp.GT.1.0) zYAV_upp = 1.0
      if (N_YAV_components.EQ.0) then
         zYAV_low = 0.0
         zYAV_upp = 0.0
      end if

      ChainPar(j_fn,1) = 1.0
      do j=1,N_base
         ChainPar(j,1) = 1.0 / float(N_base)
      end do
      ChainPar(j_AV,1)  = 0.0
      ChainPar(j_YAV,1) = 0.0

      do i_chain=2,N_chains
         ChainPar(j_fn,i_chain) = zfn_low + (zfn_upp - zfn_low)
     $        * ran2(idum)

         call Generate_PopVector(idum,N_base,xinit_max,x)
         do j=1,N_base
            ChainPar(j,i_chain) = x(j) * ChainPar(j_fn,i_chain)
         end do

         ChainPar(j_AV,i_chain)  = zAV_low + (zAV_upp - zAV_low)
     $        * ran2(idum)
         ChainPar(j_YAV,i_chain) = zYAV_low + (zYAV_upp - zYAV_low)
     $        * ran2(idum)
!         write (*,'(a,i2,a,200(1x,f6.2))') 'Ch-Init '
!     $        ,i_chain , ' | par=', (100*ChainPar(j,i_chain),j
!     $        =1,N_base) , (ChainPar(jj,i_chain),jj=j_AV,j_fn) 
      end do
c ***************************************************************************



c ***************************************************************************
c *                  Initialize help-arrays & other stuff                   *
c ***************************************************************************
c     Define arrays which say which component will be given an extra
c     help to converge and with what probability. Things are done
c     separately for the pop-vector & extinction parameters. These
c     arrays are passed by the /Scubi_TechnicalParameters/ common.
      do j=1,N_par
         ind_HelpParMove2Average(j)  = i_HelpPopVectorMove2Average
         prob_HelpParMove2Average(j) = prob_HelpPopVectorMove2Average
         if ((j.EQ.j_AV).OR.(j.EQ.j_YAV)) then
            ind_HelpParMove2Average(j)  = i_HelpAVYAVMove2Average
            prob_HelpParMove2Average(j) = prob_HelpAVYAVMove2Average
         end if
      end do
      if (N_YAV_components.EQ.0) then
         ind_HelpParMove2Average(j_YAV)  = 0
         prob_HelpParMove2Average(j_YAV) = 0.
      end if


c     Define Npar4GRtest = # of parameters used in the GR-convergence
c     (burn-in) test. 
      Npar4GRtest = N_par - 1 
      if (N_YAV_components.EQ.0) Npar4GRtest = N_par - 2


c     Set probability of picking the par_j with largest GR_R (used if
c     sampling is one-par-per-move), to help it move more & thus
c     hopefully converge faster... To turn-on this trick you must set
c     i_HelpParWithMaxR = 1. The value of prob_jRmax requires some
c     experimentation...  Note that prob_jRmax is actually set on the
c     input file. Here I just adjust it in the case of small N_par...
      prob_jRmax = max((5.0 / float(Npar4GRtest)),prob_jRmax)
c ***************************************************************************



c ***************************************************************************
c *                    FF: First Fits (w/full base)                         *
c ***************************************************************************
c     Here we perform the "First Fits" (FF). The goal is to take all
c     chains to a reasonable region of parameter space, using simulated
c     annealing plus a Gelman & Rubin burn-in verification criterion.

c     Set eff_IDEAL & IsGRTestHard (passed by common), N_sim &
c     Nmax_steps for First-Fits.  
      eff_IDEAL     = eff_IDEAL_FF
      IsGRTestHard  = IsGRTestHard_FF
      N_sim_FF      = fN_sim_FF * N_par
      Nmax_steps_FF = fNmax_steps_FF * N_par

!     Screen output
      if (i_verbose.GT.0) then
         write (*,'(80a)') ('*',j=1,80)
         write (*,'(a)') '* STARTING FIRST FITS ...'
         write (*,'(a,i3)') '* N_loops  = ' , N_loops_FF
         write (*,'(a,i3)') '* N_chains = ' , N_chains
         write (*,'(a,1p,e8.2,a,e8.2)') '* Temp     = ' , Temp_ini_FF ,
     $        ' ==> ' , Temp_fin_FF
         write (*,'(a,1p,e8.2,a,e8.2,0p,a,i2,a)') '* GR-R     = ' ,
     $        R_ini_FF ,' ==> ' , R_fin_FF , '  Method = ' ,
     $        IsGRTestHard , '  (0/1=Soft/Hard)'
         write (*,'(80a)') ('*',j=1,80)
      end if

!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
      i_ini_time_FF    = time()
      w_ini_FF_TotTime = etime(willy_tiempo)
      w_ini_FF_UsrTime = willy_tiempo(1)
      w_ini_FF_SysTime = willy_tiempo(2)


c     Go for First Fits...
 123  call Fit_Scubidu(i_verbose_anneal,idum,N_loops_FF,v0_low,v0_upp
     $     ,vd_low,vd_upp,x_min,AV_min,YAV_min,v0_min,vd_min,Ntot_FF
     $     ,eff_FF,Nl_eff,iMassAgeZ_verbose,N_par,ChainPar,N_chains
     $     ,Temp_ini_FF,Temp_fin_FF,fini_eps_FF,ffin_eps_FF,R_ini_FF
     $     ,R_fin_FF,N_sim_FF,Nmax_steps_FF,i_RestartChains_FF,Gpar_min
     $     ,GE_min,Gpar_ave,Cpar_min,CEne_min,IsKinFixed,IsBurInOver_FF)


!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
      i_fin_time_FF    = time()
      w_fin_FF_TotTime = etime(willy_tiempo)
      w_fin_FF_UsrTime = willy_tiempo(1)
      w_fin_FF_SysTime = willy_tiempo(2)

c     Update global steps counter
      Nglobal_steps = Nglobal_steps + Ntot_FF
c ***************************************************************************



c ***************************************************************************
c *                        Clip-outliers & Re-Fit                           *
c ***************************************************************************
c     Clip deviant points in the spectrum! Use best model to compute
c     residuals...
      call Update_AV_array(N_base,AV_min,YAV_min,AV_tot)
      call Kcalc_Fsyn(Nl_obs,x_min,AV_tot,v0_min,vd_min,f_syn)
      call Clip_Spectrum(clip_method_option,f_syn,sig_clip_threshold
     $     ,f_mask,NOl_eff,Nl_eff,Ntot_cliped,iCLIPBUG_flag)


c     If clipping actually cliped something, then run a single re-fit
c     with the SAME conditions as in the last loop above. Otherwise
c     no-refitting is done!
      if (Ntot_cliped.LE.0) then

         if (i_verbose.GT.0) then
            write (*,'(a,i5,a,f3.1,a,a,a)') '* CLIP>  Cliped ' ,
     $           Ntot_cliped,' points with > ' , sig_clip_threshold
     $           ,' sigma residuals with method = ' , clip_method_option
     $           , '.  No re-fitting needed!'
         end if

      else

         Temp_ini     = Temp_fin_FF
         Temp_fin     = Temp_fin_FF
         fini_eps     = ffin_eps_FF
         ffin_eps     = ffin_eps_FF
         R_ini        = R_fin_FF
         R_fin        = R_fin_FF
         N_sim        = N_sim_FF
         Nmax_steps   = Nmax_steps_FF
         N_loops      = 1
         IsGRTestHard = IsGRTestHard_FF

!Cid22Jan07-FIX! i_RestartChains was UNDEFINED here! (The
!     implication is that chains where not restarted to their best
!     values before going to the next stage).
         i_RestartChains = i_RestartChains_FF

!     Screen output
         if (i_verbose.GT.0) then
            write (*,'(80a)') ('*',j=1,80)
            write (*,'(a)') '* STARTING RE-FIT AFTER CLIPING ...'
            write (*,'(a,i5,a,f3.1,a,a)') '* Cliped ' , Ntot_cliped
     $           ,' points with > ' , sig_clip_threshold
     $           ,' sigma residuals with method = ' , clip_method_option
            write (*,'(a,i3)') '* N_loops = ' , N_loops
            write (*,'(a,1p,e8.2,a,e8.2)') '* Temp    = ' , Temp_ini ,
     $           ' ==> ' , Temp_fin
            write (*,'(a,1p,e8.2,a,e8.2,0p,a,i2,a)') '* GR-R    = ' ,
     $           R_ini ,' ==> ' , R_fin , '  Method = ' , IsGRTestHard ,
     $           '  (0/1=Soft/Hard)'
            write (*,'(80a)') ('*',j=1,80)
         end if

!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
         i_ini_time_RF    = time()
         w_ini_RF_TotTime = etime(willy_tiempo)
         w_ini_RF_UsrTime = willy_tiempo(1)
         w_ini_RF_SysTime = willy_tiempo(2)


c     Go for re-fit
         call Fit_Scubidu(i_verbose_anneal,idum,N_loops,v0_low,v0_upp
     $        ,vd_low,vd_upp,x_min,AV_min,YAV_min,v0_min,vd_min
     $        ,Ntot_ClipReFit,eff_ClipReFit,Nl_eff,iMassAgeZ_verbose
     $        ,N_par,ChainPar,N_chains,Temp_ini,Temp_fin,fini_eps
     $        ,ffin_eps,R_ini,R_fin,N_sim,Nmax_steps,i_RestartChains
     $        ,Gpar_min,GE_min,Gpar_ave,Cpar_min,CEne_min,IsKinFixed
     $        ,IsBurInOver_ClipReFit)


!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
         i_fin_time_RF    = time()
         w_fin_RF_TotTime = etime(willy_tiempo)
         w_fin_RF_UsrTime = willy_tiempo(1)
         w_fin_RF_SysTime = willy_tiempo(2)

c     Update global steps counter
         Nglobal_steps = Nglobal_steps + Ntot_ClipReFit

      end if
c ***************************************************************************



c ***************************************************************************
c *                            Burn-In Re-fit                               *
c ***************************************************************************
c     Re-fit (clipped) spectrum using the T = 1 & IsGRTestHard = 1
c     (meaning that every individual parameter must satisfy the GR-test)
c     & GR_R-limit <= 1.2. This is to ensure we're over the BURN-IN
c     phase before proceeding to further sampling or minimization. 
c     (OBS: Actually, the Hard/Soft GR-test and the R-Threshold used here
c     are define by the config values of IsGRTestHard_BurnIn &
c     R_BurnIn!)
c     If T = 1 & IsGRTestHard = 1 in the last loop above, then this
c     refit is unnecessary.
      if ((Temp_fin.LE.1.).AND.(IsGRTestHard.EQ.1).AND.(R_fin.LE.
     $     R_BurnIn)) then
         
         IsBurInOver_BurnIn = 1
         if (i_verbose.GT.0) then
            write (*,'(a,a)') '* No T = 1 & Hard-GR-test refit needed.'
     $           , ' Burn-in is already over!'
         end if

      else

!Cid11Feb07-FIX! Previsouly (StarlightChains_v02) I did the Burn-In 
!     at T = 1, seting Temp_ini = Temp_fin = 1.0. Now I do it at T =
!     Temp_fin_FF to be more general. All my fits with
!     StarlightChains_v02 used Temp_fin_FF = 1, so results should be
!     identical.
         Temp_ini     = Temp_fin_FF
         Temp_fin     = Temp_fin_FF
         fini_eps     = ffin_eps_FF
         ffin_eps     = ffin_eps_FF
         R_ini        = R_BurnIn
         R_fin        = R_BurnIn
         N_sim        = N_sim_FF
         Nmax_steps   = Nmax_steps_FF
         N_loops      = 1
         IsGRTestHard = IsGRTestHard_BurnIn

!Cid22Jan07-FIX! i_RestartChains was UNDEFINED here! (The
!     implication is that chains where not restarted to their best
!     values before going to the next stage).
         i_RestartChains = i_RestartChains_FF

!     Screen output
         if (i_verbose.GT.0) then
            write (*,'(80a)') ('*',j=1,80)
            write (*,'(a)') '* STARTING FINAL BURN-IN LOOP...'
            write (*,'(a,i3)') '* N_loops = ' , N_loops
            write (*,'(a,1p,e8.2,a,e8.2)') '* Temp    = ' , Temp_ini ,
     $           ' ==> ' , Temp_fin
            write (*,'(a,1p,e8.2,a,e8.2,0p,a,i2,a)') '* GR-R    = ' ,
     $           R_ini ,' ==> ' , R_fin , '  Method = ' , IsGRTestHard ,
     $           '  (0/1=Soft/Hard)'
            write (*,'(80a)') ('*',j=1,80)
         end if

!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
         i_ini_time_BI    = time()
         w_ini_BI_TotTime = etime(willy_tiempo)
         w_ini_BI_UsrTime = willy_tiempo(1)
         w_ini_BI_SysTime = willy_tiempo(2)


c     Go for it 
         call Fit_Scubidu(i_verbose_anneal,idum,N_loops,v0_low,v0_upp
     $        ,vd_low,vd_upp,x_min,AV_min,YAV_min,v0_min,vd_min
     $        ,Ntot_BurnIn,eff_BurnIn,Nl_eff,iMassAgeZ_verbose,N_par
     $        ,ChainPar,N_chains,Temp_ini,Temp_fin,fini_eps,ffin_eps
     $        ,R_ini,R_fin,N_sim,Nmax_steps,i_RestartChains,Gpar_min
     $        ,GE_min,Gpar_ave,Cpar_min,CEne_min,IsKinFixed
     $        ,IsBurInOver_BurnIn)


!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
         i_fin_time_BI    = time()
         w_fin_BI_TotTime = etime(willy_tiempo)
         w_fin_BI_UsrTime = willy_tiempo(1)
         w_fin_BI_SysTime = willy_tiempo(2)

c     Update global steps counter
         Nglobal_steps = Nglobal_steps + Ntot_BurnIn

      end if
c ***************************************************************************



c ***************************************************************************
c *                   Sample PDF option (4 Jean but never used...)          *
c ***************************************************************************
c     If FitOrSamplePDF_option = 'PDF', we'll sample the Probability
c     Distribution Function (PDF), otherwise proceed to optimization
c     with a shortened base. Instead of sampling the PDF of the actual
c     synthesis parameters we store in arq_out the values of some
c     "condensed-parameters" (such as mean age, mean Z, etc) for every
c     step of every chain.

      if (FitOrSamplePDF_option.EQ.'PDF') then 

c     To sample the PDF it is first necessary to fix all technical
c     sampling knobs/tricks, such that the proposal distribution does
c     NOT vary while sampling. In other words, stop helping paameters
c     move towards convergence, stop adjusting step-size to achieve
c     desired efficiency, etc. (All MCMC stuff I;ve read says that
c     changing/adapting the proposal distribution "on-the-fly" is no
c     good while sampling the PDF.
         do j=1,N_par
            ind_HelpParMove2Average(j)  = 0.
            prob_HelpParMove2Average(j) = 0.
         end do

         i_HelpParWithMaxR           = 0
         prob_jRmax                  = 0.
         i_UpdateAVYAVStepSeparately = 0
         i_UpdateEps                 = 0
         i_UpdateAlpha               = 0
         Falpha                      = 1.0           
         i_MoveOneParOnly            = 1
         i_HelpAVYAVMove2Average     = 0
         prob_HelpAVYAVMove2Average  = 0
         i_RestartChains             = 0

c     Now fix, kinematics, T = 1 & step-sizes. 
         IsKinFixed   = 1
         Temp_ini     = 1.0
         Temp_fin     = 1.0
         fini_eps     = ffin_eps_FF
         ffin_eps     = ffin_eps_FF
         N_loops      = 1

c     To determine the number of samples, we first run Fit_Scubidu until
c     a Freeze-In condition is fulfulled, and figure out the number of
c     steps from the length of the Freeze-In period. Here we define the
c     GR-R criterion for the Freeze-in loop.
         R_ini        = R_BurnIn
         R_fin        = R_BurnIn
         IsGRTestHard = 0
         N_sim        = N_sim_FF
         Nmax_steps   = 1.e6       

!     Screen output
         if (i_verbose.GT.0) then
            write (*,'(80a)') ('*',j=1,80)
            write (*,'(a)') '* STARTING FREEZE-IN LOOP...'
            write (*,'(a,1p,e9.3)') '* alpha      = ' , alpha
            write (*,'(a,i9)')      '* N_sim      = ' , N_sim
            write (*,'(a,i9)')      '* Nmax_steps = ' , Nmax_steps
            write (*,'(a,1p,e8.2,a,e8.2,0p,a,i2,a)') '* GR-R       = ' ,
     $           R_ini ,'==> ' , R_fin , '  Method = ' , IsGRTestHard ,
     $           '  (0/1=Soft/Hard)'
            write (*,'(80a)') ('*',j=1,80)
         end if


c     Initial sampling to define Freeze-in length: N_steps_FreezeIn
         call Fit_Scubidu(i_verbose_anneal,idum,N_loops,v0_low,v0_upp
     $        ,vd_low,vd_upp,x_min,AV_min,YAV_min,v0_min,vd_min
     $        ,Ntot_FreezeIn,eff_FreezeIn,Nl_eff,iMassAgeZ_verbose
     $        ,N_par,ChainPar,N_chains,Temp_ini,Temp_fin,fini_eps
     $        ,ffin_eps,R_ini,R_fin,N_sim,Nmax_steps,i_RestartChains
     $        ,Gpar_min,GE_min,Gpar_ave,Cpar_min,CEne_min,IsKinFixed
     $        ,IsBurInOver_FreezeIn)

         N_steps_FreezeIn = float(Ntot_FreezeIn) / float(N_chains)

c     Now set a ridiculously small value for GR-R, such that we control
c     the number of steps by Nmax_steps, which is defined as twice
c     the freeze in length (or 1e6, whichever is smaller).
         R_ini        = 0.1
         R_fin        = 0.1
         Nmax_steps   = min((2. * N_steps_FreezeIn),1.e6)
                                !JEAN: Limit/Define limit file-sizes here!
                                !JEAN: You may try 2, 3...10 times the 
                                !      Freeze in length!

!     Screen output
         if (i_verbose.GT.0) then
            write (*,'(80a)') ('*',j=1,80)
            write (*,'(a)') '* STARTING PDF SAMPLING...'
            write (*,'(a,i9)') '* N_FreezeIn = ' , N_steps_FreezeIn
            write (*,'(a,i9)') '* Nmax_steps = ' , Nmax_steps
            write (*,'(a,a)')  '* Steps saved in ' , arq_out
            write (*,'(80a)') ('*',j=1,80)
         end if


c     Open output steps-file (communicated though i_UnitScubi to actual
c     sampling routine) & Go for it: sample PDF!
         i_UnitScubi = 2
         open (unit=i_UnitScubi,file=arq_out,status='unknown')

         write (2,'(79(a1))') ('#',j=1,79)
         write (2,'(a,a)') '##  OUTPUT OF StarlightChains_v04.for',
     &        ' - SAMPLE-PDF option! -  [Cid@IAA 29/May/2005]  ##'
         write (2,'(79(a1))') ('#',j=1,79)
         write (2,'(a)') '#'
         write (2,'(a,a)')  '# arq_obs       ' , arq_obs
         write (2,'(a,a)')  '# arq_base      ' , arq_base
         write (2,'(a,a)')  '# arq_config    ' , arq_config
         write (2,'(a,a)')  '# red_law       ' , red_law_option
         write (2,'(a,i8)') '# N_base      ' , N_base
         write (2,'(a,i8)') '# N_chains    ' , N_chains
         write (2,'(a,i8)') '# N_sim       ' , N_sim
         write (2,'(a,i8)') '# N_FreezeIn  ' , N_steps_FreezeIn
         write (2,'(a,i8)') '# Nmax_steps  ' , Nmax_steps
         write (2,'(a)') '#'
         write (2,'(a,a)') '# i_chain at_flux am_flux',
     $        ' at_mass am_mass AV log(Mass) & log(Chi2)'


         call Fit_Scubidu(i_verbose_anneal,idum,N_loops,v0_low,v0_upp
     $        ,vd_low,vd_upp,x_min,AV_min,YAV_min,v0_min,vd_min
     $        ,Ntot_SamplePDF,eff_SamplePDF,Nl_eff,iMassAgeZ_verbose
     $        ,N_par,ChainPar,N_chains,Temp_ini,Temp_fin,fini_eps
     $        ,ffin_eps,R_ini,R_fin,N_sim,Nmax_steps,i_RestartChains
     $        ,Gpar_min,GE_min,Gpar_ave,Cpar_min,CEne_min,IsKinFixed
     $        ,IsBurInOver_SamplePDF)

         close (i_UnitScubi)
         i_UnitScubi = 0

c     We'll not optimize the solution, since the goal here was to Sample
c     the PDF). Hence we're done!
         return

      end if
c ***************************************************************************



c ***************************************************************************
c *                         Condensed-Base Fits                             *
c ***************************************************************************
c     If FitOrSamplePDF_option is not = 'PDF' we're thrown here, where
c     we'll try to find an "optimal" solution by (1) condensing the base
c     to "relevant" elements and (2) annealing down to low T to find a
c     best-chi2 solution.

!Cid24Mar07-FIX!  In previous versions, EX0s would run even with
!     N_loops_EX0s = 0! This is fixed now. But then I need to reset the 
!     things below, just in case N_loops_EX0s = 0.
      NEX0s_base = N_base
      do j=1,N_par
         EX0sGpar_min(j) = Gpar_min(j)
      end do
      EX0sv0_min        = v0_min
      EX0svd_min        = vd_min
      v0_min_BeforeEX0s = v0_min
      vd_min_BeforeEX0s = vd_min
      idt_EX0           = 0
      wdt_EX0_TotTime   = 0
      wdt_EX0_UsrTime   = 0
      wdt_EX0_SysTime   = 0


      if (N_loops_EX0s.GE.1) then

c     Store initial Chain-positions in EX0sChainPar (since I may want to
c     save the last ChainPar from the Burn-In loop above)
         do i_chain=1,N_chains
            do j=1,N_par
               EX0sChainPar(j,i_chain) = ChainPar(j,i_chain)
            end do
         end do

c     Define "x_Test". This is the pop-vector which will be used to
c     decide who stays in the condensed base and who is eliminated! If
c     EXOs_PopVector_option = 'AVE' we'll use x_Test = global average
c     pop-vector, which is a CONSERVATIVE strategy, since it has less
c     x_j = 0 components than any best-solution pop vector. For
c     EXOs_PopVector_option = 'MIN' we'll use x_Test = global best x so
c     far, which may be a better thing to do if we wanna refine this
c     minimum (This option should also produce shorter bases).
         if (EXOs_PopVector_option.EQ.'AVE') then 
            do j=1,N_base
               x(j) = Gpar_ave(j)
            end do
         else if (EXOs_PopVector_option.EQ.'MIN') then 
            do j=1,N_base
               x(j) = Gpar_min(j)
            end do
         end if

c     Set eff_IDEAL & IsGRTestHard (passed by common), N_sim &
c     Nmax_steps for EX0s-Fits
         eff_IDEAL       = eff_IDEAL_EX0s
         IsGRTestHard    = IsGRTestHard_EX0s
         N_sim_EX0s      = fN_sim_EX0s * N_par
         Nmax_steps_EX0s = fNmax_steps_EX0s * N_par

!     Screen output
         if (i_verbose.GT.0) then
            write (*,'(80a)') ('*',j=1,80)
            write (*,'(a)') '* STARTING EX0s FITS ...'
            write (*,'(a,a)') 
     $           '* EX0_method     =   ', EXOs_method_option
            write (*,'(a,f7.3)')
     $           '* EX0s_Threshold = ' , EX0s_Threshold 
            write (*,'(a,a)')
     $           '* EXOs_PopVector =     ' , EXOs_PopVector_option
            write (*,'(a,i3)') '* N_loops = ' , N_loops_EX0s
            write (*,'(a,1p,e8.2,a,e8.2)') '* Temp    = ',Temp_ini_EX0s,
     $           ' ==> ' , Temp_fin_EX0s
            write (*,'(a,1p,e8.2,a,e8.2,0p,a,i2,a)')  '* GR-R    = ' ,
     $           R_ini_EX0s ,' ==> ' , R_fin_EX0s , '  Method = ' ,
     $           IsGRTestHard , '  (0/1=Soft/Hard)'
            write (*,'(80a)') ('*',j=1,80)
         end if

!Cid18Feb07-FIX! Define minimum acceptable base-size!
         N_base_min = nint(fEX0_MinBaseSize * N_base)
         if (N_base_min.LT.3) N_base_min = 3

!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
         i_ini_time_EX0    = time()
         w_ini_EX0_TotTime = etime(willy_tiempo)
         w_ini_EX0_UsrTime = willy_tiempo(1)
         w_ini_EX0_SysTime = willy_tiempo(2)


c     Go for it!
         call EX0s_Fit_Scubi(i_verbose_anneal,idum,N_loops_EX0s,v0_low
     $     ,v0_upp,vd_low,vd_upp,x_min,AV_min,YAV_min,EX0sv0_min
     $     ,EX0svd_min,Ntot_EX0s,eff_NEX0s,Nl_eff,iMassAgeZ_verbose
     $     ,N_par,EX0sChainPar,N_chains,Temp_ini_EX0s,Temp_fin_EX0s
     $     ,fini_eps_EX0s,ffin_eps_EX0s,R_ini_EX0s,R_fin_EX0s
     $     ,N_sim_EX0s,Nmax_steps_EX0s,i_RestartChains_EX0s,EX0sGpar_min
     $     ,EX0sGE_min,EX0sGpar_ave,EX0sCpar_min,EX0sCEne_min,x
     $     ,EXOs_method_option,EX0s_Threshold,N_base_min,IsKinFixed
     $     ,IsBurInOver_EX0s,NEX0s_base,IsScaleNstepsInEX0sFits
     $     ,IsNoKin4LargeBaseInEX0sFits,frac_NoKin4LargeBaseInEX0sFits)


!Cid24Mar07-FIX!  Store EX0s_v0 & vd on v0_min & vd_min.
         v0_min = EX0sv0_min
         vd_min = EX0svd_min

!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
         i_fin_time_EX0    = time()
         w_fin_EX0_TotTime = etime(willy_tiempo)
         w_fin_EX0_UsrTime = willy_tiempo(1)
         w_fin_EX0_SysTime = willy_tiempo(2)

c     Update global steps counter
         Nglobal_steps = Nglobal_steps + Ntot_EX0s

      end if
c ***************************************************************************

c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                       DONE WITH SPECTRAL FITTING:)                      +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                     FITS WITH JUST ONE COMPONENT                        +
c +                                                                         +
c +    ~Quick fits with individual f_base spectra to see which individual   +
c +    base component would best fit the observed spectrum!                 +
c +    (These fits do NOT interfere with the composite fits done above!)    +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c ***************************************************************************
c *             Single-Component-fits - Cid@Lagoa - 20/Jan/2007             *
c ***************************************************************************
c     Just for the "fun" of it, before closing STARLIGHT will now fit
c     f_obs with each of the N_base individual SSPs in the base. With
c     just one component, there are only 2 parameters: AV and the
c     normalizatior factor. The best fit values are saved as SSP_AV()
c     and x_SSP() arrays, along with the corresponding SSP_chi2() and
c     SSP_adev() arrays. Minimization in SSP_AV is done with a Golden
c     Search, using FUNCTION F_SSP_chi2_AV(AV_SSP), and SSP_x is fitted
c     analytically inside that function.

c     The kinematical parameters used are the best ones at this stage,
c     and not changed from SSP to SSP fit. This exercise may prove
c     useful for, say, elliptical galaxies, to find the best
c     single-population fit. (Indeed, this option was inspired during
c     Ignacio de la Rosa's visit to Florianopolis on Jan/2007.)

c     Also useful to fit star-clusters, in which case it's a good idea
c     to chose an FXK fit and config parameters which minimize the time
c     spent on the fit-pop-mixtures part of the code (like N_loops_FF =
c     1 at high temperature, R-limits, no clipping , etc...), such that
c     the code gets to this point ASAP.

c     Cid@Lagoa - 20/Jan/2007

      do j=1,N_base

c     Apply kinematical filter to the j'th SSP, unless this is a FXK fit
c     (in which case f_base is already smoothed & shifted). The
c     resulting spectrum is stored in f_SSP().
         if (FitOrSamplePDF_option.EQ.'FXK') then
            do i=1,Nl_obs
               f_SSP(i) = f_base(j,i)
            end do
         else
            Nl_aux = Nl_obs
            do i=1,Nl_obs
               l_aux(i)  = l_obs(i)
               f_aux1(i) = f_base(j,i)
            end do
            call GaussSmooth(Nl_aux,l_aux,f_aux1,v0_min,vd_min,
     &           N_int_Gauss,Nl_obs,l_obs,f_SSP)
         end if

c     Golden Search for best SSP_AV (best norm.  factor x_SSP computed
c     analytically in F_SSP_chi2_AV and returned via common). Notice
c     tolerance = 1/1000, more than enough for this purpose!
         ax  = AV_low
         bx  = AV_upp
         tol = 1.e-3
         call mnbrak(ax,bx,cx,fa,fb,fc,F_SSP_chi2_AV)
!v05! ATT! "cfx" BUG-DETECTED BY CID&ROSA@Lagoa - 10/May/2008!!
!was this:  chi2_AV_Gold = golden(ax,bx,cxf,F_SSP_chi2_AV,tol,AV_Gold)
         chi2_AV_Gold = golden(ax,bx,cx,F_SSP_chi2_AV,tol,AV_Gold)

c     Do not allow SSP_AV outside the AV_low --> AV_upp limits.
         if (AV_Gold.LT.AV_low) AV_Gold = AV_low
         if (AV_Gold.GT.AV_upp) AV_Gold = AV_upp
         SSP_chi2(j) = F_SSP_chi2_AV(AV_Gold)
         SSP_x(j)    = x_SSP
         SSP_AV(j)   = AV_Gold

         do i=1,Nl_obs
            f_SSP(i) = 10.**(AV_Gold * red_term(i)) * x_SSP * f_SSP(i)
         end do
         SSP_adev(j) = F_adev(f_SSP)

!v05! Storing best-fit SSP spectrum
         if (j.EQ.1) Best_SSP_chi2 = SSP_chi2(j)
         if (SSP_chi2(j).LE.Best_SSP_chi2) then
            Best_SSP_chi2  = SSP_chi2(j)
            index_Best_SSP = j
            do i=1,Nl_obs
               Best_f_SSP(i) = f_SSP(i)
            end do
         end if

      end do
c ***************************************************************************
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+




c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                        POST-PROCESSING & OUTPUT                         +
c +                                                                         +
c +   Now that we have the final model, do a few final post-processing      +
c + (like converting flux to mass fractions) & produce output file arq_out. +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c ***************************************************************************
c *                              OUTPUT                                     *
c ***************************************************************************
!Cid29Dec06-FIX! Introduced time-counters (to help in config-tests)
      i_fin_time      = time()
      idt_all         = i_fin_time     - i_ini_time
      idt_FF          = i_fin_time_FF  - i_ini_time_FF
      idt_RF          = i_fin_time_RF  - i_ini_time_RF
      idt_BI          = i_fin_time_BI  - i_ini_time_BI
      idt_EX0         = i_fin_time_EX0 - i_ini_time_EX0
      w_fin_TotTime   = etime(willy_tiempo)
      w_fin_UsrTime   = willy_tiempo(1)
      w_fin_SysTime   = willy_tiempo(2)
      wdt_TotTime     = w_fin_TotTime     - w_ini_TotTime
      wdt_UsrTime     = w_fin_UsrTime     - w_ini_UsrTime
      wdt_SysTime     = w_fin_SysTime     - w_ini_SysTime
      wdt_FF_TotTime  = w_fin_FF_TotTime  - w_ini_FF_TotTime
      wdt_FF_UsrTime  = w_fin_FF_UsrTime  - w_ini_FF_UsrTime
      wdt_FF_SysTime  = w_fin_FF_SysTime  - w_ini_FF_SysTime
      wdt_RF_TotTime  = w_fin_RF_TotTime  - w_ini_RF_TotTime
      wdt_RF_UsrTime  = w_fin_RF_UsrTime  - w_ini_RF_UsrTime
      wdt_RF_SysTime  = w_fin_RF_SysTime  - w_ini_RF_SysTime
      wdt_BI_TotTime  = w_fin_BI_TotTime  - w_ini_BI_TotTime
      wdt_BI_UsrTime  = w_fin_BI_UsrTime  - w_ini_BI_UsrTime
      wdt_BI_SysTime  = w_fin_BI_SysTime  - w_ini_BI_SysTime
      wdt_EX0_TotTime = w_fin_EX0_TotTime - w_ini_EX0_TotTime
      wdt_EX0_UsrTime = w_fin_EX0_UsrTime - w_ini_EX0_UsrTime
      wdt_EX0_SysTime = w_fin_EX0_SysTime - w_ini_EX0_SysTime

c     Mark PL with an absurd L/M ratio... (for output-safety)
      if (i_FitPowerLaw.EQ.1) fbase_norm(N_base) = 0.



!Cid29Mar07-FIX! Now write arq_out to out_dir, so figure what is the first
!     blank char in obs_dir & concatenate it: arq_out --> arq_aux
      i_last_non_blank_char = 0
      do i=1,99
         if ((out_dir(i:i+1).EQ.' ').AND.(i_last_non_blank_char.EQ.0))
     &        i_last_non_blank_char = i - 1
      end do
      arq_aux = out_dir(1:i_last_non_blank_char) // arq_out

c     Open output file & write header
      open (unit=2,file=arq_aux,status='unknown')

      write (2,190)
 190  format(67('#'))
      write (2,'(a,4x,a)') '## OUTPUT of StarlightChains_v05.for',
     &     '[Cid@UFSC - 10/May/2008] ##'
      write (2,190)

      write (2,*)
      write (2,'(a)') '## Some input info'
      write (2,'(a40,a)')       arq_obs     , ' [arq_obs]'
      write (2,'(a40,a)')       arq_base    , ' [arq_base]'
      write (2,'(a40,a)')       arq_masks   , ' [arq_masks]'
      write (2,'(a40,a)')       arq_config  , ' [arq_config]'
      write (2,'(i5,35x,a)')    N_base      , ' [N_base]'
      write (2,'(i5,35x,a)') N_YAV_components ,
     &    ' [N_YAV_components = # of components with extra extinction!]'
      write (2,'(i5,35x,a)')  i_FitPowerLaw  , 
     &     ' [i_FitPowerLaw (1/0 = Yes/No)]'
      write (2,'(f5.2,35x,a)')  alpha_PowerLaw  , 
     &     ' [alpha_PowerLaw]'

      write (2,'(a3,37x,a)') red_law_option , ' [red_law_option]'
      write (2,'(f9.6,31x,a)') q_norm , ' [q_norm = A(l_norm)/A(V)]'

      write (2,*)
      write (2,'(a)') '## (Re)Sampling Parameters'
      write (2,'(f8.2,32x,a)') Olsyn_ini     , ' [l_ini (A)]'
      write (2,'(f8.2,32x,a)') Olsyn_fin     , ' [l_fin (A)]'

!Cid20Jan07-FIX! Now echoing dl_cushion just in case...
      write (2,'(f8.2,32x,a,f8.2,a)') Odl_syn ,
     $     ' [dl    (A)]     (OBS: dl_cushion = ', dl_cushion , ' A)'

      write (2,*)
      write (2,'(a)') '## Normalization info'
      write (2,'(f8.2,32x,a)')     l_norm   , ' [l_norm (A) - for base]'
      write (2,'(f8.2,32x,a)')  llow_norm   , 
     &     ' [llow_norm (A) - window for f_obs]'
      write (2,'(f8.2,32x,a)')  lupp_norm   , 
     &     ' [lupp_norm (A) - window for f_obs]'
      write (2,'(1p,e12.6,28x,a)') fobs_norm, 
     &     ' [fobs_norm (in input units)]'

      write (2,*)
      write (2,'(a)') '## S/N'
      write (2,'(f8.2,32x,a)') llow_SN,' [llow_SN (A) - window for S/N]'
      write (2,'(f8.2,32x,a)') lupp_SN,' [lupp_SN (A) - window for S/N]'
      write (2,'(f8.3,32x,a)') SN_snwin   ,' [S/N in S/N window]'
      write (2,'(f8.3,32x,a)') SN_normwin ,' [S/N in norm. window]'
      write (2,'(f8.3,32x,a)')SNerr_snwin  ,' [S/N_err in S/N window]'
      write (2,'(f8.3,32x,a)')SNerr_normwin,' [S/N_err in norm. window]'
      write (2,'(f8.4,32x,a)') fscale_chi2 , ' [fscale_chi2]'

      write (2,*)
      write (2,'(a)') '## etc...'
!Cid29Dec06-FIX! just changed idum-format.
      write (2,'(i12,28x,a)')   idum_orig   , ' [idum_orig]'
      write (2,'(i6,34x,a)')   NOl_eff      , ' [NOl_eff]'
      write (2,'(i6,34x,a)')   Nl_eff       , ' [Nl_eff]'
      write (2,'(i6,4x,a6,25x,a)') Ntot_cliped , clip_method_option ,
     $     '[Ntot_cliped & clip_method]'
      write (2,'(i9,31x,a)')  Nglobal_steps , ' [Nglobal_steps]'
      write (2,'(i4,36x,a)')  N_chains , ' [N_chains]'
      write (2,'(i4,36x,a)')  NEX0s_base ,
     $     ' [NEX0s_base = N_base in EX0s-fits]'
!Cid24Mar07-FIX! Added censored weights info here
      write (2,'(3(i2,2x),2x,i5,2x,f4.1,2x,1p,e10.3,3x,a,a)') 
     $     iCLIPBUG_flag ,i_RC_CRASH_FLAG, IsBurInOver_BurnIn , 
     $     n_censored_weights , wei_nsig_threshold , wei_limit ,
     $     ' [Clip-Bug, RC-Crash & Burn-In warning-flags,' , 
     $     ' n_censored_weights, wei_nsig_threshold & wei_limit]'
!Cid29Dec06-FIX! Removed one empty line to write time-counters info!
      write (2,'(i6,3(3x,f8.3),1x,a)') 
     $     idt_all , wdt_TotTime , wdt_UsrTime , wdt_SysTime ,
     $     ' [idt_all, wdt_TotTime, wdt_UsrTime & wdt_SysTime (sec)]'


c     Synthesis results - Global Minimum!
      write (2,*)
      write (2,'(a)') '## Synthesis Results - Best model ##'
      write (2,*)

!Cid24Mar07-FIX! Before I mixed EX0sGpar_min, EX0sv0_min & etc with 
!     x_min, AV_min etc in this output stage. I've reorganized it so that
!     now everything is in terms of the *_min quantities.
      call Update_AV_array(N_base,AV_min,YAV_min,AV_tot)
      call Kcalc_Fsyn(Nl_obs,x_min,AV_tot,v0_min,vd_min,f_syn)
      chi2_min = FK_chi2(x_min,AV_tot,v0_min,vd_min) / float(Nl_eff)
      adev     = F_adev(f_syn)
      sum_x    = 100. * Sum_of_x(x_min,N_base)

      call ConvertLight2Mass(N_base,x_min,AV_tot,q_norm,fobs_norm
     $     ,fbase_norm,i_FitPowerLaw,Mstars,Flux_tot,Mini_tot,Mcor_tot
     $     ,mu_ini,mu_cor)

      write (2,'(1p,e12.5,28x,a)') chi2_min  , ' [chi2/Nl_eff]'
      write (2,'(f12.5,28x,a)') adev         , ' [adev (%)]'
      write (2,*)
      write (2,'(f12.5,28x,a)') sum_x        , ' [sum-of-x (%)]'
      write (2,'(1p,e12.5,28x,a)') Flux_tot  , 
     &     ' [Flux_tot (units of input spectrum!)]'
      write (2,'(1p,e12.5,28x,a)') Mini_tot      , ' [Mini_tot (???)]'
      write (2,'(1p,e12.5,28x,a)') Mcor_tot      , ' [Mcor_tot (???)]'

      write (2,*)
      write (2,'(f7.2,34x,a)') v0_min  ,'[v0_min  (km/s)]'
      write (2,'(f7.2,34x,a)') vd_min  ,'[vd_min  (km/s)]'
      write (2,'(f7.4,34x,a)') AV_min  ,'[AV_min  (mag)]'
      write (2,'(f7.4,34x,a)') YAV_min ,'[YAV_min (mag)]'

      write (2,*)

!Cid20Jan07-FIX! included an alpha/Fe array in pop-vector table, as well as
!     new single component fits SSP_* arrays!
      write (2,191)
  191  format ('# j     x_j(%)      Mini_j(%)     Mcor_j(%)' , 
     &     '     age_j(yr)     Z_j      (L/M)_j    YAV?  Mstars',
     &     '   component_j        a/Fe...',
     &     '       SSP_chi2r SSP_adev(%)   SSP_AV   SSP_x(%)')

      do j=1,N_base
         write (2,192) j , (100.*x_min(j)) , 
     &        (100.*mu_ini(j)) , (100.*mu_cor(j)) , 
     &        age_base(j) , Z_base(j) , fbase_norm(j) , 
     &        ind_YAV(j) , Mstars(j) , base_name(j) , alphaFe_base(j) ,
     &        SSP_chi2(j) / float(Nl_eff) , SSP_adev(j) , 
     &        SSP_AV(j) , 100. * SSP_x(j)
      end do
 192  format (i3,2x,f10.4,3x,
     &     1p,e12.4,2x,e12.4,4x,
     &     e12.6,2x,0p,f8.5,2x,1p,e9.3,3x,
     &     0p,i2,3x,f6.4,3x,a15,4x,f9.4,4x,
     &     1p,e11.4,0p,3x,f8.4,2x,
     &     f7.4,2x,f9.4)


!     Screen output Best model results
      if (i_verbose.GT.0) then
         write (*,*)
         write (*,'(a,a)') '**> RESULTS for ' , arq_out
         write (*,812) N_base
 812     format ('   N_tot chi2/Nl_eff    adev |     v0    vd | ',
     &        'sum       A_V  Y_AV |     x_j... [%], j = 1 ...',i3)
         write (*,813) Nglobal_steps , chi2_min , adev , 
     &        v0_min , vd_min , sum_x , AV_min , YAV_min , 
     &        (100.*x_min(j),j=1,min(10,N_base))
 813     format (i8,1x,1p,e10.4,0p,2x,f7.4,' | ' ,
     &        f6.1,1x,f5.1,' | ',f7.3,1x,f5.2,1x,f5.2,' | ',10(1x,f6.2))
            do j_out=2,int(N_base/10)+1
               j1 = (j_out - 1) * 10 + 1
               j2 = min(j1+9,N_base)
               write (*,'(68x,10(1x,f6.2))') (100.*x_min(j),j=j1,j2)
            end do
            write (*,'(70x,a,1p,e12.4,a,0p,i6,a)') 'Total chi2 = ' ,
     $           chi2_min * Nl_eff , '  for Nl_eff =' , Nl_eff ,
     $           ' lambdas!'
         write (*,*)

!Cid29Dec06-FIX! Screen-echo time-counters (to help in config-tests)
         write (*,'(5(a,i8,2x),a,a)') 
     $        '*CidTIME>  FF= ' , idt_FF ,
     $        ' RF= ' , idt_RF  , ' BI= ' , idt_BI ,
     $        ' EX0= ', idt_EX0 , ' ALL= ', idt_all    ,
     $        ' sec - fit = ' , arq_out
         write (*,'(5(a,f8.3,2x),a,a)') 
     $        '*TotTIME>  FF= ' , wdt_FF_TotTime ,
     $        ' RF= ' , wdt_RF_TotTime  , ' BI= ' , wdt_BI_TotTime ,
     $        ' EX0= ', wdt_EX0_TotTime , ' ALL= ', wdt_TotTime    ,
     $        ' sec - fit = ' , arq_out
         write (*,'(5(a,f8.3,2x),a,a)') 
     $        '*UsrTIME>  FF= ' , wdt_FF_UsrTime ,
     $        ' RF= ' , wdt_RF_UsrTime  , ' BI= ' , wdt_BI_UsrTime ,
     $        ' EX0= ', wdt_EX0_UsrTime , ' ALL= ', wdt_UsrTime    ,
     $        ' sec - fit = ' , arq_out
         write (*,'(5(a,f8.3,2x),a,a)') 
     $        '*SysTIME>  FF= ' , wdt_FF_SysTime ,
     $        ' RF= ' , wdt_RF_SysTime  , ' BI= ' , wdt_BI_SysTime ,
     $        ' EX0= ', wdt_EX0_SysTime , ' ALL= ', wdt_SysTime    ,
     $        ' sec - fit = ' , arq_out
         write (*,*)
      end if


c     Now output results for best (repeated), average & last-chain
c     parameters after Burn-In loop! (This should give us an idea of the
c     errors, and in any case it'd be a waste to throw all these numbers
c     away!)
      write (2,*)
      write (2,*)
      write (2,'(a)') '## Synthesis Results - Average & Chains ##'
      write (2,*)

c     Store chi2 & AV for each chain is f_aux variables, chain-mass
c     fractions in EX0sChainPar array & chain-masses in CEne_min (not
c     used anymore)...
      do i_chain=1,N_chains
         do j=1,N_par
            f_aux1(j) = ChainPar(j,i_chain)
         end do
         call Unpack_par2xAVYAVfn(N_par,f_aux1,x,AV,YAV,fn)
         call Update_AV_array(N_base,AV,YAV,AV_tot)
         chi2_red = FK_chi2(x,AV_tot,v0_min,vd_min) / float(Nl_eff)
         f_aux2(i_chain) = AV
         f_aux3(i_chain) = chi2_red

         call ConvertLight2Mass(N_base,x,AV_tot,q_norm,fobs_norm
     $        ,fbase_norm,i_FitPowerLaw,Mstars,Flux_tot,Mini_tot
     $        ,chain_Mcor_tot,mu_ini,aux1)
         do j=1,N_base
            EX0sChainPar(j,i_chain) = aux1(j) 
         end do
         aux2(i_chain) = chain_Mcor_tot
      end do

c     Now compute things for average solution
      call Unpack_par2xAVYAVfn(N_par,Gpar_ave,x,AV_ave,YAV_ave,fn_ave)
      call Update_AV_array(N_base,AV_ave,YAV_ave,AV_tot)
      chi2_ave = FK_chi2(x,AV_tot,v0_min,vd_min) / float(Nl_eff)
      call ConvertLight2Mass(N_base,x,AV_tot,q_norm,fobs_norm
     $     ,fbase_norm,i_FitPowerLaw,Mstars,Flux_tot,Mini_tot
     $     ,Mcor_tot_ave,mu_ini,aux3)

c     Print out x, mu, AV, chi2 & Masses for ave & chain-solutions
      write (2,'(a,i3,a)')
     $     '# j      x_j: min, <> & last-chain-values for 1 ...'
     $     , N_chains , ' chains'
      do j=1,N_base
         write (2,'(i3,20(2x,f10.4))') j , 100.*EX0sGpar_min(j) ,
     $        100.*Gpar_ave(j) , ((100.*ChainPar(j,i_chain)),i_chain=1
     $        ,N_chains)
      end do

      write (2,*)
      write (2,'(a,i3,a)')
     $     '# j      m_j: min, <> & last-chain-values for 1 ...'
     $     ,N_chains , ' chains'
      do j=1,N_base
         write (2,'(i3,1p,20(2x,e10.4))') j , 100.*mu_cor(j) , 100.
     $        *aux3(j), ((100.*EX0sChainPar(j,i_chain)),i_chain=1
     $        ,N_chains)
      end do

      write (2,*)
      write (2,'(a,i3,a)') '# AV, chi2 & Mass for <> & i_chain = 1 ...'
     $     , N_chains , ' solutions'
      write (2,'(a5,20(f10.4,2x))') 'AV   ' , AV_min , AV_ave  ,
     $     (f_aux2(i_chain),i_chain=1,N_chains)
      write (2,'(a5,1p,20(e10.4,2x))') 'chi2 ', chi2_min , chi2_ave ,
     $     (f_aux3(i_chain),i_chain=1,N_chains)
      write (2,'(a5,1p,20(e10.4,2x))') 'Mass ' , Mcor_tot ,
     $     Mcor_tot_ave ,(aux2(i_chain),i_chain=1,N_chains)
!Cid24Mar07-FIX! Now the v0 & vd values Before EX0s have proper names...
      write (2,'(f7.2,34x,a)') v0_min_BeforeEX0s ,
     $     '[v0_min (km/s) before EX0s...]'
      write (2,'(f7.2,34x,a)') vd_min_BeforeEX0s , 
     $     '[vd_min (km/s) before EX0s...]'


c     Observed, Synthetic & Weight Spectra: Save f_obs, f_syn &
c     weight_obs. Save only points inside originally input range:
c     Olsyn_ini ==> Olsyn_fin!  Points which have been clipped (those
c     with f_mask < 0) are saved with -1 in the weight_obs column just
c     for control.
      Nl_obs_save = 0
      do i=1,Nl_obs
         if ((l_obs(i).GE.Olsyn_ini).AND.(l_obs(i).LE.Olsyn_fin)) then
            Nl_obs_save = Nl_obs_save + 1
         end if
      end do

      write (2,*)
      write (2,*)

!v05! Saving Best_f_SSP() too (if wanted)
      if (i_SaveBestSingleCompFit.EQ.0) then
         write (2,'(a,a)') '## Synthetic spectrum (Best Model) ##',
     &        'l_obs f_obs f_syn wei'
      else 
         write (2,'(a,a)') '## Synthetic spectrum (Best Model) ##',
     &        'l_obs f_obs f_syn wei Best_f_SSP'
      end if
      write (2,'(i5,4x,i5,4x,a)') Nl_obs_save , index_Best_SSP  ,
     $     '[Nl_obs & index_Best_SSP]'

      do i=1,Nl_obs

         if ((l_obs(i).GE.Olsyn_ini).AND.(l_obs(i).LE.Olsyn_fin)) then
            aux = weight_obs(i)
            if  (f_mask(i).LT.0.) aux = f_mask(i)
!Cid30Mar07-FIX! Format of l_obs changed from i4 to f8.2 to cope with IR
!     data and high-res too.

!v05! Saving Best_f_SSP() too (if wanted)
            if (i_SaveBestSingleCompFit.EQ.0) then
               write (2,'(f8.2,1x,f10.5,1x,f9.5,1x,f8.3)') l_obs(i) , 
     &              f_obs(i) , f_syn(i) , aux
            else
               write (2,'(f8.2,1x,f10.5,1x,f9.5,1x,f8.3,1x,f9.5)') 
     &              l_obs(i) , f_obs(i) , f_syn(i) , aux , Best_f_SSP(i)
               
            end if
         end if

      end do

      close(2)
c ***************************************************************************
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                  DONE WITH POST-PROCESSING & OUTPUT:)                   +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+



      end
c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c @                           END OF MAIN CODE                              @
c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                       A BUNCH OF SIMPLE ROUTINES                        +
c +                                                                         +
c + ReadSpecWithHeader                                                      +
c + calc_Fnorm                                                              +
c + Stats_F                                                                 +
c + Sum_of_x                                                                +
c + F_adev                                                                  +
c + Interpol_Spec - not really used!!                                       +
c + RebinSpec                                                               +
c + GaussSmooth                                                             +
c + Calc_Mean_Age_and_Z                                                     +
c + Update_AV_array                                                         +
c + Clip_Spectrum                                                           +
c + ConvertLight2Mass                                                       +
c + F_SSP_chi2_AV (added on 20/Jan/2007)                                    +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+



c ###########################################################################
c Reads a spectrum, but skips first N_skip lines if they start with a '#'.
c OBS: '#' must be in the 1st column!!
c Cid@lynx - 15/Aug/2003

      SUBROUTINE ReadSpecWithHeader(arq,N_max,N,l,f)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real l(N_max) , f(N_max)
      character*100 arq
      character*1 cerquinha

c ** First find-out how many header lines (those which start with '#') to skip
      N_skip = 0
      open (unit=1,file=arq,status='old')
      do i=1,N_max
         read (1,'(a1)') cerquinha
         if (cerquinha.EQ.'#') then 
            N_skip = N_skip + 1
         else
            goto 10
         end if
      end do
 10   close(1)

c ** Now reads spectrum, skiping 1st N_skip lines
      open (unit=1,file=arq,status='old')
      do i=1,N_skip
         read (1,*)
      end do
      do i=1,N_max
         read (1,*,END=20) l(i) , f(i)
      end do
      write (*,*) ' [ReadSpecWithHeader] BUG! N > Nl_max!! ' , N_max
 20   close(1)
      N = i - 1

      end
c ###########################################################################


c ###########################################################################
c Interpolates flux at l_norm Angstrons in a f(l) flux vector. 
c Used for normalization purposes.
c Cid@Lynx - Aug/09/2003

      SUBROUTINE calc_Fnorm(N,l,f,l_norm,f_norm)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real l(N) , f(N)

      do i=1,N-1
         if ((l(i).LE.l_norm).AND.(l(i+1).GE.l_norm)) then
            a = (f(i+1) - f(i)) / (l(i+1) - l(i))
            f_norm = a * (l_norm - l(i+1)) + f(i+1)
            return
         end if
      end do

      write (*,'(3(a,f7.1))')
     $     '[calc_Fnorm] Spectrum does NOT contain l_norm = ' , l_norm ,
     $     ' l goes from ' , l(1) , ' -> ' , l(N)
      stop

      end
c ###########################################################################


c ###########################################################################
c     Compute average, sigma & median flux between l1 & l2.
c     Cid@Lynx - Aug/24/2003
c     Added safety-check when i_tot < 2. Cid@Lagoa - 08/Oct/2005

      SUBROUTINE Stats_F(N,l,f,l1,l2,f_ave,f_sig,f_med)

      parameter (Nl_max = 30000)
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real l(N) , f(N) , f_aux(Nl_max)

      i_tot = 0
      do i=1,N
         if ((l(i).GE.l1).AND.(l(i).LE.l2)) then
            i_tot = i_tot + 1
            f_aux(i_tot) = f(i)
         end if
      end do

      f_ave = 0.
      f_sig = 0.
      f_med = 0.
      if (i_tot.GT.1) then
         call avevar(f_aux,i_tot,f_ave,f_var)
         f_sig = sqrt(f_var)

         call MDIAN1(f_aux,i_tot,f_med)
      end if

      end
c ###########################################################################


c ###########################################################################
c     Compute average & sigma of f between l1 & l2 but considering only
c     points with w > 0. Silly routine used in the computation of S/N
c     ratios (for output purposes only!).
c     Cid@Lynx - Jan/27/2005
c     Added safety-check when i_tot < 2. Cid@Lagoa - 08/Oct/2005

      SUBROUTINE Stats4SN(N,l,f,w,l1,l2,f_ave,f_sig)

      parameter (Nl_max = 30000)
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real l(N) , f(N) , w(N) , f_aux(Nl_max)

      i_tot = 0
      do i=1,N
         if ((l(i).GE.l1).AND.(l(i).LE.l2).AND.(w(i).GT.0.)) then
            i_tot = i_tot + 1
            f_aux(i_tot) = f(i)
         end if
      end do

      f_ave = 0.
      f_sig = 0.
      if (i_tot.GT.1) then
         call avevar(f_aux,i_tot,f_ave,f_var)
         f_sig = sqrt(f_var)
      end if

      end
c ###########################################################################


c ###########################################################################
c # Returns the sum of x
c # Cid@Lynx - Feb/21/2005

      FUNCTION Sum_of_x(x,N)

      parameter (Nb_max = 400)
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real x(Nb_max)

      sum_x = 0.
      do j=1,N
         sum_x = sum_x + abs(x(j))
      end do
      Sum_of_x = sum_x

      end
c ###########################################################################


c ###########################################################################
c # Compute mean absolute relative deviation, in percent. 
c # OBS: only for the Nl_eff points actually used in the synthesis!
c # Note that most of the stuff is passed by a common!
c # Cid@Lynx - Mar/16/2003

c # Minor change: Nl_eff is now computed here.
c # Cid@Lynx - Aug/17/2003

      FUNCTION F_adev(f_syn)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real f_syn(Nl_max)
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs

      adev   = 0.
      Nl_eff = 0
      do i=1,Nl_obs
         if (weight_obs(i).GT.0.) then
            adev = adev + abs((f_obs(i) - f_syn(i)) / f_obs(i))
            Nl_eff = Nl_eff + 1
         end if
      end do
      F_adev = 100. * adev / float(Nl_eff)

      return
      end
c ###########################################################################


c ###########################################################################
c # Simple linear interpolation of an input spectrum fo(lo) on to fs(ls),
c # where ls is given by the user.
c # Cid@Lynx - 03/Jan/2003

      SUBROUTINE Interpol_Spec(No,lo,fo,Ns,ls,fs)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real lo(No) , fo(No) , ls(Ns) , fs(Ns)

      do is=1,Ns
         do io=1,No-1
            if ((ls(is).GE.lo(io)).AND.(ls(is).LE.lo(io+1))) then
               a = (fo(io+1) - fo(io)) / (lo(io+1) - lo(io))
               b = fo(io) - a * lo(io)
               fs(is) = a * ls(is) + b
               goto 10
            end if
         end do

         write (*,*) ' [Interpol_Spec] BUG! ' , is , ls(is)
         stop

 10      do_nothing_on_this_silly_line = 0.
      end do

      end
c ###########################################################################


c ###########################################################################
c # Rebin an input spectrum fo(lo) on to fs(ls) by direct integration...
c # Both lo & ls MUST BE sorted in increasing order.
c # This routine treats a spectrum as a "histogram": 
c #   fo(lo) is taken as constant within the lo bin
c #   lo(i) represents a bin from [lo(i-1)+lo(i)]/2 ==> [lo(i)+lo(i+1)]/2
c # Note that lo (nor ls) does not need to be uniformily spaced.
c # Flux density is preserved.
c # Cid@Lynx - 24/Aug/2003

c # I've introduced a i_FastBC03_FLAG which greatly speeds up rebinning
c # for a BC03 base and when dl = 1 and Ol_obs is within 3322-->9300 Angs. 
c # In this case no interpolation is needed! I just cut-out the piece of the
c # BC03 spectrum which matches the desired ls lambdas (assumed to be spaced
c # by 1 Angs, as the BC03 files).
c # Cid@Lagoa - 7/May/2004

c     I now test whether dls = ls(2) - ls(1) = 1 Angs, the BC03 sampling
c     in the high resolution optical range. If this is not 1 Angs, the
c     FastBC03 rebinning-trick should not be aplied!
c     Cid@Lagoa - 28/Jan/2005

c     I've adapted the fast-rebin trick to work with Base-Rosa spectra
c     (br04*spec files), which cover the 3001 --> 6996 Angs range. To
c     use fast-rebin with BR04 you must set i_FastBC03_FLAG = 2.
c
c     Cid@IAA - 27/May/2005

      SUBROUTINE RebinSpec(i_FastBC03_FLAG,No,lo,fo,Ns,ls,fs)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real lo(No) , fo(No) , ls(Ns) , fs(Ns)

      if ((i_FastBC03_FLAG.GE.1).AND.((ls(2) - ls(1)).NE.1.0)) then
         write (*,'(a,a,i4,f8.3)') ' [RebinSpec] Tried FastBC03-' ,
     &        'rebinning but dl NE 1 Angs. Going for slow rebinning!' , 
     &        i_FastBC03_FLAG , ls(2) - ls(1)
      end if

      if ((i_FastBC03_FLAG.EQ.1).AND.((ls(2) - ls(1)).EQ.1.0).AND.
     $     (int(ls(1)).GE.3222).AND.(int(ls(Ns)).LE.9300)) then


c ** Just cut-out a BC03 spectrum to ls. See notes above.
         do is=1,Ns
            ind_in_BC03 = int(ls(is)) - 3322 + 348
            fs(is) = fo(ind_in_BC03)

c ** Paranoid check...
            if (int(ls(is)).NE.int(lo(ind_in_BC03))) then
               write (*,*) ' [RebinSpec] BUG in FastBC03 scheme! ' , 
     &              is , ls(is) , ind_in_BC03 , lo(ind_in_BC03)
               write (*,*) ' [RebinSpec] Try setting',
     $              ' i_FastBC03_FLAG = 0 in arq_config' 
               stop
            end if
         end do


! Same thing but for br04*spec, which cover 3001-->6996 Angs.
      else if ((i_FastBC03_FLAG.EQ.2).AND.((ls(2) - ls(1)).EQ.1.0).AND.
     $        (int(ls(1)).GE.3001).AND.(int(ls(2)).LE.6996)) then


c ** Just cut-out a BR04spectrum to ls. See notes above.
         do is=1,Ns
            ind_in_BR04 = int(ls(is)) - 3001 + 1
            fs(is) = fo(ind_in_BR04)

c ** Paranoid check...
            if (int(ls(is)).NE.int(lo(ind_in_BR04))) then
               write (*,*) ' [RebinSpec] BUG in FastBR04 scheme! ' , 
     &              is , ls(is) , ind_in_BR04 , lo(ind_in_BR04)
               write (*,*) ' [RebinSpec] Try setting',
     $              ' i_FastBC03_FLAG = 0 in arq_config' 
               stop
            end if
         end do


! Slow-rebinning.
      else


c ** Loop over ls-bins
      do is=1,Ns


c ** Define bin limits corresponding to ls(is)
c ** (1st & last bins are forced to be simmetric)
         if (is.EQ.1) then 
            ls_low = ls(is) - (ls(is+1) - ls(is)) / 2.
         else 
            ls_low = (ls(is) + ls(is-1)) / 2.
         end if

         if (is.EQ.Ns) then 
            ls_upp = ls(is) + (ls(is) - ls(is-1)) / 2.
         else 
            ls_upp = (ls(is) + ls(is+1)) / 2.
         end if


c ** Find lo points within current ls bin & integrate flux
         sum_bin = 0.
         do io=1,No


c ** Define bin limits corresponding to lo(io)
c ** (1st & last bins are forced to be simmetric)
            if (io.EQ.1) then 
               lo_low = lo(io) - (lo(io+1) - lo(io)) / 2.
            else 
               lo_low = (lo(io) + lo(io-1)) / 2.
            end if
            
            if (io.EQ.No) then 
               lo_upp = lo(io) + (lo(io) - lo(io-1)) / 2.
            else 
               lo_upp = (lo(io) + lo(io+1)) / 2.
            end if


c ** Check is ls & lo bins overlap. If not, skip this lo-bin. 
c ** If yes, integrate fo(lo) contribution to current ls bin.
            if ((ls_upp.LE.lo_low).OR.(ls_low.GE.lo_upp)) then

               ls_bin_does_not_overlap_with_current_lo_bin = 0.

            else

c ** set l_inf = MAX(lo_low,ls_low) and l_sup = MIN(lo_upp,ls_upp)
               l_inf = lo_low
               if (ls_low.GT.lo_low) l_inf = ls_low

               l_sup = lo_upp
               if (ls_upp.LT.lo_upp) l_sup = ls_upp

               sum_bin= sum_bin+ fo(io) * (l_sup - l_inf)
            end if 

         end do


c ** sum_bin= integrated flux in ls-bin & fs = sum_bin per delta-ls
         fs(is) = sum_bin/ (ls_upp - ls_low)


c ** Screen check...
!         write (*,11) 'Rebin> ' , is , ls_low,ls(is),ls_upp,fs(is),sum
! 11      format (a,i5,2x,3(2x,f7.2),1p,2(2x,e9.3))
      end do


      end if


      end
c ###########################################################################


c ###########################################################################
c # Smooth an input spectrum fo(lo) with a Gaussian centered at velocity
c # v = v0 and with sig as velocity dispersion. The output is fs(ls).
c # The convolution is done with a BRUTE FORCE integration of n_u points
c # from u = -n_sig to u = +n_sig, where u = (v - v0) / sig.
c # OBS: Assumes original spectrum fo(lo) is EQUALLY SPACED in lambda!!
C #      Units of v0 & sig = km/s
c # Cid@Lynx - 05/Dec/2002

c # Added n_u as a parameter!
c # Cid@Lynx - 16/Mar/2003

      SUBROUTINE GaussSmooth(No,lo,fo,v0,sig,n_u,Ns,ls,fs)

** Definitions
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      real lo(No) , fo(No) , ls(Ns) , fs(Ns)
      pi = 4. * atan(1.)
      c  = 2.997925e5


c ** Check n_u
      if (n_u.LT.5) then
         write (*,*) ' [GaussSmooth] nu too small! ' , n_u
         stop
      end if


c ** Parameters for the brute-force integration in velocity
      n_sig = 6
      u_low = -n_sig
      u_upp = n_sig
      du    = (u_upp - u_low) / float(n_u - 1)
      d_lo  = lo(2) - lo(1)


c ** loop over ls, convolving fo(lo) with a gaussian
      do is=1,Ns

c ** reset integral of {fo[ll = ls/(1+v/c)] * Gaussian} & start integration
         sum_fg = 0.
         do u=u_low,u_upp,du

c ** define velocity & lambda corresponding to u
            v  = v0 + abs(sig) * u
            ll = ls(is) / (1. + v/c)

c ** find fo flux for lambda = ll
            if (ll.LT.lo(1)) then 
               ff = fo(1)
            else if (ll.GT.lo(No)) then 
               ff = fo(No)
            else
               ind = (ll - lo(1)) / d_lo + 1
               a   = (fo(ind+1) - fo(ind)) / (lo(ind+1) - lo(ind))
               b   = fo(ind) - a * lo(ind)
               ff  = a * ll + b
!@!%TESTING%! ff  = fo(ind) treats the spectrum as a histogram...
!@!               ffold = ff
!@!               ff  = fo(ind)
!@!               write (*,*) ' test in Gauss-Smooth' , ffold,ff,ffold/ff
            end if

            sum_fg = sum_fg + ff * exp(-(u**2/2.)) 
         end do

c ** smoothed spectrum
         fs(is) = sum_fg * du / sqrt(2. * pi)

      end do


      end
c ###########################################################################


c ###########################################################################
c Silly mean age & Z - just for screen output/check purposes
c Cid@lynx - 04/Feb/04

      SUBROUTINE Calc_Mean_Age_and_Z(N_base,x,a_age,s_age,a_met,s_met)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400)
      real x(Nb_max) , age_base(Nb_max) , Z_base(Nb_max)
      common/Age_and_Met/ age_base , Z_base

      sum_t  = 0.
      sum_t2 = 0.
      sum_z  = 0.
      sum_z2 = 0.
      sum_x  = 0.
      do i=1,N_base
         sum_t  = sum_t  + x(i) * log10(age_base(i) + 1.)
         sum_t2 = sum_t2 + x(i) * (log10(age_base(i) + 1.))**2
         sum_z  = sum_z  + x(i) * Z_base(i)
         sum_z2 = sum_z2 + x(i) * (Z_base(i))**2
         sum_x  = sum_x  + x(i)
      end do

      a_age = sum_t / sum_x
      s_age = sqrt(abs((sum_t2/sum_x) - a_age**2))

      a_met = sum_z / sum_x
      s_met = sqrt(abs((sum_z2/sum_x) - a_met**2))

      end
c ###########################################################################


c ###########################################################################
c #    COMPUTE "TOTAL MASS" & MASS FRACTIONS IN COMPONENTS j=1...N_base     #
c ###########################################################################
c     Flux_tot is the total **dereddened** synthetic (WITHOUT
c     kinematics!) flux at l_norm, in the same units as in the input
c     spectrum.  Mass_tot is the total stellar mass. It's units must be
c     fixed by the user a posteriori!  For instance, if (a) the input
c     spectrum comes in units of 1e-17 erg/s/cm2/A, (2) the distance to
c     the galaxy (in cm) is d, and (3) the base spectra were given in
c     units of L_sun/A/M_sun (as is the case for the BC03 spectra), then
c     the actual mass is given by Mass_tot * 1e-17 * 4 * pi * d**2 /
c     3.826e33.
c
c     ATT: There are 2 versions of Mass_tot: Mini_tot & Mcor_tot! Why?
c     The original BC03 SSP spectra come normalized to a total INITIAL
c     mass of 1 M_sun. As time goes by, however, part of this stellar
c     mass goes back to the ISM. This implies that the mass (Mini_tot)
c     and mass-fractions (mu_ini(j)) are NOT the current ones! To
c     correct for the returned mass, I use the array Mstars (read fron
c     the base file), which says what fraction of the initial mass in
c     population # j is still in stars at the age of this same
c     component. The corrected total mass and mass-fractions are given
c     in Mcor_tot & mu_cor(j).
c     
c     OBS1: Note that a Power-Law component (which, if present, is
c     component # N_base) is excluded from the mass calculations (for
c     obvious reasons)!
c     OBS2: Masses & mass fractions may not make any sense at all if,
c     for instance, the base is made of template galaxies or normalized
c     models...
c     OBS3: Note that now AV is population dependent: AV_tot = AV + YAV!
c
c     Cid@Lagoa - 30/Jan/2005

c     Removed unnecessary common/YAV_stuff/ which was here.
c     Cid@Lagoa - 21/Feb/2007

      SUBROUTINE ConvertLight2Mass(N_base,x,AV_tot,q_norm,fobs_norm
     $     ,fbase_norm,i_FitPowerLaw,Mstars,Flux_tot,Mini_tot
     $     ,Mcor_tot,mu_ini,mu_cor)

c     Definitions
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400)

c     L2Mass (@ l_norm) in the base
      real fbase_norm(Nb_max) 

c     Population vector arrays in flux (x) & mass (mu_ini) fractions
      real x(Nb_max) , mu_ini(Nb_max)

c     Mstars = array which says what fraction of the INITIAL stellar
c     mass in population # j is still in stars at age(j). mu_cor() are
c     the corrected mass-fractions.
      real Mstars(Nb_max) , mu_cor(Nb_max)

c     AV_array: AV_tot(j) = population dependent extinction
      real AV_tot(Nb_max)
      common/YAV_stuff/ ind_YAV(Nb_max) , N_YAV_components


c     Computes Flux_tot & total initial & actual (corrected) masses.
      Flux_tot = 0.
      Mini_tot = 0.
      Mcor_tot = 0.
      do j=1,N_base
         flux_j = x(j) * fobs_norm * 10.**(0.4 * AV_tot(j) * q_norm)
         mini_j = flux_j / fbase_norm(j)
         mcor_j = flux_j / fbase_norm(j) * Mstars(j)
         if ((j.EQ.N_base).AND.(i_FitPowerLaw.EQ.1)) then 
            mini_j = 0.
            mcor_j = 0.
         end if
         Flux_tot = Flux_tot + flux_j
         Mini_tot = Mini_tot + mini_j
         Mcor_tot = Mcor_tot + mcor_j
      end do

c     Compute mass fractions (mu_ini & mu_cor) associated with each base
c     component.
      do j=1,N_base
         flux_j = x(j) * fobs_norm * 10.**(0.4 * AV_tot(j) * q_norm)
         mini_j = flux_j / fbase_norm(j)
         mcor_j = flux_j / fbase_norm(j) * Mstars(j)
         if ((j.EQ.N_base).AND.(i_FitPowerLaw.EQ.1)) then 
            mini_j = 0.
            mcor_j = 0.
         end if

         mu_ini(j)  = mini_j / Mini_tot
         mu_cor(j)  = mcor_j / Mcor_tot
      end do

      end
c ###########################################################################


c ###########################################################################
c     Simple routine just to store AV & YAV onto an AV_tot array. The
c     array ind_YAV(Nb_max), defined elsewhere, says which components
c     have the extra extinction YAV. Routines which compute a synthetic
c     spectrum and the chi2-routines allow for N_base different values
c     of AV(j), but my implementation of different extinctions allows
c     for just two values: AV_tot(j) = AV & AV + YAV, as done here.

c     Cid@Lagoa - 26/Jan/2005

      SUBROUTINE Update_AV_array(N_base,AV,YAV,AV_tot)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400)
      real AV_tot(Nb_max)
      common/YAV_stuff/ ind_YAV(Nb_max) , N_YAV_components

      do j=1,N_base
         AV_tot(j) = AV
         if (ind_YAV(j).EQ.1) AV_tot(j) = AV + YAV
      end do

      end
c ###########################################################################


c ###########################################################################
c #                      SIGMA-CLIPPING (excluding outliers)                #
c ###########################################################################
c     Given an observed (f_obs) and a model spectrum (f_syn)
c     mask-out/clip points with large residuals. 3 clipping methods are
c     offered: clip_method_option = "ABSRES", "RELRES" & "NSIGMA". In
c     the ABSRES method we clip points with |f_obs-f_syn| "ABSOLUTE"
c     residuals > sig_clip_threshold X the rms (f_obs - f_syn) over all
c     (non-masked) lambdas. [This was the only option in STARLIGHT up to
c     March/2005]. "RELRES" clipping consists of excluding points whose
c     RELATIVE residual |f_obs-f_syn|/error deviates by more than
c     sig_clip_threshold X the rms of this same quantity over all non
c     masked points. Finally, "NSIGMA" clipping consists of excluding
c     points whose |f_obs-f_syn| residual deviates by more than
c     sig_clip_threshold X the LOCAL error in f_obs. This scheme should
c     clip less points than ABSRES, since it may allow points with large
c     errors to be kept in f_obs for fitting. In all cases, exclusions
c     are added to weight_obs array, and clipped points are flaged as
c     f_mask(i) = -1 (just to mark them for output/control purposes).
c
c     Cid@Lagoa - 24/Mar/2005

      SUBROUTINE Clip_Spectrum(clip_method_option,f_syn
     $     ,sig_clip_threshold,f_mask,NOl_eff,Nl_eff,Ntot_cliped
     $     ,iCLIPBUG_flag)

               
c     Definitions
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      real l_obs(Nl_max) , f_obs(Nl_max)  , weight_obs(Nl_max)
      real f_syn(Nl_max) , f_mask(Nl_max) , f_aux1(Nl_max)
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      character*6 clip_method_option


c     Clip-spectrum according to clip_method_option.
      if (clip_method_option.EQ.'NSIGMA') then


c     "NSIGMA" clipping! Compute # of points to be clipped, assign zero
c     weight & flag cliped points with f_mask = -1. Corrects
c     Nl_eff. Store original weight temporarely in f_aux1 (in case I
c     have to undo it below!)
         Ntot_cliped = 0
         do i=1,Nl_obs
            f_aux1(i) = weight_obs(i)
            if (weight_obs(i).GT.0.) then
               rel_resid = (f_syn(i) - f_obs(i)) * weight_obs(i)
               if (abs(rel_resid).GT.sig_clip_threshold) then
                  weight_obs(i) = 0.
                  f_mask(i)     = -1.
                  Ntot_cliped   = Ntot_cliped + 1
                  Nl_eff        = Nl_eff - 1
               end if
            end if
         end do


      else if (clip_method_option.EQ.'RELRES') then


c     "RELRES" clipping!  Store RELATIVE residuals (= residue/error for
c     each lambda) in f_aux1. Compute mean & std dev of relative
c     residuals (only for non masked points).
         i_aux = 0
         do i=1,Nl_obs
            if (weight_obs(i).GT.0.) then
               i_aux = i_aux + 1
               f_aux1(i_aux) = (f_syn(i) - f_obs(i)) * weight_obs(i)
            end if
         end do
         call avevar(f_aux1,i_aux,ave_rel_resid,var_rel_resid)
         sig_rel_resid = sqrt(var_rel_resid)

c     Compute # of points to be clipped, assign zero weight & flag
c     cliped points with f_mask = -1. Corrects Nl_eff. Store original
c     weight temporarely in f_aux1 (in case I have to undo it below!)
         Ntot_cliped = 0
         do i=1,Nl_obs
            f_aux1(i) = weight_obs(i)
            if (weight_obs(i).GT.0.) then
               rel_resid = (f_syn(i) - f_obs(i)) * weight_obs(i)
               if (abs(rel_resid).GT.(sig_clip_threshold*sig_rel_resid))
     $              then
                  weight_obs(i) = 0.
                  f_mask(i)     = -1.
                  Ntot_cliped   = Ntot_cliped + 1
                  Nl_eff        = Nl_eff - 1
               end if
            end if
         end do


      else if (clip_method_option.EQ.'ABSRES') then


c     "ABSRES" clipping!  Store residuals in f_aux1. Compute mean & std
c     dev of residuals (only for non masked points).
         i_aux = 0
         do i=1,Nl_obs
            if (weight_obs(i).GT.0.) then
               i_aux = i_aux + 1
               f_aux1(i_aux) = f_syn(i) - f_obs(i)
            end if
         end do
         call avevar(f_aux1,i_aux,ave_resid,var_resid)
         sig_resid = sqrt(var_resid)

c     Compute # of points to be clipped, assign zero weight & flag
c     cliped points with f_mask = -1. Corrects Nl_eff. Store original
c     weight temporarely in f_aux1 (in case I have to undo it below!)
         Ntot_cliped = 0
         do i=1,Nl_obs
            f_aux1(i) = weight_obs(i)
            if (weight_obs(i).GT.0.) then
               resid = f_syn(i) - f_obs(i)
               if (abs(resid).GT.(sig_clip_threshold*sig_resid)) then
                  weight_obs(i) = 0.
                  f_mask(i)     = -1.
                  Ntot_cliped   = Ntot_cliped + 1
                  Nl_eff        = Nl_eff - 1
               end if
            end if
         end do


      else


         if (clip_method_option.NE.'NOCLIP') then
            write (*,'(a,a)') '[Clip_Spectrum] Bad clip_method_option: '
     $           ,clip_method_option
            write (*,'(a)')   '[Clip_Spectrum] Assuming method = NOCLIP'
            clip_method_option = 'NOCLIP'
         end if


      end if


c     For safety, I first test whether Nl_eff has not become
c     ridiculously small due to excessive clipping. If that happens,
c     clipping is aborted (ie, I restore weight_obs & Nl_eff) and a
c     clip-bug-flag is turned on!
      if (Nl_eff.LT.50) then
         write (*,'(a,a,i5)') '[Clip_Spectrum] Too few points to model!'
     $        ,' clipping aborted ' , Nl_eff
         do i=1,Nl_obs
            weight_obs(i) = f_aux1(i)
         end do
         Nl_eff        = NOl_eff
         Ntot_cliped   = 0
         iCLIPBUG_flag = 1
      end if

c     Warn if > 20% of the data is being clipped...
      if (Ntot_cliped.GT.(0.2*NOl_eff)) then
         write (*,'(a,a,i5,3x,i5)')
     $        '[Clip_Spectrum] WARNING ! > 20% of original data',
     $        ' are being clipped-out... ' , Ntot_cliped , NOl_eff
      end if


      end 
c ###########################################################################


c ###########################################################################
c     Compute the chi2 between a spectrum in f_SSP (passes via common!)
c     and an observed spectrum f_obs, as a function of the
c     extinction. The normalization x_SSP of the fitted-spectrum is
c     computed analyticaly (it's the one satisfying dchi2/dx = 0 for the
c     given AV), and returned via common).  Useful to fit single
c     population spectra.
c
c     Cid@Lagoa - 20/Jan/2007

      FUNCTION F_SSP_chi2_AV(AV_SSP)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/Red_factor/ red_term(Nl_max)
      common/SSP_chi2_stuff/ x_SSP , f_SSP(Nl_max)

      sum1 = 0.
      sum2 = 0.
      do i=1,Nl_obs
         if (weight_obs(i).GT.0.) then
            z1 = weight_obs(i)**2
            z2 = 10.**(AV_SSP * red_term(i)) * f_SSP(i)
            sum1 = sum1 + z1 * z2 * f_obs(i)
            sum2 = sum2 + z1 * z2 * z2
         end if
      end do
      x_SSP = sum1 / sum2

      chi2_SSP = 0.
      do i=1,Nl_obs
         if (weight_obs(i).GT.0.) then
            z1 = 10.**(AV_SSP * red_term(i)) * x_SSP * f_SSP(i)
            chi2_SSP = chi2_SSP + (weight_obs(i) * (f_obs(i) - z1))**2
         end if
      end do
 
      F_SSP_chi2_AV = chi2_SSP

      return
      end
c ###########################################################################
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                    ROUTINES FROM NUMERICAL-RECIPES                      +
c +                                                                         +
c + avevar, mdian1, sort, ran2, golden, mnbrak, gasdev, indexx              +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+



CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE avevar(data,n,ave,var)
      INTEGER n
      REAL ave,var,data(n)
      INTEGER j
      REAL s,ep
      ave=0.0
      do 11 j=1,n
        ave=ave+data(j)
11    continue
      ave=ave/n
      var=0.0
      ep=0.0
      do 12 j=1,n
        s=data(j)-ave
        ep=ep+s
        var=var+s*s
12    continue
      var=(var-ep**2/n)/(n-1)
      return
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE MDIAN1(X,N,XMED)
      DIMENSION X(N)
      CALL SORT(N,X)
      N2=N/2
      IF(2*N2.EQ.N)THEN
        XMED=0.5*(X(N2)+X(N2+1))
      ELSE
        XMED=X(N2+1)
      ENDIF
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE SORT(N,RA)
      DIMENSION RA(N)
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          RRA=RA(L)
        ELSE
          RRA=RA(IR)
          RA(IR)=RA(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            RA(1)=RRA
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(RA(J).LT.RA(J+1))J=J+1
          ENDIF
          IF(RRA.LT.RA(J))THEN
            RA(I)=RA(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        RA(I)=RRA
      GO TO 10
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      FUNCTION ran2(idum)
      INTEGER idum,IM1,IM2,IMM1,IA1,IA2,IQ1,IQ2,IR1,IR2,NTAB,NDIV
      REAL ran2,AM,EPS,RNMX
      PARAMETER (IM1=2147483563,IM2=2147483399,AM=1./IM1,IMM1=IM1-1,
     *IA1=40014,IA2=40692,IQ1=53668,IQ2=52774,IR1=12211,IR2=3791,
     *NTAB=32,NDIV=1+IMM1/NTAB,EPS=1.2e-7,RNMX=1.-EPS)
      INTEGER idum2,j,k,iv(NTAB),iy
      SAVE iv,iy,idum2
      DATA idum2/123456789/, iv/NTAB*0/, iy/0/
      if (idum.le.0) then
        idum=max(-idum,1)
        idum2=idum
        do 11 j=NTAB+8,1,-1
          k=idum/IQ1
          idum=IA1*(idum-k*IQ1)-k*IR1
          if (idum.lt.0) idum=idum+IM1
          if (j.le.NTAB) iv(j)=idum
11      continue
        iy=iv(1)
      endif
      k=idum/IQ1
      idum=IA1*(idum-k*IQ1)-k*IR1
      if (idum.lt.0) idum=idum+IM1
      k=idum2/IQ2
      idum2=IA2*(idum2-k*IQ2)-k*IR2
      if (idum2.lt.0) idum2=idum2+IM2
      j=1+iy/NDIV
      iy=iv(j)-idum2
      iv(j)=idum
      if(iy.lt.1)iy=iy+IMM1
      ran2=min(AM*iy,RNMX)
      return
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      FUNCTION golden(ax,bx,cx,f,tol,xmin)
      REAL golden,ax,bx,cx,tol,xmin,f,R,C
      EXTERNAL f
      PARAMETER (R=.61803399,C=1.-R)
      REAL f1,f2,x0,x1,x2,x3
      x0=ax
      x3=cx
      if(abs(cx-bx).gt.abs(bx-ax))then
        x1=bx
        x2=bx+C*(cx-bx)
      else
        x2=bx
        x1=bx-C*(bx-ax)
      endif
      f1=f(x1)
      f2=f(x2)
1     if(abs(x3-x0).gt.tol*(abs(x1)+abs(x2)))then
        if(f2.lt.f1)then
          x0=x1
          x1=x2
          x2=R*x1+C*x3
          f1=f2
          f2=f(x2)
        else
          x3=x2
          x2=x1
          x1=R*x2+C*x0
          f2=f1
          f1=f(x1)
        endif
      goto 1
      endif
      if(f1.lt.f2)then
        golden=f1
        xmin=x1
      else
        golden=f2
        xmin=x2
      endif
      return
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC 
      SUBROUTINE mnbrak(ax,bx,cx,fa,fb,fc,func)
      REAL ax,bx,cx,fa,fb,fc,func,GOLD,GLIMIT,TINY
      EXTERNAL func
      PARAMETER (GOLD=1.618034, GLIMIT=100., TINY=1.e-20)
      REAL dum,fu,q,r,u,ulim
      fa=func(ax)
      fb=func(bx)
      if(fb.gt.fa)then
        dum=ax
        ax=bx
        bx=dum
        dum=fb
        fb=fa
        fa=dum
      endif
      cx=bx+GOLD*(bx-ax)
      fc=func(cx)
1     if(fb.ge.fc)then
        r=(bx-ax)*(fb-fc)
        q=(bx-cx)*(fb-fa)
        u=bx-((bx-cx)*q-(bx-ax)*r)/(2.*sign(max(abs(q-r),TINY),q-r))
        ulim=bx+GLIMIT*(cx-bx)
        if((bx-u)*(u-cx).gt.0.)then
          fu=func(u)
          if(fu.lt.fc)then
            ax=bx
            fa=fb
            bx=u
            fb=fu
            return
          else if(fu.gt.fb)then
            cx=u
            fc=fu
            return
          endif
          u=cx+GOLD*(cx-bx)
          fu=func(u)
        else if((cx-u)*(u-ulim).gt.0.)then
          fu=func(u)
          if(fu.lt.fc)then
            bx=cx
            cx=u
            u=cx+GOLD*(cx-bx)
            fb=fc
            fc=fu
            fu=func(u)
          endif
        else if((u-ulim)*(ulim-cx).ge.0.)then
          u=ulim
          fu=func(u)
        else
          u=cx+GOLD*(cx-bx)
          fu=func(u)
        endif
        ax=bx
        bx=cx
        cx=u
        fa=fb
        fb=fc
        fc=fu
        goto 1
      endif
      return
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      FUNCTION gasdev(idum)
      INTEGER idum
      REAL gasdev
CU    USES ran2!
      INTEGER iset
      REAL fac,gset,rsq,v1,v2,ran2
      SAVE iset,gset
      DATA iset/0/
      if (iset.eq.0) then
1       v1=2.*ran2(idum)-1.
        v2=2.*ran2(idum)-1.
        rsq=v1**2+v2**2
        if(rsq.ge.1..or.rsq.eq.0.)goto 1
        fac=sqrt(-2.*log(rsq)/rsq)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE indexx(n,arr,indx)
      INTEGER n,indx(n),M,NSTACK
      REAL arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,indxt,ir,itemp,j,jstack,k,l,istack(NSTACK)
      REAL a
      do 11 j=1,n
        indx(j)=j
11    continue
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 13 j=l+1,ir
          indxt=indx(j)
          a=arr(indxt)
          do 12 i=j-1,1,-1
            if(arr(indx(i)).le.a)goto 2
            indx(i+1)=indx(i)
12        continue
          i=0
2         indx(i+1)=indxt
13      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        itemp=indx(k)
        indx(k)=indx(l+1)
        indx(l+1)=itemp
        if(arr(indx(l+1)).gt.arr(indx(ir)))then
          itemp=indx(l+1)
          indx(l+1)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l)).gt.arr(indx(ir)))then
          itemp=indx(l)
          indx(l)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l+1)).gt.arr(indx(l)))then
          itemp=indx(l+1)
          indx(l+1)=indx(l)
          indx(l)=itemp
        endif
        i=l+1
        j=ir
        indxt=indx(l)
        a=arr(indxt)
3       continue
          i=i+1
        if(arr(indx(i)).lt.a)goto 3
4       continue
          j=j-1
        if(arr(indx(j)).gt.a)goto 4
        if(j.lt.i)goto 5
        itemp=indx(i)
        indx(i)=indx(j)
        indx(j)=itemp
        goto 3
5       indx(l)=indx(j)
        indx(j)=indxt
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in indexx'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                        REDDENING-RELATED ROUTINES                       +
c +                                                                         +
c +   Calc_red_term - computes -0.4 * (A(lambda)/A_V - A(l_norm)/A_V)       +
c +   Cardelli_RedLaw, Calzetti_RedLaw & HYPERZ_RedLaw                      +
c +                                                                         +
c +   To use HYPERZ_RedLaw use red_law_option = 'HZ1' ...'HZ5', where the   +
c +   number = 1 for Allen 76 MW, = 2 for Seaton 79 + Fitzpatrick 86 MW,    +
c +   = 3 for Fitzpatrick 86 LMC, = 4 for Prevot et al 84 and Bouchet et al +
c +  85 SMC & = 5 for Calzetti (astro-ph/9911459).                          +
c +                                                                         +
c +   For Gordon et al (2003, ApJ, 594, 279, Table 4), use red_law_option = +
c +   'GD1' => SMC Bar, 'GD2' => LMC2 SuperShell, 'GD3' => LMC  Average     +
c +                                                                         +
c +   The HZ1...HZ5 options are new to v02.       (Cid@Lagoa - 14/Oct/2005) + 
c +   And the GD1...GD3 options are new to v03.   (Cid@Lagoa - 20/Jan/2007) + 
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c ###########################################################################
c     Routine Calc_red_term computes the "reddening term", defined by
c
c        red_term(lambda) = -0.4 * (A(lambda) / A_V  -  A(l_norm) / A_V)
c 
c     such that to redden a flux at a certain lambda and keep the
c     normalization at l_norm all we need to do is to multiply the
c     reddening-free flux by
c
c        10**(A_V * red_term(lambda))
c
c     The A(lambda) / A_V (== q) law is from of Cardelli et al (1989) or
c     Calzetti et al. (according to red_law_option)
c 
c     Cid@INAOE - July/06/2004
c
c     + Force red_term = 0 at l_norm (it may be NE 0 due to precision
c     issues...)
c     Cid@Lagoa - 06/Jan/2005
c
c     Introduced HYPERZ laws (red_law_option = 'HZ1' ... 'HZ5')

c     OBS: I didn't quite like their sloppy interpolation at optical
c          wavelengths of the SMC law (eg, a straight line between 5291
c          & 12500 Angs), bur could find nothing better!
c
c     Cid@Lagoa - 14/October/2005

c     Introduced Gordon SMC & LMC extinction curves, plus a general
c     "from-a-file" reddening law read from a file named
c     ExtinctionLaw.red_law_option! (Not fully tested yet!!) The
c     power-law option was included and then removed after implementing
c     the from-a-file option.
c
c     Cid@Lagoa - 07/February/2007


      SUBROUTINE Calc_red_term(red_law_option,l_norm,R_V,
     &     nobs,lambda,red_term,q_norm)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)

      parameter (Nl_FromFileMax = 100000)
      real l_FromFile(Nl_FromFileMax) , q_FromFile(Nl_FromFileMax)

      real lambda(nobs) , red_term(nobs)
      real l_norm ,q_norm , R_V , q
      character*3 red_law_option

      if (red_law_option.EQ.'CCM') then 

         q_norm = Cardelli_RedLaw(l_norm,R_V)
         do i=1,nobs
            q = Cardelli_RedLaw(lambda(i),R_V)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else if (red_law_option.EQ.'CAL') then 

         q_norm = Calzetti_RedLaw(l_norm)
         do i=1,nobs
            q = Calzetti_RedLaw(lambda(i))
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else if (red_law_option.EQ.'HZ1') then 

         q_norm = HYPERZ_RedLaw(l_norm,1)
         do i=1,nobs
            q = HYPERZ_RedLaw(lambda(i),1)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else if (red_law_option.EQ.'HZ2') then 

         q_norm = HYPERZ_RedLaw(l_norm,2)
         do i=1,nobs
            q = HYPERZ_RedLaw(lambda(i),2)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else if (red_law_option.EQ.'HZ3') then 

         q_norm = HYPERZ_RedLaw(l_norm,3)
         do i=1,nobs
            q = HYPERZ_RedLaw(lambda(i),3)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else if (red_law_option.EQ.'HZ4') then 

         q_norm = HYPERZ_RedLaw(l_norm,4)
         do i=1,nobs
            q = HYPERZ_RedLaw(lambda(i),4)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else if (red_law_option.EQ.'HZ5') then 

         q_norm = HYPERZ_RedLaw(l_norm,5)
         do i=1,nobs
            q = HYPERZ_RedLaw(lambda(i),5)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

!Cid29Dec06-FIX! Introduced Gordon SMC & LMC extinction curves
      else if ((red_law_option.EQ.'GD1').OR.
     $         (red_law_option.EQ.'GD2').OR.
     $         (red_law_option.EQ.'GD3')) then

         q_norm = GordonTab4_RedLaw(l_norm,red_law_option)
         do i=1,nobs
            q = GordonTab4_RedLaw(lambda(i),red_law_option)
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

      else

!Cid01Feb07-FIX! If none of the red_law_options above worked, then we'll
!     try reading a lambda q_lambda table from a file called:
!     'ExtinctionLaw'.red_law_option and interpolate from it! If that
!     fails, stop run!  The interpolation is in x = 10000 / lambda, and
!     done in a very non-elegant fashion!
         write (*,'(a,a,a,a)')
     $        '[Calc_red_term] --> Will try reading extinction law ' , 
     $        'from file ExtinctionLaw.' , red_law_option , ' ... '

         open (unit=1,file='ExtinctionLaw.'//red_law_option,
     $        status='old',err=501)
         do j=1,Nl_FromFileMax
            read (1,*,END=502) l_FromFile(j) , q_FromFile(j)
         end do
         close(1)
         write (*,'(a)') '[Calc_red_term] OOOPS! Should not be here!'
         write (*,'(a,a)') '[Calc_red_term] ' ,
     $      'Too many points in  your ExtinctionLaw file? Run aborted!'
         stop

 501     write (*,'(a,a)') '[Calc_red_term] OOPS! Bad red_law_option: ',
     $        red_law_option
         write (*,'(a,a,a)') '[Calc_red_term] File ExtinctionLaw.' ,
     $        red_law_option , ' does not exist either!'
         write (*,'(a,a,a)') '[Calc_red_term] Run aborted!'
         stop

 502     Nl_FromFile = j - 1
         close(1)

         q_norm = 0.
         do j=1,Nl_FromFile-1
            if ((l_FromFile(j).LE.l_norm).AND.
     $           (l_norm.LE.l_FromFile(j+1))) then
               x  = 10000. / l_norm
               x1 = 10000. / l_FromFile(j)
               x2 = 10000. / l_FromFile(j+1)
               y1 = q_FromFile(j)
               y2 = q_FromFile(j+1)
               q_norm  = y1 + (y2 - y1) / (x2 - x1) * (x - x1)
            end if
         end do

         do i=1,nobs
            q = 0.
            do j=1,Nl_FromFile-1
               if ((l_FromFile(j).LE.lambda(i)).AND.
     $              (lambda(i).LE.l_FromFile(j+1))) then
                  x  = 10000. / lambda(i)
                  x1 = 10000. / l_FromFile(j)
                  x2 = 10000. / l_FromFile(j+1)
                  y1 = q_FromFile(j)
                  y2 = q_FromFile(j+1)
                  q  = y1 + (y2 - y1) / (x2 - x1) * (x - x1)
               end if
            end do
            red_term(i) = -0.4 * (q - q_norm)
            if (lambda(i).EQ.l_norm) red_term(i) = 0.
         end do

         write (*,'(a,a)') '[Calc_red_term] --> ' ,
     $        'Ok! Read Extinction Law file!'
      end if

      end
c ###########################################################################

c ###########################################################################
c     q = A_lambda / A_V for Cardelli et al reddening law
c     l = lambda, in Angstrons
c     x = 1 / lambda in 1/microns
c     q = a + b / R_V; where a = a(x) & b = b(x)
c     Cid@INAOE - 6/July/2004

      FUNCTION Cardelli_RedLaw(l,R_V)

      real l , a , b , F_a , F_b , x , y , R_V , q

      a = 0.
      b = 0.
      x = 10000. / l


c     Far-Ultraviolet: 8 <= x <= 10 ; 1000 -> 1250 Angs 
      if ((x.GE.8.).AND.(x.LE.10.)) then

         a = -1.073 - 0.628 * (x - 8.) + 0.137 * (x - 8.)**2 - 
     &        0.070 * (x - 8.)**3
         b = 13.670 + 4.257 * (x - 8.) - 0.420 * (x - 8.)**2 +
     &        0.374 * (x - 8.)**3

c     Ultraviolet: 3.3 <= x <= 8 ; 1250 -> 3030 Angs 
      else if ((x.GE.3.3).AND.(x.LE.8.)) then

         F_a = 0
         F_b = 0
         if ((x.GE.5.9).AND.(x.LE.8)) then
            F_a = -0.04473 * (x - 5.9)**2 - 0.009779 * (x - 5.9)**3
            F_b =  0.2130 * (x - 5.9)**2 + 0.1207 * (x - 5.9)**3
         end if

         a =  1.752 - 0.316 * x - 0.104 / ((x - 4.67)**2 + 0.341) + F_a
         b = -3.090 + 1.825 * x + 1.206 / ((x - 4.62)**2 + 0.263) + F_b

c     Optical/NIR: 1.1 <= x <= 3.3 ; 3030 -> 9091 Angs ; 
      else if ((x.GE.1.1).AND.(x.LE.3.3)) then

         y = 10000. / l - 1.82

         a = 1.+ 0.17699 * y - 0.50447 * y**2 - 0.02427 * y**3 + 
     &        0.72085 * y**4 + 0.01979 * y**5 - 0.77530 * y**6 + 
     &        0.32999 * y**7
         b = 1.41338 * y + 2.28305 * y**2 + 1.07233 * y**3 - 
     &        5.38434 * y**4 - 0.62251 * y**5 + 5.30260 * y**6 -
     &        2.09002 * y**7  


c     Infrared: 0.3 <= x <= 1.1 ; 9091 -> 33333 Angs ; 
      else if ((x.GE.0.3).AND.(x.LE.1.1)) then

         a =  0.574 * x**1.61
         b = -0.527 * x**1.61

      end if


      q = a + b / R_V


c     Issue a warning if lambda falls outside 1000 -> 33333 Angs range of CCM89
      if ((x.LT.0.3).OR.(x.GT.10.)) then
         q = 0.
!         write (*,*) '[Cardelli_RedLaw] WARNING! ' , 
!     &        'lambda outside valid range: ' , l , x
      end if


      Cardelli_RedLaw = q

      return
      end
c ###########################################################################

c ###########################################################################
c     q = A_lambda / A_V for Calzetti et al reddening law (formula from
c     hyperz-manual).
c     l = lambda, in Angstrons
c     x = 1 / lambda in 1/microns
c     Cid@INAOE - 6/July/2004

      FUNCTION Calzetti_RedLaw(l)

      real l , x , R_V , q

      x   = 10000. / l
      R_V = 4.05


c     UV -> Optical: 1200 -> 6300 Angs
      if ((l.GE.1200.).AND.(l.LE.6300.)) then

         q = (2.659 / R_V) * (-2.156 + 1.509 * x - 0.198 * x**2 + 
     &        0.011 * x**3) + 1.

c     Red -> Infrared
      else if ((l.GE.6300.).AND.(l.LE.22000.)) then

         q = (2.659 / R_V) * (-1.857 + 1.040 * x) + 1.

c     Issue a warning if lambda falls outside 1200->22000 Angs range
      else 

         q = 0.
!         write (*,*) '[Calzetti_RedLaw] WARNING! ' , 
!     &        'lambda outside valid range: ' , l , x

      end if


      Calzetti_RedLaw = q

      return
      end
c ###########################################################################

c ###########################################################################
c     q = A_lambda / A_V for one of the 5 reddening laws in HYPERZ.
c     This is just a sloppy DRIVER for the HYPERZ fortran routine red_spec!
c     I call their routine with a wavelength array containing just one
c     lambda (l_aux(1) = l), and tell it to return me the reddened
c     spectrum f_obs_aux(1) for an intrinsic spectrum f_int_aux(1) = 1.0
c     and A_V = 1. From this we derive q = -2.5 * log10(f_obs_aux(1)).
c
c     l = lambda, in Angstrons
c     ilaw_rd=1 : the standard variation of absorption with wavelength 
c                 is used (Allen, 76) - MW
c     ilaw_rd=2 : the analytic fit of Seaton (79) curve by 
c                 Fitzpatrick (86) - MW 
c     ilaw_rd=3 : Fitzpatrick (86) - LMC
c     ilaw_rd=4 : Prevot et al. (1984) and Bouchet et al. (1985) - SMC
c     ilaw_rd=5 : Calzetti (astro-ph/9911459) - SB
c
c     Cid@Lagoa - 14/October/2005

      FUNCTION HYPERZ_RedLaw(l,ilaw_rd)

      real l_aux(1) , f_int_aux(1) , f_obs_aux(1)
      real l , q

      l_aux(1)     = l
      f_int_aux(1) = 1.
      call HYPERZ_red_spec(l_aux,f_int_aux,1.0,f_obs_aux,1,1,ilaw_rd)
      q = -2.5 * log10(f_obs_aux(1))

      HYPERZ_RedLaw = q

      return
      end
c ###########################################################################


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     This was "borrowed" from hyperz @ webast.ast.obs-mip.fr/hyperz/
C     I've just copied their red_spec.f (in zphot_src_1.1.tar.gz). I
C     have, though fixed their 'if lambda is in an interval' conditions,
C     as marked below.
C     Cid@Lagoa - 14/October/2005


      SUBROUTINE HYPERZ_red_spec(wl,fl,av,yob,ired,iwavl,law_rd)
c     Reads a spectrum and calculates the new one with internal
c     reddening. 
c     law_rd=1 : the standard variation of absorption with wavelength 
c                is used (Allen, 76) - MW
c     law_rd=2 : the analytic fit of Seaton (79) curve by 
c                Fitzpatrick (86) - MW 
c     law_rd=3 : Fitzpatrick (86) - LMC
c     law_rd=4 : Prevot et al. (1984) and Bouchet et al. (1985) - SMC
c     law_rd=5 : Calzetti (astro-ph/9911459) - SB
c    

!Cid! INCLUDE 'dimension.dec' I've included it here:)
      PARAMETER (mxz=141,mxage=51,mxtyp=15,mxfil=20,mxred=11,mxwl=1300)

      INTEGER ired,iwavl,law_rd
      REAL yob,alambda,wl,fl,wab,amab,aab,av,ff,ff11,ff12,slope,rv
      DIMENSION yob(mxwl),wl(mxwl),fl(mxwl),wab(30),amab(30),aab(30)

c ..... 1. Allen (1976) : wab = lambda in A, amab = k(lambda)/R, 
c          A(lambda) = k(lambda)*E(B-V) = amab*A_V ; R = 3.1

      IF (law_rd.eq.1) THEN
         rv=3.1
         wab(1)=1000.
         amab(1)=4.2
         wab(2)=1110.
         amab(2)=3.7
         wab(3)=1250.
         amab(3)=3.3
         wab(4)=1430.
         amab(4)=3.
         wab(5)=1670.
         amab(5)=2.7
         wab(6)=2000.
         amab(6)=2.8
         wab(7)=2220.
         amab(7)=2.9
         wab(8)=2500.
         amab(8)=2.30
         wab(9)=2850.
         amab(9)=1.97
         wab(10)=3330.
         amab(10)=1.69
         wab(11)=3650.
         amab(11)=1.58
         wab(12)=4000.
         amab(12)=1.45
         wab(13)=4400.
         amab(13)=1.32
         wab(14)=5000.
         amab(14)=1.13
         wab(15)=5530.
         amab(15)=1.00
         wab(16)=6700.
         amab(16)=0.74
         wab(17)=9000.
         amab(17)=0.46
         wab(18)=10000.
         amab(18)=0.38
         wab(19)=20000.
         amab(19)=0.11
         wab(20)=100000.
         amab(20)=0.0
         DO i=1,19
            aab(i)=amab(i)*av
         END DO
         DO ik=1,iwavl
            IF (wl(ik).le.wab(1)) then 
               ccc=(aab(1)-aab(2))/(wab(2)-wab(1))
               alambda=aab(1)+(wab(1)-wl(ik))*ccc
            ELSE IF(wl(ik).ge.wab(19))then 
               ccc=(aab(19)-aab(20))/(wab(20)-wab(19))
               alambda=aab(19)+(wab(19)-wl(ik))*ccc
            ELSE IF (wl(ik).gt.wab(1).and.wl(ik).lt.wab(19)) then
               DO i=1,19
!Cid-fix!         IF (wl(ik).ge.wab(i).and.wl(ik).lt.wab(i+1)) then
                  IF (wl(ik).ge.wab(i).and.wl(ik).le.wab(i+1)) then
                     ccc=(aab(i)-aab(i+1))/(wab(i+1)-wab(i))
                     alambda=aab(i)+(wab(i)-wl(ik))*ccc
                  END IF
               END DO
            END IF
            IF (alambda.lt.0.) alambda=0.
            IF (ired.eq.1) yob(ik)=fl(ik)*(10**(-0.4*alambda))
            IF (ired.eq.2) yob(ik)=fl(ik)*(10**(0.4*alambda))
         END DO
         RETURN

c ..... 2. Seaton (1979) - Fitzpatrick fit : 
c                       for lambda < 1200 A slope from 1100-1200
c                       for lambda > 3650 A points from Allen
c                       ff = E(lambda-V)/E(B-V) = k(lambda)-R
c                       A(lambda)=A_V*(ff/R +1) ; R = 3.1

      ELSE IF (law_rd.eq.2) THEN
         rv=3.1
         al0=4.595
         ga=1.051
         c1=-0.38
         c2=0.74
         c3=3.96
         c4=0.26
         call red(1100.,al0,ga,c1,c2,c3,c4,ff)
         ff11=ff
         call red(1200.,al0,ga,c1,c2,c3,c4,ff)
         ff12=ff
         slope=(ff12-ff11)/100.
         wab(11)=3650.
         amab(11)=1.58
         wab(12)=4000.
         amab(12)=1.45
         wab(13)=4400.
         amab(13)=1.32
         wab(14)=5000.
         amab(14)=1.13
         wab(15)=5530.
         amab(15)=1.00
         wab(16)=6700.
         amab(16)=0.74
         wab(17)=9000.
         amab(17)=0.46
         wab(18)=10000.
         amab(18)=0.38
         wab(19)=20000.
         amab(19)=0.11
         wab(20)=100000.
         amab(20)=0.0
         DO i=11,19
            aab(i)=amab(i)*av
         END DO
         DO ik=1,iwavl
            IF (wl(ik).ge.1200..and.wl(ik).le.3650.) then
               call red(wl(ik),al0,ga,c1,c2,c3,c4,ff)
               alambda=av*(ff/rv+1.) 
            ELSE IF (wl(ik).lt.1200.) then
               ff=ff11+(wl(ik)-1100.)*slope
               alambda=av*(ff/rv+1.)
            ELSE IF(wl(ik).ge.wab(19))then !extrapol at high wavelengths
               ccc=(aab(19)-aab(20))/(wab(20)-wab(19))
               alambda=aab(19)+(wab(19)-wl(ik))*ccc
            ELSE IF (wl(ik).gt.3650..and.wl(ik).lt.wab(19)) then
               DO i=11,19
!Cid-fix!         IF (wl(ik).gt.wab(i).and.wl(ik).lt.wab(i+1)) then
                  IF (wl(ik).ge.wab(i).and.wl(ik).le.wab(i+1)) then
                     ccc=(aab(i)-aab(i+1))/(wab(i+1)-wab(i))
                     alambda=aab(i)+(wab(i)-wl(ik))*ccc
                  END IF
               END DO
            END IF
            IF (alambda.lt.0.) alambda=0.
            IF (ired.eq.1) yob(ik)=fl(ik)*(10**(-0.4*alambda))
            IF (ired.eq.2) yob(ik)=fl(ik)*(10**(0.4*alambda))
         END DO
         RETURN
      
c ..... 3. Fitzpatrick (1986) : 
c                       for lambda < 1200 A slope derived from 1100-1200
c                       for lambda > 3330 A points from Allen
c                       ff = E(lambda-V)/E(B-V) = k(lambda)-R
c                       A(lambda) = A_V*(ff/R +1) ; R = 3.1

      ELSE IF (law_rd.eq.3) THEN
         rv=3.1
         al0=4.608
         ga=0.994
         c1=-0.69
         c2=0.89
         c3=2.55
         c4=0.50
         call red(1100.,al0,ga,c1,c2,c3,c4,ff)
         ff11=ff
         call red(1200.,al0,ga,c1,c2,c3,c4,ff)
         ff12=ff
         slope=(ff12-ff11)/100.
         wab(10)=3330.
         amab(10)=1.682
         wab(11)=3650.
         amab(11)=1.58
         wab(12)=4000.
         amab(12)=1.45
         wab(13)=4400.
         amab(13)=1.32
         wab(14)=5000.
         amab(14)=1.13
         wab(15)=5530.
         amab(15)=1.00
         wab(16)=6700.
         amab(16)=0.74
         wab(17)=9000.
         amab(17)=0.46
         wab(18)=10000.
         amab(18)=0.38
         wab(19)=20000.
         amab(19)=0.11
         wab(20)=100000.
         amab(20)=0.0
         DO i=10,19
            aab(i)=amab(i)*av
         END DO
         DO ik=1,iwavl
            IF (wl(ik).ge.1200..and.wl(ik).le.3330.) then
               call red(wl(ik),al0,ga,c1,c2,c3,c4,ff)
               alambda=av*(ff/rv+1.)
            ELSE IF (wl(ik).lt.1200.) then
               ff=ff11+(wl(ik)-1100.)*slope
               alambda=av*(ff/rv+1.)
            ELSE IF(wl(ik).ge.wab(19))then !extrapol at high wavelengths
               ccc=(aab(19)-aab(20))/(wab(20)-wab(19))
               alambda=aab(19)+(wab(19)-wl(ik))*ccc
            ELSE IF (wl(ik).gt.3330..and.wl(ik).lt.wab(19)) then
               DO i=10,19
!Cid-fix!         IF (wl(ik).gt.wab(i).and.wl(ik).lt.wab(i+1)) then
                  IF (wl(ik).ge.wab(i).and.wl(ik).le.wab(i+1)) then
                     ccc=(aab(i)-aab(i+1))/(wab(i+1)-wab(i))
                     alambda=aab(i)+(wab(i)-wl(ik))*ccc
                  END IF
               END DO
            END IF
            IF (alambda.lt.0.) alambda=0.
            IF (ired.eq.1) yob(ik)=fl(ik)*(10**(-0.4*alambda))
            IF (ired.eq.2) yob(ik)=fl(ik)*(10**(0.4*alambda))
         END DO
         RETURN

c ..... 4. Prevot (1984) and Bouchet (1985) :
c                        wab = lambda in A,  
c                        amab = E(lambda-V)/E(B-V) = k(lambda)-R
c                        A(lambda) = A_V*(amab/R +1)
c                        N.B. R = 2.72

         ELSE IF (law_rd.eq.4) THEN
            rv=2.72
            wab(1)=1275.
            amab(1)=13.54
            wab(2)=1330.
            amab(2)=12.52
            wab(3)=1385.
            amab(3)=11.51
            wab(4)=1435.
            amab(4)=10.80
            wab(5)=1490.
            amab(5)=9.84
            wab(6)=1545.
            amab(6)=9.28
            wab(7)=1595.
            amab(7)=9.06
            wab(8)=1647.
            amab(8)=8.49
            wab(9)=1700.
            amab(9)=8.01
            wab(10)=1755.
            amab(10)=7.71
            wab(11)=1810.
            amab(11)=7.17
            wab(12)=1860.
            amab(12)=6.90
            wab(13)=1910.
            amab(13)=6.76
            wab(14)=2000.
            amab(14)=6.38
            wab(15)=2115.
            amab(15)=5.85
            wab(16)=2220.
            amab(16)=5.30
            wab(17)=2335.
            amab(17)=4.53
            wab(18)=2445.
            amab(18)=4.24
            wab(19)=2550.
            amab(19)=3.91
            wab(20)=2665.
            amab(20)=3.49
            wab(21)=2778.
            amab(21)=3.15
            wab(22)=2890.
            amab(22)=3.00
            wab(23)=2995.
            amab(23)=2.65
            wab(24)=3105.
            amab(24)=2.29
            wab(25)=3704.
            amab(25)=1.81
            wab(26)=4255.
            amab(26)=1.00
            wab(27)=5291.
            amab(27)=0.00
            wab(28)=12500.
            amab(28)=-2.02
            wab(29)=16500.
            amab(29)=-2.36
            wab(30)=22000.
            amab(30)=-2.47

            DO i=1,30
               aab(i)=av*(amab(i)/rv+1.)
            END DO

            DO ik=1,iwavl
               IF (wl(ik).le.wab(1)) THEN !extrapol at short wavelengths
                  ccc=(aab(1)-aab(2))/(wab(2)-wab(1))
                  alambda=aab(1)+(wab(1)-wl(ik))*ccc
               ELSE IF(wl(ik).ge.wab(30))then !extrapol at high wavelengths
                  ccc=(aab(29)-aab(30))/(wab(30)-wab(29))
                  alambda=aab(29)+(wab(29)-wl(ik))*ccc  
               ELSE IF (wl(ik).gt.wab(1).and.wl(ik).lt.wab(30)) THEN
                  DO i=1,30
!Cid-fix!            IF (wl(ik).gt.wab(i).and.wl(ik).lt.wab(i+1)) THEN
                     IF (wl(ik).ge.wab(i).and.wl(ik).le.wab(i+1)) THEN
                        ccc=(aab(i)-aab(i+1))/(wab(i+1)-wab(i))
                        alambda=aab(i)+(wab(i)-wl(ik))*ccc
                     END IF
                  END DO
               END IF   
               IF (alambda.lt.0.) alambda=0.
               IF (ired.eq.1) yob(ik)=fl(ik)*(10**(-0.4*alambda))
               IF (ired.eq.2) yob(ik)=fl(ik)*(10**(0.4*alambda))
            END DO
            RETURN


c ..... Calzetti (1999) : 
c                   for lambda < 1200 A slope derived from 1100-1200 A
c                   for lambda > 22000A slope derived from 21900-22000 A
c                   ff = k(lambda); A(lambda) = A_V*k(lambda)/R ; R = 4.05

      ELSE IF (law_rd.eq.5) THEN
         rv=4.05
         p11=1/0.11
         ff11=2.659*(-2.156+1.509*p11-0.198*p11**2
     .        +0.011*p11**3)+rv
         p12=1/0.12
         ff12=2.659*(-2.156+1.509*p12-0.198*p12**2
     .        +0.011*p12**3)+rv
         slope1=(ff12-ff11)/100.
         ff99=2.659*(-1.857+1.040/2.19)+rv
         ff100=2.659*(-1.857+1.040/2.2)+rv
         slope2=(ff100-ff99)/100.
         DO ik=1,iwavl
            ala=wl(ik)*1.E-4    !wavelength in microns
            p=1./ala
            IF (ala.ge.0.63.and.ala.le.2.2) then
               ff=2.659*(-1.857+1.040*p)+rv
            ELSE IF (ala.ge.0.12.and.ala.lt.0.63) then
               ff=2.659*(-2.156+1.509*p-0.198*p**2+0.011*p**3)+rv
            ELSE IF (ala.lt.0.12) THEN
               ff=ff11+(wl(ik)-1100.)*slope1
            ELSE IF (ala.gt.2.2) THEN
               ff=ff99+(wl(ik)-21900.)*slope2
            END IF
            alambda=av*ff/rv
            IF (alambda.lt.0.) alambda=0.
            IF (ired.eq.1) yob(ik)=fl(ik)*(10**(-0.4*alambda))
            IF (ired.eq.2) yob(ik)=fl(ik)*(10**(0.4*alambda))
         END DO
         RETURN
      END IF

      WRITE(*,*) ' No reddening law is given.....'
      RETURN
      END

c ----------------------------------------------------------------------
      SUBROUTINE red(alan,al0,ga,c1,c2,c3,c4,ff)

c.......According to Fitzpatrick, 86

      ala=alan*1.E-4           !wavelength in microns
      p=1.0/ala
      IF (p.lt.5.9) c4=0.0
      t1=c3/(((p-(al0**2/p))**2)+ga*ga)
      t2=c4*(0.539*((p-5.9)**2)+0.0564*((p-5.9)**3))
      ff=c1+c2*p+t1+t2
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c ###########################################################################
c     Gordon SMC & LMC extinction curves
c     After failing to implement Gordon's LMC & SMC laws in analytical
c     form (their equation 5, with parameters from Table 3 and R_V from
c     Sec 3.4 & Table 2), I have just copied their Table 4 and
c     interpolate linearly (in x).
c
c     q = A_lambda / A_V from Gordon et al 2003, ApJ, 594, 279, Table 4.
c     l = lambda, in Angstrons
c     x = 1 / lambda in 1/microns
c     red_law_option = 'GD1' => SMC Bar
c                    = 'GD2' => LMC2 SuperShell
c                    = 'GD3' => LMC Average
c
c     Cid@Lagoa - 30/December/2006

      FUNCTION GordonTab4_RedLaw(l,red_law_option)

      implicit real (a-h,j-m,o-z)
      implicit integer (i,n)
      parameter (N_Gordon = 29)
      real x_Gordon(N_Gordon) , q_Gordon(N_Gordon)
      real q_SMCbar(N_Gordon) , q_LMC2(N_Gordon) , q_LMCave(N_Gordon)
      character*3 red_law_option

c     Table 4 of Gordon et al (Sample Average Curves)
c     OBS: The Table has 2 empty entries for LMC2 & LMCave, at x = 1.235
c     & 1.538. These entries were interpolated linearly in the
c     data-blocks below.
      data x_Gordon/ 0.455 , 0.606 , 0.800 , 1.235 , 1.538 , 1.818 , 
     $       2.273 , 2.703 , 3.375 , 3.625 , 3.875 , 4.125 , 4.375 ,
     $       4.625 , 4.875 , 5.125 , 5.375 , 5.625 , 5.875 , 6.125 ,
     $       6.375 , 6.625 , 6.875 , 7.125 , 7.375 , 7.625 , 7.875 , 
     $       8.125 , 8.375 /
      data q_SMCbar/ 0.016 , 0.169 , 0.131 , 0.567 , 0.801 , 1.000 , 
     $       1.374 , 1.672 , 2.000 , 2.220 , 2.428 , 2.661 , 2.947 , 
     $       3.161 , 3.293 , 3.489 , 3.637 , 3.866 , 4.013 , 4.243 , 
     $       4.472 , 4.776 , 5.000 , 5.272 , 5.575 , 5.795 , 6.074 , 
     $       6.297 , 6.436 /
      data q_LMC2/   0.101 , 0.097 , 0.299 , 0.5004, 0.7600, 1.000 , 
     $       1.349 , 1.665 , 1.899 , 2.067 , 2.249 , 2.447 , 2.777 , 
     $       2.922 , 2.921 , 2.812 , 2.805 , 2.863 , 2.932 , 3.060 , 
     $       3.110 , 3.299 , 3.408 , 3.515 , 3.670 , 3.862 , 3.937 , 
     $       4.055 , 3.969 /
      data q_LMCave/ 0.030 , 0.186 , 0.257 , 0.4705, 0.7457, 1.000 ,
     $       1.293 , 1.518 , 1.786 , 1.969 , 2.149 , 2.391 , 2.771 ,
     $       2.967 , 2.846 , 2.646 , 2.565 , 2.566 , 2.598 , 2.607 ,
     $       2.668 , 2.787 , 2.874 , 2.983 , 3.118 , 3.231 , 3.374 ,
     $       3.366 , 3.467 /

c     Define x & store appropriate q onto q_Gordon
      x = 10000. / l
      do i=1,N_Gordon
         if (red_law_option.EQ.'GD1') then
            q_Gordon(i) = q_SMCbar(i)
         else if (red_law_option.EQ.'GD2') then
            q_Gordon(i) = q_LMC2(i)
         else if (red_law_option.EQ.'GD3') then
            q_Gordon(i) = q_LMCave(i)
         else
            write (*,*) 'OOPS! Bad red_law_option = ',red_law_option
            stop
         end if
      end do

c     Linear interpolation of q = a x + b
      do i=1,N_Gordon-1
         if ((x.GE.x_Gordon(i)).AND.(x.LE.x_Gordon(i+1))) then
            q = q_Gordon(i) + (q_Gordon(i+1) - q_Gordon(i)) * 
     $           (x - x_Gordon(i)) / (x_Gordon(i+1) - x_Gordon(i))
            GordonTab4_RedLaw = q
            return
         end if
      end do

c     If lambda is outside Table 4 range, return zero,
      GordonTab4_RedLaw = 0.

      return
      end
c ###########################################################################
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +        SYNTHETIC SPECTRUM, CHI2 & RAPID-CHI2 ("RC") ROUTINES            +
c +                                                                         +
c + calc_Fsyn                                                               +
c + Kcalc_Fsyn                                                              +
c + Slow_Fchi2                                                              +
c + FK_chi2                                                                 +
c + Calc_RC_terms                                                           +
c + VeryFastF_chi2                                                          +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+


c ###########################################################################
c     Compute synthetic spectrum f_syn(l_syn) for given population &
c     extinction vectors, x(j) & AV_tot(j). Stellar kinematics (if any)
c     is inside the f_base matrix (as set elsewhere)!
c     OBS: Reddening term, base matrix & Nl_obs passed by common!  
c     Cid@Lagoa - Jan/11/2005

      SUBROUTINE calc_Fsyn(x,AV_tot,f_syn)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real x(Nb_max) , AV_tot(Nb_max)
      real f_base(Nb_max,Nl_max) , f_syn(Nl_max)

      common/Red_factor/ red_term(Nl_max)
      common/Base/ N_base , f_base
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs

      do i=1,Nl_obs
         f_syn(i) = 0.
         do j=1,N_base
            f_syn(i) = f_syn(i) + f_base(j,i) * abs(x(j)) * 
     &           10.**(AV_tot(j) * red_term(i))
         end do
      end do

      end
c ###########################################################################



c ###########################################################################
c     Compute synthetic spectrum f_syn(l_syn) for given population &
c     extinction vectors, x(j) & AV_tot(j), AND kinematical parameters
c     v0 & vd. A synthetic spectrum f(x,AV_tot) is 1st computed & then
c     convolved with a Gaussian of velocity center = v0 & velocity
c     dispersion = vd (both in km/s).  
c     Cid@Lagoa - Jan/11/2005

      SUBROUTINE Kcalc_Fsyn(Nl_syn,x,AV_tot,v0,vd,f_syn)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real l_aux(Nl_max) , f_aux(Nl_max) , f_syn(Nl_max)
      real x(Nb_max) , AV_tot(Nb_max)

      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/N_Gauss/ N_int_Gauss

!Cid14Jan07-FXK! Added FXK = Fixed kinematics option!!
      common/FXK_fits/ IsThisAnFXKFit

!Cid14Jan07-FXK! If this is a FXK-fit then we should NOT apply the
!     kinematical filter, as it has already been applied to the base!!
!     Just call calc_Fsyn & return.
      if (IsThisAnFXKFit.EQ.1) then
         call calc_Fsyn(x,AV_tot,f_syn)
         return
      end if

c     Synthetic spectrum f_aux. Also define l_aux (used below). It is
c     presumed that the base spectra in f_base are in their original
c     form (ie, not convolved with a kinematical filter)!!
      call calc_Fsyn(x,AV_tot,f_aux)

c     Apply kinematical filter! (aux variables used to avoid repeating
c     variable names in the call to GaussSmooth). Returns the convolved
c     spectrum f_syn.
      Nl_aux = Nl_syn
      do i=1,Nl_syn
         l_aux(i) = l_obs(i)
      end do
      call GaussSmooth(Nl_aux,l_aux,f_aux,v0,vd,
     &     N_int_Gauss,Nl_syn,l_obs,f_syn)


      end
c ###########################################################################



c ###########################################################################
c     Compute chi2(x,AV_tot). Kinematical parameters are implicit in f_base!
c     I now call this function "Slow" because it is slow compared to the
c     Rapid-Chi2 tricks developed. Most of the code used rapid-chi2
c     functions.  This one is only used to check results.
c     Cid@Lagoa - Jan/11/2005

      FUNCTION SlowF_chi2(x,AV_tot)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real x(Nb_max) , AV_tot(Nb_max)
      real f_base(Nb_max,Nl_max) , f_syn(Nl_max)

      common/Red_factor/ red_term(Nl_max)
      common/Base/ N_base , f_base
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs

      call calc_Fsyn(x,AV_tot,f_syn)

      chi2 = 0.
      do i=1,Nl_obs
         chi2 = chi2 + (f_syn(i) - f_obs(i)) * weight_obs(i) *
     &        (f_syn(i) - f_obs(i)) * weight_obs(i)
      end do
      SlowF_chi2 = chi2

      return
      end
c ###########################################################################



c ###########################################################################
c     Compute chi2(x,AV_tot,v0,vd), INCLUDING kinematical parameters!
c     This a "Slow" function, ie, no RC-tricks are used. In fact, it is
c     Very slow, since it convolves the spectrum with the G(v0,vd)
c     filter...
c     Cid@Lagoa - Jan/11/2005

      FUNCTION FK_chi2(x,AV_tot,v0,vd)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real x(Nb_max) , AV_tot(Nb_max) , f_syn(Nl_max)

      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs

      call Kcalc_Fsyn(Nl_obs,x,AV_tot,v0,vd,f_syn)

      chi2 = 0.
      do i=1,Nl_obs
         chi2 = chi2 + (f_syn(i) - f_obs(i)) * weight_obs(i) *
     &        (f_syn(i) - f_obs(i)) * weight_obs(i)
      end do
      FK_chi2 = chi2

      end
c ###########################################################################



c ###########################################################################
c     Compute RC-terms. These are the terms in chi2 which do NOT depend
c     on either x, AV nor YAV, and thus do not be recomputed every time
c     x, AV or YAV changes, allowing enormous speed gains! This trick
c     requires expanding the 10**(AV_tot(j) * red_term(lambda)) factor
c     in a series and truncating it to its first NRC_AV terms, which is
c     Ok as long as |AV| does not go much beyond ~ 1... The first term
c     (RC1) is always the same, whereas the 2nd and 3rd terms (RC2 &
c     RC3) need to be re-computed everytime something was done to v0
c     and/or vd.... The VeryFastF_chi2 routine below use RC1, RC2 & RC3,
c     as well as NRC_AV (the number of terms in the truncated
c     AV-series). See notes for more details!
c
c     This routine can be made faster by introducing some "memory" so
c     that only new terms in the AV-series need to be computed. However,
c     this should NOT be useful, since I expect it to be called mostly
c     when a full-recalculation is needed (eg, when f_base
c     changes). Re-calls to the routine due to precision bugs should be
c     only occasional. Hence I did not attempt to optimize it further.
c
c     Cid@Lagoa  - 9/May/2004

c     OBS: Some of the initial steps, like the definition of the
c     factorial and beta arrays, could be done only once in the code,
c     but they take so little time that I kept them here for
c     conciseness...

c     Now adapted to AV = AV_tot(j), ie, population dependent
c     extinctions!
c     Cid@Lagoa - 12/Jan/2005


      SUBROUTINE Calc_RC_terms


c ***************************************************************************
c *                                Definitions                              *
c ***************************************************************************
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400,NRCAV_max = 20)

      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real f_base(Nb_max,Nl_max) 

      double precision RC1 , RC2(NRCAV_max,Nb_max) , 
     &     RC3(NRCAV_max,Nb_max,Nb_max) , eta_AV
      double precision factorial(NRCAV_max) , rho(Nl_max) , beta(Nl_max)
      double precision soma , ln10

      common/Red_factor/ red_term(Nl_max)
      common/Base/ N_base , f_base
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/RapidChi2/ RC1 , RC2 , RC3 , eta_AV
      common/RC_AV_Stuff/ NRC_AV , i_RC_CRASH_FLAG , NRC_AV_Default
c ***************************************************************************


c ***************************************************************************
c *                              Initialization                             *
c ***************************************************************************
c     Test if NRC_AV exceed its limit! If so then set a CRASH_FLAG!
c     OBS: Once i_RC_CRASH_FLAG = 1, it'll stay so till the end of the
c          code! If a "crash" occurs we set NRC_AV = NRCAV_max and go on...
      if (NRC_AV.GT.NRCAV_max) then 
         write (*,9) 
 9       format (55('@'))
         write (*,*) '[calc_RC_terms] SERIOUS BUG! NRC_AV > NRCAV_max ',
     &        NRC_AV , NRCAV_max
         write (*,9) 
         NRC_AV =  NRCAV_max
         i_RC_CRASH_FLAG = 1
      end if


c     Define factorial of 0...NRC_AV-1. 
c     Note array off-set: factorial(i) = (i-1)!
      factorial(1) = 1.
      factorial(2) = 1.
      do i=3,NRC_AV
         factorial(i) = factorial(i-1) * (i - 1)
      end do


c     Define rho(lambda) = ln(10) * red_term(lambda) and compute its
c     mean.  No weighting is applied. This mean serves as the factor
c     "eta_AV".
      ln10 = log(10.)
      soma = 0.
      do i=1,Nl_obs
         rho(i)  = ln10 * red_term(i)
         soma = soma + rho(i)
      end do
      eta_AV = soma / Nl_obs


c     Define beta(lambda) = rho(lambda) - eta_AV
      do i=1,Nl_obs
         beta(i) = rho(i) - eta_AV
      end do
c ***************************************************************************


c ***************************************************************************
c *                    Compute RC terms RC1, RC2 & RC3                      *
c ***************************************************************************
c     RC1 = Sum-over-l  (O_l * w_l)**2. 
      RC1 = 0.
      do i=1,Nl_obs
         RC1 = RC1 + f_obs(i) * weight_obs(i) * f_obs(i) * weight_obs(i)
      end do


c     RC2 = -2 * Sum-over-l w_l**2 * O_l * gamma_l,j * beta_l**n
c     RC2 = RC2(iRC_AV,j)
      do j=1,N_base
         do iRC_AV=1,NRC_AV
            soma = 0.
            do i=1,Nl_obs
               soma = soma + f_obs(i) * weight_obs(i) * f_base(j,i) * 
     &              weight_obs(i) * beta(i)**(iRC_AV - 1)
            end do
            RC2(iRC_AV,j) = -(2 * soma) / factorial(iRC_AV)

! Screen check
!           write (*,2) '[calc_RC_terms] RC2: ' , j , iRC_AV , 
!    &           RC2(iRC_AV,j)
!2          format (a,2(2x,i4),3x,1p,3(4x,e12.4))
         end do
      end do


c     RC3 = Sum-over-l w_l**2 * gamma_l,j1 * gamma_l,j2 * beta_l**n
c     RC3 = RC3(iRC_AV,j1,j2)
c     OBS: Explore fact that RC3(iRC_AV,j1,j2) = RC3(iRC_AV,j2,j1)...
      do j1=1,N_base
         do j2=1,j1
            do iRC_AV=1,NRC_AV
               soma = 0.
               do i=1,Nl_obs
                  soma = soma + f_base(j1,i) * weight_obs(i) * 
     &                 f_base(j2,i) * weight_obs(i) * 
     &                 beta(i)**(iRC_AV - 1)
               end do
               RC3(iRC_AV,j1,j2) = soma / factorial(iRC_AV)
               RC3(iRC_AV,j2,j1) = RC3(iRC_AV,j1,j2)

! Screen check
!           write (*,3) '[calc_RC_terms] RC3: ' , j1 , j2 , iRC_AV ,
!    &           RC3(iRC_AV,j1,j2)
!3           format (a,3(2x,i4),3x,1p,e12.4)
            end do
         end do
      end do
c ***************************************************************************


      end
c ###########################################################################



c ###########################################################################
c     This version of chi2(x,AV_tot) uses the information in the flag
c     i_IsAVstillTheSame, which is = 1 ("YES") if AV is the same as in
c     the last call to this function. This allows us to skip the sum
c     over the NRC_AV terms in the truncated-series approximation to
c     chi2, thus saving quite a bit of time, hence allowing many more
c     Metropolis steps! See notes for more details!
c     Cid@Lagoa - 09/May/2004
c
c     Now adapted to AV = AV_tot(j), ie, population dependent
c     extinctions! Double precision variables were defined too!
c     Cid@Lagoa - 12/Jan/2005

      FUNCTION VeryFastF_chi2(x,AV_tot,i_IsAVstillTheSame)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400,NRCAV_max = 20)

      real x(Nb_max) , abs_x(Nb_max) , AV_tot(Nb_max)
      real f_base(Nb_max,Nl_max)
      double precision RC1 , RC2(NRCAV_max,Nb_max) , 
     &     RC3(NRCAV_max,Nb_max,Nb_max) , eta_AV

      double precision SMAT1(Nb_max) , SMAT2(Nb_max,Nb_max)
      double precision chi2 , chi2_1 , chi2_2 , chi2_3 , soma , soma_j2

      SAVE SMAT1 , SMAT2

      common/Base/ N_base , f_base
      common/RapidChi2/ RC1 , RC2 , RC3 , eta_AV
      common/RC_AV_Stuff/ NRC_AV , i_RC_CRASH_FLAG , NRC_AV_Default


c     Define auxiliar i_IsAVstillTheSame flag, which may be changed in
c     case of trouble (see bottom)
      i_aux_IsAVstillTheSame = i_IsAVstillTheSame


c     1st term = Sum_l (w_l O_l)**2
 10   chi2_1 = RC1 

c     If AV is NOT the same as in the last call then compute chi2 as in
c     the fast-chi2 routine, but saving the SMAT1 & SMAT2 arrays for
c     later use
      if (i_aux_IsAVstillTheSame.NE.1) then 


c     2nd term = -2 Sum_l w_l**2 O_l * M_l. 
c     Also, define abs_x() for convenience (= speed), as it's used below.
         chi2_2 = 0.
         do j=1,N_base
            abs_x(j) = abs(x(j))
            soma = 0.
            do iRC_AV=1,NRC_AV
               soma = soma + AV_tot(j)**(iRC_AV - 1) * RC2(iRC_AV,j)
            end do
            SMAT1(j) = soma * exp(eta_AV * AV_tot(j))
            chi2_2   = chi2_2 + abs_x(j) * SMAT1(j)
         end do


c     3rd term = Sum_l (w_l M_l)**2.
c     To speed up the triple loop, first sum over the off-diagnonal
c     elements j2 < j1, then use the (j1,j2) = (j2,j1) simmetry in RC3
c     (hence the chi2_3 = 2 * chi2_3 line below), and finally sum over
c     the j1 = j2 diagonal elements.
         chi2_3 = 0.
         do j1=1,N_base
            soma_j2 = 0.
            do j2=1,j1-1
               AV_aux = AV_tot(j1) + AV_tot(j2)
               soma   = 0.
               do iRC_AV=1,NRC_AV
                  soma = soma + AV_aux**(iRC_AV - 1) * RC3(iRC_AV,j1,j2)
               end do
               SMAT2(j1,j2) = soma * exp(eta_AV * AV_aux)
               SMAT2(j2,j1) = SMAT2(j1,j2)
               soma_j2      = soma_j2 + abs_x(j2) * SMAT2(j1,j2)
            end do
            chi2_3 = chi2_3 + soma_j2 * abs_x(j1)
         end do
         chi2_3 = 2 * chi2_3

         do j1=1,N_base
            AV_aux = AV_tot(j1) + AV_tot(j1)
            soma = 0.
            do iRC_AV=1,NRC_AV
               soma = soma + AV_aux**(iRC_AV - 1) * RC3(iRC_AV,j1,j1)
            end do
            SMAT2(j1,j1) = soma * exp(eta_AV * AV_aux)
            chi2_3       = chi2_3 + abs_x(j1) * abs_x(j1) * SMAT2(j1,j1)
         end do


      else


c     If AV IS the same as in the last call, then use previously defined
c     SMAT1 & SMAT2 arrays to speed up computation (skiping the
c     AV-series-loop)

c     2nd term ... Using SMAT1!!
         chi2_2 = 0.
         do j=1,N_base
            abs_x(j) = abs(x(j))
            chi2_2 = chi2_2 + abs_x(j) * SMAT1(j)
         end do

c     3rd term  ... Using SMAT2!!
         chi2_3 = 0.
         do j1=1,N_base
            soma_j2 = 0.
            do j2=1,j1-1
               soma_j2 = soma_j2 + abs_x(j2) * SMAT2(j1,j2)
            end do
            chi2_3 = chi2_3 + soma_j2 * abs_x(j1)
         end do
         chi2_3 = 2 * chi2_3

         do j1=1,N_base
            chi2_3 = chi2_3 + abs_x(j1) * abs_x(j1) * SMAT2(j1,j1)
         end do


      end if


c     Add the 3 terms to obtain the total chi2, test it & go back:)
      chi2           = chi2_1 + chi2_2 + chi2_3
      VeryFastF_chi2 = chi2


c     Test whether series approximation produced chi2 < 0! If so, try
c     again with one further term [NRC_AV <== NRC_AV + 1]
      if (chi2.LT.0.) then
         NRC_AV = NRC_AV + 1
         write (*,'(a,i3,a,f12.3,a)')
     &        '[VeryFastF_chi2] ATT!! Resized NRC_AV to' , NRC_AV , 
     &        '  because chi2 = ' , chi2 , '!'

         call Calc_RC_terms

!CRASH: get out in case RC-crashes! If no-crash, re-do chi2 calculation.
         if (i_RC_CRASH_FLAG.NE.0) return

         i_aux_IsAVstillTheSame = 0
         go to 10
      end if


      return
      end
c ###########################################################################
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +          CHI2 FUNCTIONS USED IN THE KINEMATICAL FITS (v0 & vd)          +
c +                                                                         +
c + fun_vd_chi2                                                             +
c + fun_v0_chi2                                                             +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+



c ###########################################################################
c     This function is used in the Golden-Section-Search for a best vd.
c     It computes chi2 as a function of vd using the kinematics-free
c     (ie, v0 = vd = 0) synthetic spectrum "GSf_syn_NoKin", computed
c     elsewhere for the current best model. v0_min is passed by the
c     /GoldenSearch_aux/ common, as it is needed for GaussSmooth.
c     Nl_aux & l_aux are = Nl_obs & l_obs, but defined elesewhere
c     for convenience (= speed); their only role is to avoid calling
c     GaussSmooth with equally named variables...

c     Cid@Lagoa - Jan/13/2005
      
      FUNCTION fun_vd_chi2(vd)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real l_aux(Nl_max) , f_syn(Nl_max)

      common/N_Gauss/ N_int_Gauss
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/GoldenSearch_aux/ v0_min , vd_min ,
     &     Nl_aux , l_aux , GSf_syn_NoKin(Nl_max)

      call GaussSmooth(Nl_aux,l_aux,GSf_syn_NoKin,v0_min,vd,
     &     N_int_Gauss,Nl_obs,l_obs,f_syn)

      chi2 = 0.
      do i=1,Nl_obs
         chi2 = chi2 + (f_syn(i) - f_obs(i)) * weight_obs(i) *
     &        (f_syn(i) - f_obs(i)) * weight_obs(i)
      end do
      fun_vd_chi2 = chi2

      end
c ###########################################################################

c ###########################################################################
c     This function is used in the Golden-Section-Search for a best v0.
c     It computes chi2 as a function of v0 using the kinematics-free
c     (ie, v0 = vd = 0) synthetic spectrum "GSf_syn_NoKin", computed
c     elsewhere for the current best model. vd_min is passed by the
c     /GoldenSearch_aux/ common, as it is needed for GaussSmooth.
c     Nl_aux & l_aux are = Nl_obs & l_obs, but defined elesewhere
c     for convenience (= speed); their only role is to avoid calling
c     GaussSmooth with equally named variables...

c     Cid@Lagoa - Jan/13/2005

      FUNCTION fun_v0_chi2(v0)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real l_aux(Nl_max) , f_syn(Nl_max)

      common/N_Gauss/ N_int_Gauss
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/GoldenSearch_aux/ v0_min , vd_min ,
     &     Nl_aux , l_aux , GSf_syn_NoKin(Nl_max)


      call GaussSmooth(Nl_aux,l_aux,GSf_syn_NoKin,v0,vd_min,
     &     N_int_Gauss,Nl_obs,l_obs,f_syn)

      chi2 = 0.
      do i=1,Nl_obs
         chi2 = chi2 + (f_syn(i) - f_obs(i)) * weight_obs(i) *
     &        (f_syn(i) - f_obs(i)) * weight_obs(i)
      end do
      fun_v0_chi2 = chi2

      end
c ###########################################################################
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+





c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
c +                      MARKOV-CHAINS RELATED ROUTINES                     +
c +             (This is where the "soul" of the code resides!!)            +
c +                                                                         +
c + Calc_GR_R                                                               +
c + Generate_PopVector                                                      +
c + Fpar_Slowchi2                                                           +
c + Fpar_VFchi2                                                             +
c + EX0s_Fit_Scubi                                                          +
c + Fit_Scubidu                                                             +
c + ScubiduCadeVoce                                                         +
c + Unpack_par2xAVYAVfn                                                     +
c + SaveStep                                                                +
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+




c ###########################################################################
c     Compute Gelman & Rubin "R" convergence-test-ratio GR_R(j) for each
c     parameter j = 1...N_par, given N_chains chains. The input matrices
c     Cpar_ave & Cpar_var must provide the average and variance of
c     parameter j in each chain i_chain. N_steps is the number of steps
c     taken by each chain. See Doran & Muller, astroph/0311331. Return
c     the R-tations for each parameter (GR_R) and the average, as well
c     as flags which verify if these R's are <= the given R_threshold.
c
c     Cid@Lagoa - 04/Feb/2005

      SUBROUTINE Calc_GR_R(N_steps,N_chains,N_par,Cpar_ave,Cpar_var
     $     ,R_threshold,GR_R,GR_R_ave,IsAveBurnInOver,IsIndBurnInOver)


c     Definitions
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Npar_max = 403 , Nchains_max = 1000)
      real Cpar_ave(Npar_max,Nchains_max)
      real Cpar_var(Npar_max,Nchains_max)
      real GR_R(Npar_max)


c     Start loop over all j = 1...N_par parameters
      sum_R_ave = 0.
      do j=1,N_par

c     Compute global average of parameter j over all chains! (= "mean of
c     means").
         sum_p = 0.
         do i_chain=1,N_chains
            sum_p = sum_p + Cpar_ave(j,i_chain)
         end do
         global_ave = sum_p / float(N_chains)

c     Define GR_B & GR_W: GR_B = "Between chains variance", while GR_W =
c     "Within chains variance" (eqs 5 & 6 of Doran & Muller).
         sum_B = 0.
         sum_W = 0.
         do i_chain=1,N_chains
            sum_B = sum_B + (Cpar_ave(j,i_chain) - global_ave)
     $           * (Cpar_ave(j,i_chain) - global_ave)
            sum_W = sum_W + Cpar_var(j,i_chain)
         end do
         GR_B = (float(N_steps) / float(N_chains - 1)) * sum_B
         GR_W = sum_W / float(N_chains)

c     Define GR_R(j) = convergence ratio for parameter # j (eq 7 of
c     Doran & Muller). Also, update summation for mean R.
!     Introduced "protection" against W = j (chains which do not walk!).
!     When this happens (eg, due to a small N_sim), I set GR_R(j) to a
!     very large value, such that neither <GR_R> not GR_R(j) will
!     satisfy the convergence criterion!
         z = N_steps
         if (GR_W.LE.0.) then
!            write (*,'(a,i3,1x,f9.6))')
!     $           '[Calc_GR_R] W_j <= 0! This should NOT happen!',j,GR_W
            GR_R(j) = 10. * N_par * R_threshold + 9999.
         else 
            GR_R(j) = (((z - 1.) / z) * GR_W + GR_B / z) / GR_W
         end if
         sum_R_ave = sum_R_ave + GR_R(j)

c     Check whether GR_R(j) <= 0, which may happen due to precision bugs!
         if (GR_R(j).LE.0.) then
            write (*,'(a,i3,1x,1p,e12.3))')
     $           '[Calc_GR_R] R_j <= 0! This should NOT happen!' , j ,
     $           GR_R(j)
         end if
      end do


c     Mean GR_R over all N_par parameters
      GR_R_ave = sum_R_ave / float(N_par)


c     Verify if Burn-In is over for the Average & Individual R's, given
c     the input threshold. Set 0/1 (No/Yes) flags accordingly.
      IsAveBurnInOver = 1
      if (GR_R_ave.GT.R_threshold) IsAveBurnInOver = 0

      IsIndBurnInOver = 1
      do j=1,N_par
         if (GR_R(j).GT.R_threshold) IsIndBurnInOver = 0
      end do


      end
c ###########################################################################


c ###########################################################################
c # Generate random pop vector x(j), j=1...N, with a scheme
c # that allows a few components to dominate. (Simple normalization
c # "a la Laerte" does not do this). x() is normalized to sum x = 1 
c # (to within roundoff errors...)
c # Cid@Lynx - Dec/15/2002

c # Introduced a maximum x_j amplitude (x_max)
c # Cid@Lynx - Nov/24/2004

! This may be re-written using sort!

      SUBROUTINE Generate_PopVector(idum,N,x_max,x)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400)
      real x(Nb_max) , ind(Nb_max)

      if (N.GT.Nb_max) then
         write (*,*) ' [Generate_PopVector] N > N_max! ' , N , Nb_Max
         stop
      end if

      do j=1,N
         ind(j) = -99
      end do

      do j=1,N
 1       ix = N * ran2(idum) + 1
         do i=1,N
            if (ind(i).EQ.ix) goto 1
         end do
         ind(j) = ix
      end do

      sum_x = 0.
      do j=1,N-1
         i = ind(j)
 2       x(i) = (1. - sum_x) * ran2(idum)
         if (x(i).GT.x_max) goto 2

         sum_x = sum_x + x(i)
      end do
      i = ind(N)
      x(i) = 1. - sum_x

      end
c ###########################################################################



c ###########################################################################
c     Unpack par-vector to (x,fn,AV,YAV) & compute chi2 using
c     Slow_chi2 Assumes x is already normalized to sum x_j = fn!

      FUNCTION Fpar_Slowchi2(N_par,par)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400,Npar_max = 403)
      real par(Npar_max) , x(Nb_max) , AV_tot(Nb_max)

      N_base = N_par - 3
      j_AV   = N_base + 1
      j_YAV  = N_base + 2

      do j=1,N_base
         x(j) = par(j)
      end do
      AV  = par(j_AV)
      YAV = par(j_YAV)

      call Update_AV_array(N_base,AV,YAV,AV_tot)

      Fpar_Slowchi2 = SlowF_chi2(x,AV_tot)

      return
      end
c ###########################################################################


c ###########################################################################
c     Unpack par-vector to (x,fn,AV,YAV) & compute chi2 using
c     VeryFastF_chi2 Assumes x is already normalized to sum x_j = fn!
c     This version use the i_IsAVstillTheSame-trick!

      FUNCTION Fpar_VFchi2(N_par,par,i_IsAVstillTheSame)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400,Npar_max = 403)
      real par(Npar_max) , x(Nb_max) , AV_tot(Nb_max)

      N_base = N_par - 3
      j_AV   = N_base + 1
      j_YAV  = N_base + 2

      do j=1,N_base
         x(j) = par(j)
      end do
      AV  = par(j_AV)
      YAV = par(j_YAV)

      call Update_AV_array(N_base,AV,YAV,AV_tot)

      Fpar_VFchi2 = VeryFastF_chi2(x,AV_tot,i_IsAVstillTheSame)

      return
      end
c ###########################################################################



c ###########################################################################
c #       CONDENSE BASE (excluding irrelevant x_j) & RUN Fit_Scubi          #
c ###########################################################################
c     This routine reduces/condenses the spectral base (Of_base &
c     f_base, used in all routines), excluding all components in
c     x_Test() which fail and EX0s-criterion which depends on the
c     EX0s_method_option-string. In the 'CUMUL' method we exclude the
c     weaker components which add up to < EX0s_Threshold. In the 'SMALL'
c     method we throw away any component which contributes with less
c     flux than EX0s_Threshold / N_base. This reduction is limited to a
c     smaller base dimension of N_base_min. x_Test may be, for instance,
c     the best, or the global-average population vector.
c
c     All "Irrelevant" components are zero'ed. We then call Fit_Scubidu
c     to obtain new fits.  Before exiting, the base matrices & other
c     arrays are restored to their original format, otherwise things
c     would become a total mess!  
c
c     Cid@Lagoa - 27/Mar/2005

      SUBROUTINE EX0s_Fit_Scubi(i_verbose,idum,N_loops,v0_low,v0_upp
     $     ,vd_low,vd_upp,x_min,AV_min,YAV_min,v0_min,vd_min,Ntot_EX0s
     $     ,eff_EX0s,Nl_eff,iMassAgeZ_verbose,N_par,ChainPar,N_chains
     $     ,Temp_ini,Temp_fin,fini_eps,ffin_eps,R_ini,R_fin,N_sim
     $     ,Nmax_steps,i_RestartChains,Gpar_min,GE_min,Gpar_ave,Cpar_min
     $     ,CEne_min,x_Test,EXOs_method_option,EX0s_Threshold,N_base_min
     $     ,IsKinFixed_EX0s,IsBurInOver_EX0s,N_base_Relevant
     $     ,IsScaleNstepsInEX0sFits,IsNoKin4LargeBaseInEX0sFits
     $     ,frac_NoKin4LargeBaseInEX0sFits)

c ***************************************************************************
c *                               DEFINITIONS                               *
c ***************************************************************************
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      parameter (Nchains_max = 1000,Npar_max = 403)
      
c     Observed spectra & etc. (Not really needed here, but I'm using the
c     Obs-common to read Nl_obs)
      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)

c     Population vector & base arrays
      real x_Test(Nb_max) , x_min(Nb_max)
      real f_base(Nb_max,Nl_max) , Of_base(Nb_max,Nl_max)
      real FULL_Of_base(Nb_max,Nl_max)

c     Auxiliar pop vector & its sorting-index-array
      real x_aux(Nb_max) 
      integer ind_x_aux(Npar_max) 

c     Auxiliar array which map components with x(j) > x_RelevantThreshold!
      integer ind_Relevant(Npar_max) 

c     Original/FULL ind_YAV_array (which contains 0/1 indicating who has
c     and extra extinction).
      integer IFULL_ind_YAV(Nb_max)

c     Original Full-Base parameter limits
      real Opar_low(Npar_max) , Opar_upp(Npar_max)

c     Chain-stuff-arrays!
      real ChainPar(Npar_max,Nchains_max)
      real Gpar_min(Npar_max) , Gpar_ave(Npar_max)
      real Cpar_min(Npar_max,Nchains_max)  , CEne_min(Nchains_max)

c     Original Full-Base HelpParMove2Average arrays
      integer iO_ind_HelpParMove2Average(Npar_max)
      real Oprob_HelpParMove2Average(Npar_max)

c     Short-arrays!
      real Shortx_min(Nb_max) , ShortChainPar(Npar_max,Nchains_max)
      real ShortGpar_min(Npar_max) , ShortGpar_ave(Npar_max)
      real ShortCpar_min(Npar_max,Nchains_max)
      real ShortCEne_min(Nchains_max)

c     Option-String which defines which method will be used to define
c     who is relevant and who is not in the pop vector x_Test!
      character*5 EXOs_method_option

c     commons...
      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/Base/ N_base , f_base
      common/Obase/ Of_base
      common/YAV_stuff/ ind_YAV(Nb_max) , N_YAV_components

c     Lower & upper limits on parameters (x,AV,YAV,fn)
      common/ParLimits/ par_low(Npar_max) , par_upp(Npar_max)

c     ScubiduCadeVoce Technical Parameters - All we actually need here
c     are the ind_HelpParMove2Average & prob_HelpParMove2Average arrays!
      common/Scubi_TechnicalParameters/ alpha , Falpha , IsGRTestHard ,
     $     eff_IDEAL , i_UpdateEps , Feps , i_UpdateAlpha ,
     $     i_MoveOneParOnly , i_HelpParWithMaxR , prob_jRmax ,
     $     Npar4GRtest ,i_UpdateAVYAVStepSeparately ,
     $     ind_HelpParMove2Average(Npar_max) ,
     $     prob_HelpParMove2Average(Npar_max)
c ***************************************************************************


c ***************************************************************************
c *                  INITIALIZATION: STORE ORIGINAL STUFF                   *
c ***************************************************************************
c     Store original N_base, N_YAV_compontes, N_par & AV/YAV/fn indices.
      NO_base           = N_base
      NO_YAV_components = N_YAV_components
      NO_par = N_base + 3
      jO_AV  = N_base + 1
      jO_YAV = N_base + 2
      jO_fn  = N_base + 3

c     Store original full base Of_base ==> FULL_Of_base, since Of_base
c     will be modified below!  Do the same thing for the ind_YAV array.
      do j=1,NO_base
         do i=1,Nl_obs
            FULL_Of_base(j,i) = Of_base(j,i)
         end do
         IFULL_ind_YAV(j) = ind_YAV(j)
      end do

c     Store original par-limits & arrays which say whether to help
c     parameters to converge and with what probability.
      do j=1,NO_par
         Opar_low(j) = par_low(j)
         Opar_upp(j) = par_upp(j)

         iO_ind_HelpParMove2Average(j) = ind_HelpParMove2Average(j)
         Oprob_HelpParMove2Average(j)  = prob_HelpParMove2Average(j)
      end do
c ***************************************************************************


c ***************************************************************************
c *            CONSTRUCT INDEX-ARRAY OF "RELEVANT" COMPONENTS               *
c ***************************************************************************
c     Zero-in ind_Relevant() array for precaution
      do j=1,Npar_max
         ind_Relevant(j) = 0
      end do
      N_base_Relevant = 0
      sum_x_Relevant  = 0

c     Figure out the normalization of x_Test
      sum_x_Test = 0
      do j=1,NO_base
         sum_x_Test = sum_x_Test + x_Test(j)
      end do

c     Now create x_aux, the normalized version of x_Test, and indexx it
c     in ind_x_aux, such that x_aux(ind_x_aux) is sorted in a increasing
c     order (stronger components last).
      do j=1,N_base
         x_aux(j) = x_Test(j) / sum_x_Test
      end do
      call indexx(N_base,x_aux,ind_x_aux)

!     Screen output
      if (i_verbose.GT.1) then 
         sum_x = 0
         do j=1,NO_base
            j_aux = ind_x_aux(j)
            sum_x = sum_x + x_aux(j_aux)
            write (*,'(2(a,i3),2(a,f7.3),a)') '[EX0s_Fit_Scubi] ', j
     $           ,' : x_Test(',j_aux,' ) = ' ,100.*x_aux(j_aux)
     $           ,'%   ==>  Cumulative Sum = ' ,100*sum_x ,'%'
         end do
      end if


c     CUMULATIVE-EX0s-METHOD: In this case EX0s_Threshold means the
c     fraction of light in x_Test which which will be thrown away!
      if (EXOs_method_option.EQ.'CUMUL') then

c     Construct array ind_Relevant() which contains the indices of the
c     N_base_Relevant components in x_Test(j) which add-upp to >= 1 -
c     EXOs_Threshold. (Actually, the loop below works the other way
c     around: it discards from ind_Relevant() the smallest-x elements,
c     which add up to a fraction <= EX0s_Threshold.)
         sum_x  = 0
         i_flag = 0
         do j=1,NO_base
            j_aux = ind_x_aux(j)
            sum_x = sum_x + x_aux(j_aux)
            if ((i_flag.EQ.0).AND.(sum_x.GT.EX0s_Threshold)) i_flag = 1
            if (i_flag.EQ.1) then
               sum_x_Relevant  = sum_x_Relevant + x_aux(j_aux)
               N_base_Relevant = N_base_Relevant + 1
               ind_Relevant(N_base_Relevant) = j_aux
!               write (*,'(a,4(i4),3(f10.3))')
!     $              '[EX0s_Fit_Scubi] Relevant comps.: ', j ,j_aux
!     $              ,N_base_Relevant ,ind_Relevant(N_base_Relevant) ,100
!     $              *x_aux(j_aux),100*sum_x,100*sum_x_Relevant
            end if
         end do

c     SMALL-X-EX0s-METHOD: In this case EX0s_Threshold / N_base defines
c     the minimum value of x for a component to be kept in the
c     base. Weaker components will be thrown away!
      else if (EXOs_method_option.EQ.'SMALL') then

c     Set Threshold for elimination!
         x_RelevantThreshold = EX0s_Threshold / float(N_base)

c     Construct array ind_Relevant() which contains the indices of the
c     N_Relevant components in x_aux (= normalized x_Test) which are
c     stronger (ie, >) than x_RelevantThreshold.
         do j=1,NO_base
            if (x_aux(j).GT.x_RelevantThreshold) then
               N_base_Relevant = N_base_Relevant + 1
               ind_Relevant(N_base_Relevant) = j
               sum_x_Relevant = sum_x_Relevant + x_aux(j)
!              write (*,'(a,3(i4),2(f10.3))')
!    $              '[EX0s_Fit_Scubi] Relevant comps.: ', j
!    $              ,N_base_Relevant ,ind_Relevant(N_base_Relevant) ,100
!    $              *x_aux(j),100*sum_x_Relevant
            end if
         end do

      end if


c     PROTECTION against too large reductions in the base! If less than
c     N_base_min of the original base elements survived the reduction
c     above then just pick the N_base_min stonger elements. (Use the
c     sorting/index-array for this.)
      if (N_base_Relevant.LT.N_base_min) then

         write (*,'(a,i3,a,i3,a)')
     $        '[EX0s_Fit_Scubi] ATT: N_base_Relevant = ' ,
     $        N_base_Relevant , ' < N_base_min = ' ,N_base_min
     $        ,' elements in the reduced base! '
         write (*,'(a,f7.3,a)') '[EX0s_Fit_Scubi] They they add up to '
     $        ,100.*sum_x_Relevant , '% of the light.'
         write (*,'(a,i3,a)')
     $        '[EX0s_Fit_Scubi] Will fix this picking the ',N_base_min ,
     $        ' strongest ones!!'
         write (*,*)

         N_base_Relevant = 0
         sum_x_Relevant  = 0
         i_flag = 0
         do j=NO_base,(NO_base-N_base_min+1),-1
            j_aux = ind_x_aux(j)
            sum_x_Relevant  = sum_x_Relevant + x_aux(j_aux)
            N_base_Relevant = N_base_Relevant + 1
            ind_Relevant(N_base_Relevant) = j_aux
!            write (*,'(a,3(i4),2(f10.3))')
!     $           '[EX0s_Fit_Scubi] Relevant comps.: ', j ,j_aux
!     $           ,N_base_Relevant ,100 *x_aux(j_aux),100 *sum_x_Relevant
         end do

      end if


!     Screen output
      if (i_verbose.GT.0) then 
         write (*,'(a,i3,a,f6.3,a)') '[EX0s_Fit_Scubi] There are '
     $        , N_base_Relevant ,
     $        ' elements in the reduced base, and they add up to ' ,
     $        100.*sum_x_Relevant , '% of the light.'
         write (*,'(a,a)') '[EX0s_Fit_Scubi] EX0-method     = '
     $        ,EXOs_method_option
         write (*,'(a,f7.3)')
     $        '[EX0s_Fit_Scubi] EX0s_Threshold = ' , EX0s_Threshold 
      end if


c     Freeze kinematics for EX0-fits if condensed base is still "too
c     large": > frac_NoKin4LargeBaseInEX0sFits * N_base. OBS: In
c     StarlightChains_v01 we froze the kinematics for > 50
c     components. Now we set this with the *NoKin4LargeBaseInEX0sFits
c     config variables.
!Cid14Jan07-FXK! IsKinFixed_EX0s may already be 1 if doing FXK fits!
      N_aux = nint(frac_NoKin4LargeBaseInEX0sFits * N_base)
      if ((IsKinFixed_EX0s.NE.1).AND.(IsNoKin4LargeBaseInEX0sFits.EQ.1)
     $     .AND.(N_base_Relevant.GT.N_aux)) IsKinFixed_EX0s = 1


!     Screen output - to freeze or not to freeze kinematical fits...
      if (i_verbose.GT.0) then 
         if (IsKinFixed_EX0s.EQ.1) then
!     Note: If IsThisAnFXKFit = 1, the message below will be shown but
!     means nothing which we did not already know!
            write (*,'(a,a)') '[EX0s_Fit_Scubi] ATT: ' ,
     $           ' We will freeze in the kinematical parameters!'
         else
            write (*,'(a,a)') '[EX0s_Fit_Scubi] ATT: ',
     $           ' We will NOT freeze in the kinematical parameters!'
         end if
      end if
c ***************************************************************************



c ***************************************************************************
c *              SCALE N_sim & Nmax_steps for reduced base!                 *
c *                      Cid@Lagoa - 09/October/2005                        *
c ***************************************************************************
!     I realized that in StarlightChains_v01.for I fixed the numbers of
!     steps N_sim & Nmax_steps for the EX0s runs as multiples of N_par
!     (as in other parts of the code). As a consequence runs where only
!     a few components are used in the EX0s fits do many more
!     steps/per-parameter than runs whereas galaxies with many EX0s
!     components do much less. A perhaps more reasonable thing to do is
!     to scale the number of steps in terms of the number of Relevant
!     parameters, which is only known once the code gets here. To use
!     this new feature I introduced the IsScaleNstepsInEX0sFits = Y/N
!     (1/0) flag.
      if (IsScaleNstepsInEX0sFits.EQ.1) then
         N_par_Relevant = N_base_Relevant + 3
         scale_down_fac = float(N_par_Relevant) / float(NO_par)

         N_sim      = nint(N_sim * scale_down_fac)
         Nmax_steps = nint(Nmax_steps * scale_down_fac)

!     Safety: Lower limits to N_sim & Nmax_steps
         if (N_sim.LT.(10*N_par_Relevant)) N_sim = 10 * N_par_Relevant
         if (Nmax_steps.LT.(10*N_sim)) Nmax_steps = 10 * N_sim

!     Screen output
         if (i_verbose.GT.0) then 
            write (*,'(a,i8)') '[EX0s_Fit_Scubi] N_sim      scaled to ',
     $           N_sim
            write (*,'(a,i8)') '[EX0s_Fit_Scubi] Nmax_steps scaled to ',
     $           Nmax_steps
         end if
      end if
c ***************************************************************************


c ***************************************************************************
c *              CONDENSE BASE TO ITS "RELEVANT" COMPONENTS                 *
c ***************************************************************************
c     Reset all Short* arrays
      do j=1,Nb_max
         Shortx_min(j) = 0.
      end do
      do j=1,Npar_max
         ShortGpar_min(j) = 0.
         ShortGpar_ave(j) = 0.
         do i_chain=1,Nchains_max
            ShortChainPar(j,i_chain) = 0.
         end do
      end do


c     Redefine N_base, N_par & par-indices of AV, YAV & fn
      N_base = N_base_Relevant
      N_par  = N_base + 3
      j_AV   = N_base + 1
      j_YAV  = N_base + 2
      j_fn   = N_base + 3


c     Fill in last 3 elements of ind_Relevant() with AV, YAV & fn indices.
      ind_Relevant(j_AV)  = jO_AV
      ind_Relevant(j_YAV) = jO_YAV
      ind_Relevant(j_fn)  = jO_fn


c     Condense/shorten base N_base_Relevant components. Also condense
c     ind_YAV() array & recompute N_YAV_components for the smaller base
c     (both passed by the commom /YAV_stuff/ to other routines).  OBS:
c     Everything must be restored before exiting!!
      N_YAV_components = 0
      do j=1,N_base
         j_in_full_base = ind_Relevant(j)

         do i=1,Nl_obs            
            Of_base(j,i) = FULL_Of_base(j_in_full_base,i)
            f_base(j,i)  = FULL_Of_base(j_in_full_base,i)
         end do

         ind_YAV(j) = IFULL_ind_YAV(j_in_full_base)
         if (ind_YAV(j).NE.0) N_YAV_components = N_YAV_components + 1
      end do


c     Condense par-limits & ind_HelpParMove2Average &
c     prob_HelpParMove2Average arrays (passed by common...)
      do j=1,N_par
         j_in_full_base = ind_Relevant(j)
         par_low(j) = Opar_low(j_in_full_base)
         par_upp(j) = Opar_upp(j_in_full_base)
         ind_HelpParMove2Average(j) =
     $        iO_ind_HelpParMove2Average(j_in_full_base)
         prob_HelpParMove2Average(j) =
     $        Oprob_HelpParMove2Average(j_in_full_base)
      end do


c     Condense pop vectors ChainPar() ==> ShortChainPar() for each
c     chain, plus the global average & minimum.
      do j=1,N_par
         j_in_full_base = ind_Relevant(j)
         do i_chain=1,N_chains
            ShortChainPar(j,i_chain) = ChainPar(j_in_full_base,i_chain)
         end do
         ShortGpar_min(j) = Gpar_min(j_in_full_base)
         ShortGpar_ave(j) = Gpar_ave(j_in_full_base)
      end do


c     Since we are throwing away all "irrelevant" components, it is a
c     good idea renormalize all chain-pop-vectors to their respective fn
c     (the irrelevant x_j's may still carry some flux if
c     x_RelevantThreshold > 0). 1st renomalize the chain-pop-vectors.
      do i_chain=1,N_chains
         sum_x = 0.
         do j=1,N_base
            sum_x = sum_x + ShortChainPar(j,i_chain)
         end do
         fn = ShortChainPar(j_fn,i_chain)
         do j=1,N_base
            ShortChainPar(j,i_chain) = ShortChainPar(j,i_chain)*sum_x/fn
         end do
      end do


c     Now renomalize the Global Average & Minimum pop-vectors.
c     (This is probably not necessary but it should do no harm!)
      sum1_x = 0.
      sum2_x = 0.
      do j=1,N_base
         sum1_x = sum1_x + ShortGpar_min(j)
         sum2_x = sum2_x + ShortGpar_ave(j)
      end do
      fn1 = ShortGpar_min(j_fn)
      fn2 = ShortGpar_ave(j_fn)
      do j=1,N_base
         ShortGpar_min(j) = ShortGpar_min(j) * sum1_x / fn1
         ShortGpar_ave(j) = ShortGpar_ave(j) * sum2_x / fn2
      end do


c     Define x_min, AV_min & YAV_min (probably un-necessary!)
      do j=1,N_base
         Shortx_min(j) = ShortGpar_min(j)
      end do
      AV_min  = ShortGpar_min(j_AV)
      YAV_min = ShortGpar_min(j_YAV)


c     Empty x_min, ChainPar, Gpar_min & Gpar_ave arrays. These arrays
c     will all be fill back in after the fit with a condensed base is
c     over. Here we empty them just to make sure we are zero-ing the
c     "irrelevant" components!
      do j=1,Nb_max
         x_min(j) = 0.
      end do
      do j=1,Npar_max
         Gpar_min(j)      = 0.
         Gpar_ave(j)      = 0.
         do i_chain=1,Nchains_max
            ChainPar(j,i_chain) = 0.
         end do
      end do
c ***************************************************************************


c ***************************************************************************
c *              FIT SPECTRUM USING SHORTENED/CONDENSED BASE                *
c ***************************************************************************
c     Call Fit_Scubidu with condensed base
      call Fit_Scubidu(i_verbose,idum,N_loops,v0_low,v0_upp,vd_low
     $     ,vd_upp,Shortx_min,AV_min,YAV_min,v0_min,vd_min,Ntot_EX0s
     $     ,eff_EX0s,Nl_eff,0,N_par,ShortChainPar,N_chains,Temp_ini
     $     ,Temp_fin,fini_eps,ffin_eps,R_ini,R_fin,N_sim,Nmax_steps
     $     ,i_RestartChains,ShortGpar_min,GE_min,ShortGpar_ave
     $     ,ShortCpar_min,ShortCEne_min,IsKinFixed_EX0s,
     $     IsBurInOver_EX0s)
c ***************************************************************************


c ***************************************************************************
c *             RESTORE ORIGINAL BASE & REARRANGE POP-VECTORS               *
c ***************************************************************************
c     Restore Original base: FULL_Of_base ==> Of_base & f_base
      do i=1,Nl_obs
         do j=1,NO_base
            Of_base(j,i) = FULL_Of_base(j,i)
            f_base(j,i)  = Of_base(j,i)
         end do
      end do


c     Restore Original ind_YAV array & N_YAV_components
      do j=1,NO_base
         ind_YAV(j) = IFULL_ind_YAV(j)
      end do


c     Restore original par-limits & arrays which say whether to help
c     parameters to converge and with what probability.
      do j=1,NO_par
         par_low(j) = Opar_low(j)
         par_upp(j) = Opar_upp(j)

         ind_HelpParMove2Average(j)  = iO_ind_HelpParMove2Average(j)
         prob_HelpParMove2Average(j) = Oprob_HelpParMove2Average(j)
      end do


c     Rearrange all parameters onto correct full-base order. (Irrelevant
c     components have been zero'ed above!)
      do j=1,N_par
         j_in_full_base = ind_Relevant(j)
         do i_chain=1,N_chains
            ChainPar(j_in_full_base,i_chain) = ShortChainPar(j,i_chain)

            Cpar_min(j_in_full_base,i_chain) = ShortCpar_min(j,i_chain) 
            CEne_min(i_chain)                = ShortCEne_min(i_chain)
         end do
         Gpar_min(j_in_full_base) = ShortGpar_min(j)
         Gpar_ave(j_in_full_base) = ShortGpar_ave(j)
      end do


c     Restore original base dimension, number of YAV components, number
c     of parameters & indices of AV, YAV & fn
      N_YAV_components = NO_YAV_components
      N_base = NO_base
      N_par = NO_par
      j_AV  = N_base + 1
      j_YAV = N_base + 2
      j_fn  = N_base + 3


c     Unpack Gpar_min onto x_min,AV_min & YAV_min explicitly.
      call Unpack_par2xAVYAVfn(N_par,Gpar_min,x_min,AV_min,YAV_min
     $     ,fn_min)
c ***************************************************************************



      end
c ###########################################################################




c ###########################################################################
c #          ANNEALING DRIVER FOR SCUBIDU ..........................        #
c ###########################################################################

C????? ABSTRACT!!!      
c     ... if IsKinFixed = 1 then RCtems & base-convolution are done only
c     once, before the main loop!

c     Fit f_obs(l_obs) using a recipe like this:
c     1- Smooth & shift BASE spectra to current v0 & vd (Of_base ==> f_base)
c     2- Increase weights & decrease step-size, ajdusting number of steps
c     3- Run Metropolis for (x,AV,YAV,fn) only, using smothed base (f_base)
c     4- Un-smooth base & estimate (v0,vd) for current population parameters
c       (A Golden-Section-Search is used to find v0 & vd)
c     5- Goto 1, starting from new (x,AV,YAV,fn,v0,vd) initial point.
c     Tests so far proved that this works beautifully!!
c     Cid@Lynx - 21/Aug/2003
c     ....


      SUBROUTINE Fit_Scubidu(i_verbose,idum,N_loops,v0_low,v0_upp,vd_low
     $     ,vd_upp,x_min,AV_min,YAV_min,v0,vd,Ntot_anneal,eff_anneal
     $     ,Nl_eff,iMassAgeZ_verbose,N_par,Cpar_last,N_chains,Temp_ini
     $     ,Temp_fin,fini_eps,ffin_eps,R_ini,R_fin,N_sim,Nmax_steps
     $     ,i_RestartChains,Gpar_min,GE_min,Gpar_ave,Cpar_min,CEne_min
     $     ,IsKinFixed,IsBurInOver)


c ***************************************************************************
c *                                Definitions                              *
c ***************************************************************************
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nl_max = 30000,Nb_max = 400)
      parameter (Nchains_max = 1000,Npar_max = 403)

      real eps(Npar_max) , eps_ini(Npar_max) , eps_fin(Npar_max)
      real aux1(Npar_max) , aux2(Npar_max)

      real Cpar_last(Npar_max,Nchains_max) , CEne_last(Nchains_max)
      real Cpar_min(Npar_max,Nchains_max)  , CEne_min(Nchains_max)
      real Cpar_low(Npar_max) , Cpar_upp(Npar_max)
      real Gpar_min(Npar_max) , Gpar_ave(Npar_max)

      real l_obs(Nl_max) , f_obs(Nl_max) , weight_obs(Nl_max)
      real l_aux(Nl_max) , f_aux1(Nl_max) , f_aux2(Nl_max)
      real x_min(Nb_max) , AV_tot(Nb_max) 
      real f_base(Nb_max,Nl_max) , Of_base(Nb_max,Nl_max)
      real f_syn(Nl_max) , fbase_norm(Nb_max) 

      common/Obs/ Nl_obs , l_obs , f_obs , weight_obs
      common/Base/ N_base , f_base
      common/Obase/ Of_base
      common/N_Gauss/ N_int_Gauss
      common/GoldenSearch_aux/ v0_min , vd_min ,
     &     Nl_aux , l_aux , GSf_syn_NoKin(Nl_max)
      common/RC_AV_Stuff/ NRC_AV , i_RC_CRASH_FLAG , NRC_AV_Default
      common/YAV_stuff/ ind_YAV(Nb_max) , N_YAV_components
      common/CalcMass/ q_norm , fbase_norm , i_FitPowerLaw , fobs_norm

c     Lower & upper limits on parameters (x,AV,YAV,fn)
      common/ParLimits/ par_low(Npar_max) , par_upp(Npar_max)

      external fun_vd_chi2
      external fun_v0_chi2

!Cid14Jan07-FXK! Added FXK = Fixed kinematics option!!
      common/FXK_fits/ IsThisAnFXKFit
c ***************************************************************************


c ***************************************************************************
c *                              Initialization                             *
c ***************************************************************************
c     Reset best kinematic parameters to initial values
      v0_min = v0 
      vd_min = vd


c     Define AV, YAV & fn positions in par-vector
      j_AV  = N_base + 1
      j_YAV = N_base + 2
      j_fn  = N_base + 3


c     Define AV & YAV limits explicity from their values in par_low &
c     _upp. (These are Used in the brute-force search for better AV/YAV.)
      AV_low  = par_low(j_AV)
      AV_upp  = par_upp(j_AV)
      YAV_low = par_low(j_YAV)
      YAV_upp = par_upp(j_YAV)


c     Set step-size schedule in terms of input parameters fini_eps &
c     ffin_eps. We restrict AV/YAV eps sizes to up to 1 mag / fini_eps
c     (initial) & 1 mag / ffin_eps (final).
      do j=1,N_par
         eps_ini(j) = (par_upp(j) - par_low(j)) / fini_eps
         eps_fin(j) = (par_upp(j) - par_low(j)) / ffin_eps
      end do
      eps_ini(j_AV)  = min(eps_ini(j_AV),(1.0/fini_eps))
      eps_fin(j_AV)  = min(eps_fin(j_AV),(1.0/ffin_eps))
      eps_ini(j_YAV) = min(eps_ini(j_YAV),(1.0/fini_eps))
      eps_fin(j_YAV) = min(eps_fin(j_YAV),(1.0/ffin_eps))


c     Reset total efficiency & number of steps counters. They will count
c     all steps for all chains and all annealing loops.
      eff_anneal  = 0.
      Ntot_anneal = 0


!Cid14Jan07-FXK! If IsThisAnFXKFit = 1 then base is **already smoothed**
!     and the steps below should be skipped! Start with a sanity check:
!     If IsThisAnFXKFit = 1 then IsKinFixed MUST be 1!
      if ((IsThisAnFXKFit.EQ.1).AND.(IsKinFixed.NE.1)) then
         write (*,'(a,2(a,f9.3),a)')
     $        '[Fit_Scubidu] BUG/PROBLEM! IsThisAnFXKFit = ' ,
     $        IsThisAnFXKFit , 'but IsKinFixed = ', IsKinFixed
      end if

      if (IsThisAnFXKFit.NE.1) then

c     Store l_obs & Nl_obs onto auxiliar variables (used in all calls to
c     GaussSmooth below).
         do i=1,Nl_obs
            l_aux(i)  = l_obs(i)
         end do
         Nl_aux = Nl_obs

c     Smooth & shift original base Of_base to f_base for input vd &
c     v0. This has to be done here (before the annealing loop) because
c     we need to compute chain energies. These energies MAY be used to
c     initialize the cooling schedule (but we're not really doing it!!)
         if (i_verbose.GE.1) write (*,'(a,2(a,f9.3),a)')
     $        '[Fit_Scubidu] Smoothing base to' ,' v0 = ' , v0 ,
     $        '  & vd = ' , vd , ' km/s...'
         do j=1,N_base
            do i=1,Nl_obs
               f_aux1(i) = Of_base(j,i)
            end do

            call GaussSmooth(Nl_aux,l_aux,f_aux1,v0,vd,
     &           N_int_Gauss,Nl_obs,l_obs,f_aux2)

            do i=1,Nl_obs
               f_base(j,i) = f_aux2(i)
            end do
         end do
         if (i_verbose.GT.1) write (*,'(a)') '[Fit_Scubidu] ... done!'

      end if


c     Initialize RC-terms (so that we can compute energies!) Note that
c     the 1st-guess kinematical filter has just been applied. Presumably
c     the default number of RC_AV terms will be used the first time the
c     code goes through this call to Calc_RC_terms...
      if (i_verbose.GE.1) write (*,'(a)')
     $     '[Fit_Scubidu] Computing RC-terms ...'
      call Calc_RC_terms
      if (i_verbose.GE.1) write (*,'(a)')
     $     '[Fit_Scubidu] ... RC-terms computed!'


c     Compute initial chain energies & reset initial chain-minima.
c     OBS: Note that if a previous best minimum was found and NOT stored
c     in one of the Cpar_last's then it will be lost!!
      do i_chain=1,N_chains
         do j=1,N_par
            aux1(j)             = Cpar_last(j,i_chain)
            Cpar_min(j,i_chain) = Cpar_last(j,i_chain)
         end do
         Ene_initial = 0.5 * Fpar_VFchi2(N_par,aux1,0)
         CEne_last(i_chain) = Ene_initial
         CEne_min(i_chain)  = Ene_initial
      end do


c     Reset initial GLOBAL best-model parameters & energy (Gpar_ & GE_min).
      GE_min = 2. * CEne_min(1)
      do i_chain=1,N_chains
         if (CEne_min(i_chain).LT.GE_min) then
            do j=1,N_par
               Gpar_min(j) = Cpar_min(j,i_chain)
            end do
            GE_min = CEne_min(i_chain)
         end if
      end do


c     Unpack Gpar_min to x_min , AV_min, YAV_min & fn_min
      call Unpack_par2xAVYAVfn(N_par,Gpar_min,x_min,AV_min,YAV_min
     $     ,fn_min)
c ***************************************************************************


c ***************************************************************************
c *                             Annealing loop                              *
c ***************************************************************************
c     1- Smooth & shift f_base spectra to current v0 & vd
c     2- Recompute RC-terms & test NRC_AV
c     3- Recompute Chain Energies (min, last & global-min)
c     4- Adjust Temperature, R_Threshold & step-sizes according to
c        cooling schedule
c     5- Run Subidu-Metropolis for (x,AV,YAV,fn) only, using smothed base
c        until GR-convergence is reached.
c     6- Do Quick brute-force search for better AV & YAV for fixed x,fn,v0,vd 
c     7- Restart Chains: put each chain back to its min-position & replace
c        worst chain by global minimum
c     8- Un-smooth base & estimate (v0,vd) for current population parameters
c     9- Goto 1, starting from new (x,AV,YAV,fn,v0,vd) initial point.

      do i_loop=1,N_loops


c     Given that the base was smoothed & RC-terms were already computed
c     in the initialization above, we do not have to re-do these steps
c     it in the 1st loop....
         if ((i_loop.GT.1).AND.(IsKinFixed.NE.1)) then


c     -------------------> Smooth & shift BASE spectra <---------------------
c     Smooth & shift original base Of_base to f_base for current vd & v0
c     The smoothed f_base is passed by common to other routines!  ATT:
c     f_base is NOT renormalized after convolution with G(v0,vd). After
c     checking my formulae, I concluded that renormalization is not
c     desirable afterall.  
            if (i_verbose.GE.1) write (*,'(a,2(f9.3),a)')
     $           '[Fit_Scubidu] REsmoothing base to ',v0,vd ,' km/s...'
            do j=1,N_base
               do i=1,Nl_obs
                  f_aux1(i) = Of_base(j,i)
               end do

               call GaussSmooth(Nl_aux,l_aux,f_aux1,v0,vd,N_int_Gauss
     $              ,Nl_obs,l_obs,f_aux2)

               do i=1,Nl_obs
                  f_base(j,i) = f_aux2(i)
               end do
            end do
            if (i_verbose.GE.1) write (*,'(a)') '[Fit_Scubidu] ...done!'
c     -----------------------------------------------------------------------


c     -----------------------> Recompute RC-terms <--------------------------
c     Since the base-spectra changed (new v0,vd), we need to re-compute
c     the RC-terms.  Before doing so, I estimate the number NRC_AV of
c     terms in the AV-series as a function of AV_min, such that NRC_AV
c     increases linearly with |AV_min| up to a maximum value. This
c     relation needs to be calibrated through experiments...  In any
c     case, the test a few lines below fix any bad-guess for NRC_AV.
c     ATT: I am including YAV in this guesstimate... not well tested...
            if (i_verbose.GE.1) write (*,'(a)')
     $           '[Fit_Scubidu] Computing RC-terms ...'
            z1 = 5. + 6. * abs(AV_min)
            if (N_YAV_components.NE.0) then
               z2 = 5. + 6. * abs(YAV_min)
               if (z2.GT.z1) z1 = z2
            end if
            NRC_AV = min(z1,1.*NRC_AV_Default)
            call Calc_RC_terms
            if (i_verbose.GE.1) write (*,'(a)')
     $           '[Fit_Scubidu] ... RC-terms re-computed!'
c     -----------------------------------------------------------------------


         end if


c     -------------> Test whether more RC terms  are needed <----------------
c     Compare the approximate (fast) and exact (slow) chi2's. If the
c     relative difference exceeds a threshold, then increase the number
c     of terms (NRC_AV) in the AV-truncated-series until convergence is
c     achieved. More terms are needed when AV increases... Since the two
c     terms always agree well for AV = 0, I carry out the test with AV =
c     0.3 (typical of SDSS galaxies...) unless |AV_min| is larger, and
c     likewise for YAV. Note also that the chi2 routines themselves may
c     change NRC_AV when they detect trouble (ie, negative chi2's)!
c     Hence, we've got plenty of protection. (ATT: we may fail to detect
c     a bad chi2 which, say, is going towards < 0 but hasn't yet gotten
c     there...)
 100     AV_test  = 0.3
         YAV_test = 0.3
         if (AV_test.LT.(abs(AV_min))) AV_test = AV_min
         if (YAV_test.LT.(abs(YAV_min))) YAV_test = YAV_min
         if (N_YAV_components.EQ.0) YAV_test = 0.0
         call Update_AV_array(N_base,AV_test,YAV_test,AV_tot)

         z1 = VeryFastF_chi2(x_min,AV_tot,0)
         z2 = SlowF_chi2(x_min,AV_tot)
         rel_diff = (z1 - z2) / z2
         dchi2_threshold = 1.e-3

         if (abs(rel_diff).GT.dchi2_threshold) then
            write (*,11) 'fitspec',i_loop,'  ATT: Increasing' ,
     $           ' # of terms in AV-series...' , z1 , z2 , rel_diff ,
     $           dchi2_threshold , AV_test , AV_min , YAV_test , YAV_min
     $           , NRC_AV , ' ==> ' , NRC_AV + 1
 11         format (a7,i3,a,a,1p,4(2x,e12.5),0p,4(f9.3,1x),i2,a,i2)

            NRC_AV = NRC_AV + 1
            call Calc_RC_terms

!     CRASH: Get out in case RC-crashes! Otherwise repeat test
            if (i_RC_CRASH_FLAG.NE.1) goto 100
         end if


!     Screen output - controling difference between Slow & Fast - chi2!
         if (i_verbose.GT.0) write (*,12) i_loop , z1,z2, 100.* rel_diff
     $        , AV_test ,YAV_test , NRC_AV
 12      format ('FitScub',i3,'  RC-test: Fast chi2 = ' ,1p,e12.4,3x,
     $        'Slow chi2 = ' ,e12.4,3x,' Dif = ',e12.4,
     $        '%    for AV_test = ',0p,f6.3,' YAV_test = ',f6.3,
     $        ' & NRC_AV = ',i3)
c     -----------------------------------------------------------------------


c     -----------------------> Recompute Energies <--------------------------
c     Since v0 & vd may have changed, and maybe NRC_AV too, we must
c     recompute the minimum & last chain energies & the global minimum
c     before passing these values to ScubiduCadeVoce... Note that
c     energies are recomputed even if neither v0, vd nor NRC_AV changed,
c     which should do no harm.
         do i_chain=1,N_chains
            do j=1,N_par
               aux1(j) = Cpar_min(j,i_chain)
               aux2(j) = Cpar_last(j,i_chain)
            end do
            CEne_min(i_chain)  = 0.5 * Fpar_VFchi2(N_par,aux1,0)
            CEne_last(i_chain) = 0.5 * Fpar_VFchi2(N_par,aux2,0)
         end do
         GE_min = 0.5 * Fpar_VFchi2(N_par,Gpar_min,0)
c     -----------------------------------------------------------------------


c     ---------------------> Metropolis/Scubidu Run <------------------------
c     Update Temperature, R & step-sizes. Then run ScubiduCadeVoce.
         z1 = 0.
         if (N_loops.GT.1) z1 = float(i_loop - 1) / float(N_loops - 1)
         Temperature = Temp_ini * (Temp_fin / Temp_ini)**z1
         R_Threshold = R_ini * (R_fin / R_ini)**z1
         do j=1,N_par
            eps(j) = eps_ini(j) * (eps_fin(j) / eps_ini(j))**z1
         end do
         if (N_YAV_components.EQ.0) eps(j_YAV) = 0.

         call ScubiduCadeVoce(idum,N_par,N_chains,N_sim,Nmax_steps
     $        ,Temperature,R_Threshold,i_verbose,IsBurInOver,Cpar_last
     $        ,CEne_last,eps,eff_Scubi,N_steps,Gpar_min,GE_min,Cpar_min
     $        ,CEne_min,Cpar_low,Cpar_upp,Gpar_ave)
c     -----------------------------------------------------------------------


c     ----> Quick brute force search for better AV & YAV for each chain <----
c     Try NAV_try values of AV & NYAV_try of YAV, each within +/-
c     n_BruteForce * eps magnitudes of AV_min & YAV_min, or 0.1 mag,
c     whichever is larger (but limited to AV_low ==> AV_upp). If a
c     better minimum is found then update CEne_min, Cpar_min & Cpar_last.
c     ATT: To be more elegant, this should be replaced by a
c     Golden-Search minimization...
         if (i_verbose.GT.1) write (*,'(a)')
     $        '[Fit_Scubidu] Starting BruteForce AV/YAV ...'
         i_BruteForceFlag = 0
         n_BruteForce     = 5
         AV_range  = max(n_BruteForce*eps(j_AV),0.1)
         YAV_range = max(n_BruteForce*eps(j_YAV),0.1)

         do i_chain=1,N_chains
            do j=1,N_par
               aux1(j) = Cpar_min(j,i_chain)
            end do
            call Unpack_par2xAVYAVfn(N_par,aux1,x_min,AV_min,YAV_min
     $           ,fn_min)

            AV_1 = AV_min - AV_range
            AV_2 = AV_min + AV_range
            if (AV_1.LT.AV_low) AV_1 = AV_low
            if (AV_2.GT.AV_upp) AV_2 = AV_upp
            NAV_try = 100
            if (N_YAV_components.GT.0) NAV_try = 25
            dAV_try = (AV_2 - AV_1) / float(NAV_try - 1)

            YAV_1    = 0.
            NYAV_try = 1
            dYAV_try = 0.
            if (N_YAV_components.GT.0) then
               YAV_1 = YAV_min - YAV_range
               YAV_2 = YAV_min + YAV_range
               if (YAV_1.LT.YAV_low) YAV_1 = YAV_low
               if (YAV_2.GT.YAV_upp) YAV_2 = YAV_upp
               NYAV_try = 25
               dYAV_try = (YAV_2 - YAV_1) / float(NYAV_try - 1)
            end if

            do iAV_try=1,NAV_try
               AV_try = AV_1 + (iAV_try - 1) * dAV_try
               do iYAV_try=1,NYAV_try
                  YAV_try = YAV_1 + (iYAV_try - 1) * dYAV_try

                  call Update_AV_array(N_base,AV_try,YAV_try,AV_tot)
                  Ene_try = 0.5 * VeryFastF_chi2(x_min,AV_tot,0)

                  if (Ene_try.LT.CEne_min(i_chain)) then
                     CEne_min(i_chain)        = Ene_try
                     Cpar_min(j_AV,i_chain)   = AV_try
                     Cpar_min(j_YAV,i_chain)  = YAV_try
                     Cpar_last(j_AV,i_chain)  = AV_try
                     Cpar_last(j_YAV,i_chain) = YAV_try
                     i_BruteForceFlag = 1
                  end if
               end do
            end do
         end do


c     Now do the same brute-force search for the Global-best model so far...
         call Unpack_par2xAVYAVfn(N_par,Gpar_min,x_min,AV_min,YAV_min
     $        ,fn_min)
         AV_1 = AV_min - AV_range
         AV_2 = AV_min + AV_range
         if (AV_1.LT.AV_low) AV_1 = AV_low
         if (AV_2.GT.AV_upp) AV_2 = AV_upp
         NAV_try = 100
         if (N_YAV_components.GT.0) NAV_try = 25
         dAV_try = (AV_2 - AV_1) / float(NAV_try - 1)

         YAV_1    = 0.
         NYAV_try = 1
         dYAV_try = 0.
         if (N_YAV_components.GT.0) then
            YAV_1 = YAV_min - YAV_range
            YAV_2 = YAV_min + YAV_range
            if (YAV_1.LT.YAV_low) YAV_1 = YAV_low
            if (YAV_2.GT.YAV_upp) YAV_2 = YAV_upp
            NYAV_try = 25
            dYAV_try = (YAV_2 - YAV_1) / float(NYAV_try - 1)
         end if

         do iAV_try=1,NAV_try
            AV_try = AV_1 + (iAV_try - 1) * dAV_try
            do iYAV_try=1,NYAV_try
               YAV_try = YAV_1 + (iYAV_try - 1) * dYAV_try

               call Update_AV_array(N_base,AV_try,YAV_try,AV_tot)
               Ene_try = 0.5 * VeryFastF_chi2(x_min,AV_tot,0)

               if (Ene_try.LT.GE_min) then
                  GE_min           = Ene_try
                  Gpar_min(j_AV)   = AV_try
                  Gpar_min(j_YAV)  = YAV_try
                  i_BruteForceFlag = 1
               end if
            end do
         end do
         if (i_verbose.GT.1) write (*,'(a,i2)')
     $        '[Fit_Scubidu] ... done! ',i_BruteForceFlag
c     -----------------------------------------------------------------------


c     ---------------------> A few simple things ... <-----------------------
c     Re-find best Global minimum among the chains (which may have
c     changed after the brute-force AV/YAV minimization above).
         do i_chain=1,N_chains
            if (CEne_min(i_chain).LE.GE_min) then
               do j=1,N_par
                  Gpar_min(j) = Cpar_min(j,i_chain)
               end do
               GE_min = CEne_min(i_chain)
            end if
         end do


c     Unpack Gpar_min onto x_min, AV_min, YAV_min & fn_min
         call Unpack_par2xAVYAVfn(N_par,Gpar_min,x_min,AV_min,YAV_min
     $        ,fn_min)


c     Update total annealing step-counter & efficiency summation. Note
c     that all chains are counted, but Brute-Force steps are not.
         Ntot_anneal = Ntot_anneal + N_chains * N_steps
         eff_anneal  = eff_anneal + eff_Scubi * N_chains * N_steps
c     -----------------------------------------------------------------------


c     ----------------> Replace Cpar_last by Cpar_min... <-------------------
c     Here we reset Cpar_last to Cpar_min, so that next time it will
c     start from its best location. Also, the chain with highest energy
c     is replaced by best Global model so far. This should improve their
c     convergence a tiny little bit. To use this feature you must set
c     i_RestartChains = "YES" (= 1).

!Cid01April07-FIX! Found a "BUG" here while preparing the final
!     v04! I did not realize this before because of another bug (namely,
!     that we did not use i_RestartChains = 1 in the BI-phase fits)! The
!     new bug is that by setting the last values equal to the best ones,
!     if one of these best chain values = the global best so far then
!     the trick of resetting the chain with worst energy = to the best
!     global model may result in this chain-values appearing TWICE!! Ie,
!     2 of the chains would start from IDENTICAL positions. This is not
!     a killer bug, but I never wanted to do this...  
!
!     To fix it I 1st figure out if any of the chain-minima differ from
!     the global-minimum by less than 1/1000 in energy (this is flagegd
!     with i_GEminIsAmongCEmins = 1 below). ONLY if NONE of them is this
!     close to the minimum I then apply the Worst2Best chain trick. (I
!     suspect that this is never gonna happen, but to be on the safe
!     side I decided to code it in this way).
!
!     I also introduce a new option: i_RestartChains = 2, which will
!     reset ONLY the worst chain to the best model, but leave the others
!     wherever they are.
         if (i_RestartChains.EQ.1) then
            i_GEminIsAmongCEmins = 0
            do i_chain=1,N_chains
               do j=1,N_par
                  Cpar_last(j,i_chain) = Cpar_min(j,i_chain)
               end do
               CEne_last(i_chain) = CEne_min(i_chain)

               test_diff = abs(GE_min - CEne_min(i_chain))
               if (test_diff.LE.(0.001*GE_min)) i_GEminIsAmongCEmins = 1
            end do

            if (i_GEminIsAmongCEmins.EQ.0) then
               WorstEne     = CEne_min(1)
               i_WorstChain = 1
               do i_chain=2,N_chains
                  if (CEne_min(i_chain).GT.WorstEne) then
                     WorstEne     = CEne_min(i_chain)
                     i_WorstChain = i_chain
                  end if
               end do
               do j=1,N_par
                  Cpar_last(j,i_WorstChain) = Gpar_min(j)
                  Cpar_min(j,i_WorstChain)  = Gpar_min(j)
               end do
               CEne_last(i_WorstChain) = GE_min
               CEne_min(i_WorstChain)  = GE_min
               if (i_verbose.GT.1) write (*,'(a,a)')
     $              '[Fit_Scubidu] GEmin NOT among the CEne_mins!' ,
     $              ' Worst2Best trick applied.'
            end if
         end if

!     New option: With i_RestartChains = 2 only the worst chain is
!     reset, and to the best global minimum.
         if (i_RestartChains.EQ.2) then
            WorstEne     = CEne_min(1)
            i_WorstChain = 1
            do i_chain=1,N_chains
               if (CEne_min(i_chain).GT.WorstEne) then
                  WorstEne     = CEne_min(i_chain)
                  i_WorstChain = i_chain
               end if
            end do

            do j=1,N_par
               Cpar_last(j,i_WorstChain) = Gpar_min(j)
               Cpar_min(j,i_WorstChain)  = Gpar_min(j)
            end do
            CEne_last(i_WorstChain) = GE_min
            CEne_min(i_WorstChain)  = GE_min
         end if

!     This was how things were up to v04 (ie, up to 01/April/2007).
!         if (i_RestartChains.EQ.1) then
!            WorstEne     = CEne_min(1)
!            i_WorstChain = 1
!            do i_chain=1,N_chains
!               do j=1,N_par
!                  Cpar_last(j,i_chain) = Cpar_min(j,i_chain)
!               end do
!               CEne_last(i_chain) = CEne_min(i_chain)
!
!               if (CEne_min(i_chain).GT.WorstEne) then
!                  WorstEne     = CEne_min(i_chain)
!                  i_WorstChain = i_chain
!               end if
!            end do
!
!            do j=1,N_par
!               Cpar_last(j,i_WorstChain) = Gpar_min(j)
!               Cpar_min(j,i_WorstChain)  = Gpar_min(j)
!            end do
!            CEne_last(i_WorstChain) = GE_min
!            CEne_min(i_WorstChain)  = GE_min
!         end if
c     -----------------------------------------------------------------------


c     -----> Golden Search for better kinematical parameters: v0 & vd <------
         if (IsKinFixed.NE.1) then


c     1st RESTORE f_base to original un-smoothed & un-shifted base spectra!!
            if (i_verbose.GT.1) write (*,'(a)')
     $           '[Fit_Scubidu] Starting Re-fit of v0 & vd...'
            do i=1,Nl_obs
               do j=1,N_base
                  f_base(j,i) = Of_base(j,i)
               end do
            end do


c     Define "Kinematics-free" spectrum GSf_syn_NoKin, passed by the
c     common/GoldenSearch_aux/. 
            call Update_AV_array(N_base,AV_min,YAV_min,AV_tot)
            call calc_Fsyn(x_min,AV_tot,GSf_syn_NoKin)


c     Compute chi2 of current model - USES fun_v0_chi2, which convolves
c     GSf_syn_NoKin through the kinematical filter! 
c     OBS1: This is the same as chi2 = FK_chi2(x_min,AV_tot,v0_min,vd_min)
c     OBS2: This should differ slightly from the chi2_min returned from
c           the Metropolis routine because in fun_v0_chi2 & fun_vd_chi2 
c           the reddening term is convolved with the Gaussian filter too.
c     OBS3: Note that here NO AV-truncated-series trick is used in neither 
c           fun_v0_chi2 nor fun_vd_chi2!
            chi2 = fun_v0_chi2(v0_min)


c     Find (with a Golden-Section-Search method) the vd & v0 which
c     produce the best chi2-match using the current best (x,AV,YAV)
c     estimated above. The searches are done separately, and in this
c     order. Values of v0 & vd outside their _low ==> _upp limits are
c     rejected. Seems to work fine, but it's rather slow. (Can be
c     improved with FFT-convolution...). OBS: In Starlight05 I
c     introduced the chi2 = chi2_vd & chi2 = chi2_v0 resets which were
c     not present before.
            ax  = 0.
            bx  = 500.
            tol = 1.e-6
            call mnbrak(ax,bx,cx,fa,fb,fc,fun_vd_chi2)
            chi2_vd = golden(ax,bx,cx,fun_vd_chi2,tol,vd_Gold)
            vd_Gold = abs(vd_Gold)
            if ((chi2_vd.LT.chi2).AND.(vd_Gold.GE.vd_low).AND.
     &           (vd_Gold.LE.vd_upp)) then
               vd_min = vd_Gold
               chi2   = chi2_vd
            end if

            ax  = -300.
            bx  = 300.
            tol = 1.e-6
            call mnbrak(ax,bx,cx,fa,fb,fc,fun_v0_chi2)
            chi2_v0 = golden(ax,bx,cx,fun_v0_chi2,tol,v0_Gold)
            if ((chi2_v0.LT.chi2).AND.(v0_Gold.GE.v0_low).AND.
     &           (v0_Gold.LE.v0_upp)) then
               v0_min = v0_Gold
               chi2   = chi2_v0
            end if              

            if (i_verbose.GT.1) write (*,'(a)') '[Fit_Scubidu] ...done!'
         end if


c     Store v0_min & vd_min onto v0 (we could use only the *_min variables...)
         v0 = v0_min
         vd = vd_min
c     -----------------------------------------------------------------------


c     ------------------------> Screen output <------------------------------
!     Screen check! 
         if (i_verbose.GT.0) then
            write (*,912)
 912        format ('#   i_loop    Temp  eps           N ',
     &           'chi2         adev    eff  |   v0    vd   | ',
     &           'sum      A_V   Y_AV |    x_j... [%]')

            if (IsKinFixed.EQ.1) then
               chi2 = 2. * GE_min / float(Nl_eff)
               call calc_Fsyn(x_min,AV_tot,f_syn)
            else 
               chi2 = chi2 / float(Nl_eff)
               call Kcalc_Fsyn(Nl_obs,x_min,AV_tot,v0_min,vd_min,f_syn)
            end if
            adev  = F_adev(f_syn)
            sum_x = 100. * Sum_of_x(x_min,N_base)

            write (*,913) 'FitScub' , i_loop , Temperature, eps(1) ,
     &           Ntot_anneal , chi2 , adev , 100.*eff_Scubi ,
     &           v0_min , vd_min ,
     &           sum_x , AV_min , YAV_min ,
     &           (100.*x_min(j),j=1,min(10,N_base))
 913        format (a7,i3,1x,1p,e7.1,0p,2x,f5.3,1x,i9,1x,1p,e10.4,0p,2x,
     $           f7.4,2x,f4.1,' | ',f6.1,1x,f5.1,' | ', f7.3,1x,f5.2,1x
     $           ,f5.2,' | ', 10(1x,f5.2))
            do j_out=2,int(N_base/10)+1
               j1 = (j_out - 1) * 10 + 1
               j2 = min(j1+9,N_base)
               write (*,'(101x,10(1x,f5.2))') (100.*x_min(j),j=j1,j2)
            end do

            write (*,'(102x,a,1p,e12.4)') 'Total chi2 = ', chi2 * Nl_eff

!     Screen output mean age & z to check convergence. 
!     OBS: Gives stupid results when called from EXOs-routine!
            if (iMassAgeZ_verbose.GT.0) then
               call Calc_Mean_Age_and_Z(N_base,x_min,a_age,s_age,
     &              a_met,s_met)
               write (*,914) a_age , s_age , a_met , s_met
 914           format(102x,'<log t> = ',f7.4,' +/- ',f7.4,
     &              '   <Z> = ',f7.4,' +/- ',f7.4)


!     Screen output of "total mass". Actually, Mass_tot is just
!     proportional to the true mass. I just output it here to have a
!     coarse idea of how robust the total mass is...  
!     OBS: Gives stupid results when called from EXOs-routines!
!     Cid@lynx 19/Mar/2004
               N_aux = N_base
               if (i_FitPowerLaw.EQ.1) N_aux = N_base - 1
               Mass_tot = 0.
               do j=1,N_aux
                  Mass_tot = Mass_tot + x_min(j) * 
     &                 (10.**(0.4 * AV_tot(j) * q_norm)) / fbase_norm(j)
               end do
               Mass_tot = Mass_tot * 383171.17
               write (*,915) Mass_tot , log10(Mass_tot)
 915           format(102x,'Mass = ',1p,e12.3,6x,'log Mass = ',0p,f8.3,
     &           '  [ref units]')
            end if


!     Inform whether Brute-Force AV/YAV search helped...'
               if (i_BruteForceFlag.EQ.1) write (*,'(102x,a)') 
     &           'OBS: Brute-Force AV/YAV search helped in this loop!'
         end if
c     -----------------------------------------------------------------------


c     Goto next annealing-loop iteration
      end do
c ***************************************************************************


c     Compute final annealing efficiency (just for output) & go home:)
      eff_anneal = eff_anneal / Ntot_anneal


c     RESTORE f_base to original un-smoothed & un-shifted base spectra
c     if no kinematical fitting was atempted above.
      if (IsKinFixed.EQ.1) then
         do i=1,Nl_obs
            do j=1,N_base
               f_base(j,i) = Of_base(j,i)
            end do
         end do
      end if


      end
c ###########################################################################




c ###########################################################################
c #                             SCUBIDUCADEVOCE                             #
c ###########################################################################
c     Evolve N_chains with Metropolis N_sim steps at a time, iterating
c     with more steps until GR-convenrged is achieved or more than
c     Nmax_steps are taken.
c
c     LOTS of details to comment!!!!!!!
c
c     Cid@Lagoa - 25/Feb/2005

      SUBROUTINE ScubiduCadeVoce(idum,N_par,N_chains,N_sim,Nmax_steps
     $     ,Temperature,R_Threshold,i_verbose,IsBurInOver,Cpar_last
     $     ,CEne_last,eps_in,eff_Scubi,N_steps,Gpar_min,GE_min,Cpar_min
     $     ,CEne_min,Cpar_low,Cpar_upp,Gpar_ave)


c ***************************************************************************
c *				DEFINITIONS				    *
c ***************************************************************************
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400)
      parameter (Nchains_max = 1000,Npar_max = 403)

c     Current & tentative new parameter, and step-size-scales for each
c     parameter (for built-in Metropolis)
      real par(Npar_max) , par_try(Npar_max)
      real eps_in(Npar_max) , eps(Npar_max)

c     Global minimum & average & std. dev.
      real Gpar_min(Npar_max) , Gpar_ave(Npar_max) , Gpar_sig(Npar_max)

c     "Cpar" denotes Chain-parameter (ie, for each chain...) matrices...
      real Cpar_ave(Npar_max,Nchains_max)
      real Cpar_var(Npar_max,Nchains_max)
      real Cpar_last(Npar_max,Nchains_max)
      real Cpar_min(Npar_max,Nchains_max)
      real Cpar_low(Npar_max) , Cpar_upp(Npar_max)

c     Chain energies (current/last & minimum)
      real CEne_last(Nchains_max) , CEne_min(Nchains_max)

c     Summations of par & par**2 "Memory" arrays. Using double precision!
      double precision S_par(Npar_max,Nchains_max)
      double precision S_par2(Npar_max,Nchains_max)

c     Scaling factor & efficiency arrays for each chain
      real Chain_alpha(Nchains_max) , Chain_eff(Nchains_max)

c     GR_R(j) = Gelman & Rubin covergence ratio for parameter j.
      real GR_R(Npar_max)

c     Auxiliar array(s)
      real aux1(Npar_max) , aux2(Npar_max)

c     This common is used only to read N_YAV_components
      common/YAV_stuff/ ind_YAV(Nb_max) , N_YAV_components

c     We don't change NRC_AV directly here, but it's good to know if it
c     changes on the way...
      common/RC_AV_Stuff/ NRC_AV , i_RC_CRASH_FLAG , NRC_AV_Default

c     Lower & upper limits on parameters (x,AV,YAV,fn)
      common/ParLimits/ par_low(Npar_max) , par_upp(Npar_max)

c     ScubiduCadeVoce Technical Parameters
      common/Scubi_TechnicalParameters/ alpha , Falpha , IsGRTestHard ,
     $     eff_IDEAL , i_UpdateEps , Feps , i_UpdateAlpha ,
     $     i_MoveOneParOnly , i_HelpParWithMaxR , prob_jRmax ,
     $     Npar4GRtest ,i_UpdateAVYAVStepSeparately ,
     $     ind_HelpParMove2Average(Npar_max) ,
     $     prob_HelpParMove2Average(Npar_max)

      common/OutputFileUnits/ i_UnitScubi
c ***************************************************************************


c ***************************************************************************
c *                             INITIALIZATION                              *
c ***************************************************************************
c     Reset what needs to be reset'd! 
      N_steps = 0
      do j=1,N_par
         GR_R(j) = 0.
         do i_chain=1,N_chains
            S_par(j,i_chain)    = 0.
            S_par2(j,i_chain)   = 0.
         end do
      end do

      do i_chain=1,N_chains
         Chain_alpha(i_chain) = alpha
      end do


c     Counter of number of chain-minima found throughout this Scubi run
c     (just for control).
      N_min  = 0


c     Store input eps (eps_in) in eps (which may change during the run).
      do j=1,N_par
         eps(j) = eps_in(j)
      end do


c     Define who's who in par() vector.
      N_base = N_par - 3
      j_AV   = N_base + 1
      j_YAV  = N_base + 2
      j_fn   = N_base + 3


c     Reset "Burn-In" flag & global-efficiency, as well as the index of
c     parameter with largest GR_R, and reset NRC_AV_last to its initial
c     value.
      IsBurInOver = 0
      eff_Scubi   = 0.
      jR_max      = 0
      NRC_AV_last = NRC_AV


c     Defines how many pars will enter the GR-test (exclude fn & maybe
c     YAV).  It's necessary to define it here in case we call this
c     routine using a condensed base...
      Npar4GRtest = N_par - 1 
      if (N_YAV_components.EQ.0) Npar4GRtest = N_par - 2
c ***************************************************************************


c ***************************************************************************
c *               BURN-IN CONVERGENCE LOOP (at fixed Temperature)           *
c ***************************************************************************
c     Define N_steps = number of steps taken in EACH chain AFTER the
c     next call to Metropolis is over. (Will come back here & add more
c     points if GR-convergence test fails...)
 100  N_steps = N_steps + N_sim 


c     Reset counters of total & rejected AV/YAV steps (valid only for
c     the next N_sim * N_chains moves).
      N_AVYAV_steps = 0
      N_AVYAV_skip  = 0


c     -------------------- Run N_chains Metropolis chains -------------------
c     Run N_chains Metropolis chains, each N_sim steps long.
      do i_chain=1,N_chains


c     ++++++++++++++++++ Metropolis run for chain # i_chain +++++++++++++++++
c     (0) Reset initial par-vector, Energy & N_skip. Start Metropolis run.
c     (1) Modify par => par_try (within low => upp boundaries).
c         Move can be in all par_j's or in just one-at-a-time, 
c         according to i_MoveOneParOnly = 0/1.
c     (2) Compute new energy (Ene_try); then accept or not the change.
c     (3) Verify if new state is an energy minimum (for current chain).
c     (4) Updates summations of p_j & p_j**2 & goto next step (i_sim).
c     (5) When done, compute chain-efficiency.
         do j=1,N_par
            par(j) = Cpar_last(j,i_chain)
         end do

!     OBS: Ene MUST be computed with a direct call to Fpar_VFchi2 with
!     i_IsAVstillTheSame = 0 = "NO" here, since AV/YAV varies from chain
!     to chain.
         Ene    = 0.5 * Fpar_VFchi2(N_par,par,0)
         N_skip = 0


!Cid06Dec05-FIX! Introduce counter of repeated steps (ie those which
!     don't vary)...
         N_Retry = 0

         do i_sim=1,N_sim


c     For safety, recompute Ene & CEne_min here if NRC_AV changed since
c     last time an energy was computed for this chain. NRC_AV_last
c     itself is only updated when all chains have run N_sim steps.
            if (NRC_AV.NE.NRC_AV_last) then
               Ene = 0.5 * Fpar_VFchi2(N_par,par,0)
               do j=1,N_par
                  aux1(j) = Cpar_min(j,i_chain)
               end do
               CEne_min(i_chain) = 0.5 * Fpar_VFchi2(N_par,aux1,0)
            end if


            if (i_MoveOneParOnly.EQ.1) then

c     ........... Modifies par --> par_try: One par_j per move!..............
c     First set par_try = par. Then chose ONE of the N_par variables &
c     perturb it. If the chosen variable doesn't really vary (eg, it was
c     and still is = to its lower limit), then try again, otherwise this
c     step would be wasted. This ensures that all N_sim steps actually
c     attempt to change the state. If i_HelpParWithMaxR = 1 then the
c     parameter with largest R_j will have an extra chance of beeing
c     picked.

c     par_try <= par
               do j=1,N_par
                  par_try(j) = par(j)
               end do

c     Pick jj 
 10            if ((i_HelpParWithMaxR.EQ.1).AND.(jR_max.GT.0).AND.
     $              (ran2(idum).LE.prob_jRmax)) then
                  jj = jR_max
               else
                  jj = N_par * ran2(idum) + 1
               end if 
               if (jj.GT.N_par) goto 10

c     Perturb jj-th component of par(jj) => par_try(jj).
               par_try(jj) = par(jj) + Chain_alpha(i_chain) * eps(jj)
     $              * gasdev(idum)


c     Here we give a little extra help to par_j, inverting the sign of
c     the perturbation (with probability = prob_HelpParMove2Average) if
c     the move took par_j away from the current global-average
c     value. This makes par_j converge faster, and is analogous to using
c     a smarter parameter-change proposal-scheme. Note that Gpar_ave is
c     only defined after N_steps > N_sim (ie, after the 1st loop).
               if ((ind_HelpParMove2Average(jj).EQ.1).AND.(N_steps.GT.
     $              N_sim)) then
                  z1    = par(jj) - (par_try(jj) - par(jj))
                  dist0 = abs(par_try(jj) - Gpar_ave(jj))
                  dist1 = abs(z1 - Gpar_ave(jj))
                  if ((dist1.LT.dist0).AND.(ran2(idum).LE.
     $                 prob_HelpParMove2Average(jj))) par_try(jj) = z1
               end if


c     Limit par_try(jj) to its boundaries & try again if nothing changed.
               if (par_try(jj).LT.par_low(jj)) par_try(jj) = par_low(jj)
               if (par_try(jj).GT.par_upp(jj)) par_try(jj) = par_upp(jj)
!INFINITE-LOOP-BUG!          if (par_try(jj).EQ.par(jj)) goto 10
!Cid06Dec05-FIX! Update N_Retry & only retry the step if N_Retry <= N_sim!
!     (Otherwise we may get into an infinite loop...)
               if (par_try(jj).EQ.par(jj)) then
                  N_Retry = N_Retry + 1
                  if (N_Retry.LE.N_sim) goto 10
               end if


c     Perturbed x(jj) or fn ==> so renormalize x() to sum = fn.
c     Also, AV & YAV are the same ==> so set i_IsAVstillTheSame to "YES"!
               if ((jj.LE.N_base).OR.(jj.EQ.j_fn)) then

                  fn_try = par_try(j_fn)
                  sum_x = 0.
                  do j=1,N_base
                     sum_x = sum_x + par_try(j)
                  end do
                  do j=1,N_base
                     par_try(j) = (par_try(j) / sum_x) * fn_try
                  end do
                  i_IsAVstillTheSame = 1

c     Perturbed YAV but we are not fitting YAV ==> so try again!
               else if ((jj.EQ.j_YAV).AND.(N_YAV_components.EQ.0)) then

                  goto 10

c     Perturbed AV or YAV ==> so set i_IsAVstillTheSame to "NO". Also,
c     update counter of AV/YAV steps.
               else if ((jj.EQ.j_AV).OR.(jj.EQ.j_YAV)) then 

                  i_IsAVstillTheSame = 0
                  N_AVYAV_steps = N_AVYAV_steps + 1

c     Safety precaution (shouldn't really get here anyway)
               else

                  print *, ' ??? What am I doing here??? ' , 
     $                 i_IsAVstillTheSame , jj , par(jj) , par_try(jj) 
                  par_try(jj) = par(jj)
                  goto 10

               end if
c     .......................................................................

            else

c     ........ Modifies par --> par_try: Shakes ALL par_j's per move!........
c     Perturb/move par => par_try (limiting to low <=> upp boundaries).
c     Then renormalize population vector x_try = par_try(1...N_base) to
c     fn_try = par_try(j_fn). Also turn-off i_IsAVstillTheSame if AV/YAV
c     changed (which's very likely in this scheme).
               do j=1,N_par
                  par_try(j) = par(j) + Chain_alpha(i_chain) * eps(j)
     $                 * gasdev(idum)
                  if (par_try(j).LT.par_low(j)) par_try(j) = par_low(j)
                  if (par_try(j).GT.par_upp(j)) par_try(j) = par_upp(j)
               end do

               fn_try = par_try(j_fn)
               sum_x = 0.
               do j=1,N_base
                  sum_x = sum_x + par_try(j)
               end do
               do j=1,N_base
                  par_try(j) = (par_try(j) / sum_x) * fn_try
               end do

               i_IsAVstillTheSame = 0
               if ((par_try(j_AV).EQ.par(j_AV)).AND.(par_try(j_YAV).EQ.
     $              par(j_YAV))) i_IsAVstillTheSame = 1
c     .......................................................................

            
            end if


c     Compute energy at par_try. Then accept new position or not
c     according to Metropolis rule. Also, verify if par_try is a new
c     minimum for this chain.
            Ene_try = 0.5* Fpar_VFchi2(N_par,par_try,i_IsAVstillTheSame)
            prob    = 1. - exp(-(Ene_try - Ene) / Temperature)

            if ((Ene_try.LE.Ene).OR.(ran2(idum).GT.prob)) then

               do j=1,N_par
                  par(j) = par_try(j)
               end do
               Ene = Ene_try
               if (Ene.LT.CEne_min(i_chain)) then
                  do j=1,N_par
                     Cpar_min(j,i_chain) = par(j)
                  end do
                  CEne_min(i_chain) = Ene
                  N_min = N_min + 1
               end if

            else

c     Step was rejected. Since we are using the fast skip-over-AV-loop
c     trick, the saved-summations in the fast-chi2-routine must be
c     restored to correct values if a change in AV (or YAV) was
c     rejected, as is the case in the "if" below. A simple call to the
c     routine using the previous values will fix it. In this case, we
c     also update the counter of rejected AV/YAV steps.
               N_skip = N_skip + 1
               if ((i_MoveOneParOnly.EQ.1).AND.(i_IsAVstillTheSame.EQ.0)
     $              ) rubish = Fpar_VFchi2(N_par,par,0)
               if ((jj.EQ.j_AV).OR.(jj.EQ.j_YAV)) N_AVYAV_skip =
     $              N_AVYAV_skip + 1

            end if


c     Update par & par**2 summations for this chain (used in GR-test).
            do j=1,N_par
               S_par(j,i_chain)  = S_par(j,i_chain)  + par(j)
               S_par2(j,i_chain) = S_par2(j,i_chain) + par(j) * par(j)
            end do


c     Save Step if i_UnitScubi > 0
            if (i_UnitScubi.GT.0) call SaveStep(i_sim,i_chain,N_par,par
     $           ,Ene)

c     Goto next step i_sim for chain # i_chain
         end do
c     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


!     Screen check of running-AV/YAV-efficiency...
!         AVYAV_eff = 1. - float(N_AVYAV_skip) / float(N_AVYAV_steps)
!         write (*,'(a,3(1x,i5),3(a,f10.5))') ' AVYAVeff ' , i_chain ,
!     $        N_AVYAV_steps , N_AVYAV_skip ,  '  eff%=' , 100.
!     $        * AVYAV_eff , ' eps_AV=', eps(j_AV) , '  last_AV='
!     $        ,par(j_AV)


c     Compute chain-efficiency (latest N_sim steps only) & update global
c     efficiency summation (eff_Scubi).
         Chain_eff(i_chain) = 1. - float(N_skip) / float(N_sim)
         eff_Scubi          = eff_Scubi + N_skip

c     Store final chain parameters in Cpar_last (used to restart it the
c     next time), and compute Cpar_ave = mean & Cpar_var = variance of
c     par(j), used in the GR test. Note the abs in the variance to avoid
c     precision bugs...
         do j=1,N_par
            Cpar_last(j,i_chain) = par(j)
            Cpar_ave(j,i_chain)  = S_par(j,i_chain) / float(N_steps)
            Cpar_var(j,i_chain)  = abs(S_par2(j,i_chain) -
     $           float(N_steps) * Cpar_ave(j,i_chain) * Cpar_ave(j
     $           ,i_chain)) / float(N_steps - 1)
         end do


c     Goto next chain
      end do
c     -----------------------------------------------------------------------


c     -------------- Re-evaluate CEne_min's if NRC_AV changed ---------------
c     If NRC_AV changed since last loops through the chains it's a good
c     idea to recompute all minimum chain-energies. Also, reset
c     NRC_AV_last to current value.
      if (NRC_AV.NE.NRC_AV_last) then 
         do i_chain=1,N_chains
            do j=1,N_par
               aux1(j) = Cpar_min(j,i_chain)
            end do
            CEne_min(i_chain) = 0.5 * Fpar_VFchi2(N_par,aux1,0)
         end do
         NRC_AV_last = NRC_AV
      end if
c     -----------------------------------------------------------------------


c     -- Brief break to search 4 a better Global minimum & define Gpar_ave --
c     Look up which chain has the smallest minimum & store it in the
c     Global minimum arrays if it's better than GE_min.
      do i_chain=1,N_chains
         if (CEne_min(i_chain).LT.GE_min) then
            do j=1,N_par
               Gpar_min(j) = Cpar_min(j,i_chain)
            end do
            GE_min = CEne_min(i_chain)
         end if
      end do


c     Verify if one of the N_chains-Cpar_ave's is a better Global-minimum...
      do i_chain=1,N_chains
         do j=1,N_par
            aux1(j) = Cpar_ave(j,i_chain)
         end do
         Ene_ave = 0.5 * Fpar_VFchi2(N_par,aux1,0)
         if (Ene_ave.LT.GE_min) then 
            do j=1,N_par
               Gpar_min(j) = aux1(j)
            end do
            GE_min = Ene_ave
         end if
      end do


c     Define global-par-average & verify if it is a better Global-minimum...
      do j=1,N_par
         Gpar_ave(j) = 0.
         do i_chain=1,N_chains
            Gpar_ave(j) = Gpar_ave(j) + Cpar_ave(j,i_chain)
         end do
         Gpar_ave(j) = Gpar_ave(j) / float(N_chains)
      end do
      GEne_ave = 0.5 * Fpar_VFchi2(N_par,Gpar_ave,0)
      if (GEne_ave.LT.GE_min) then 
         do j=1,N_par
            Gpar_min(j) = Gpar_ave(j)
         end do
         GE_min = GEne_ave
      end if
c     -----------------------------------------------------------------------


!!!????SERIOUS - YAV(?)-BUG????????? - por ora esquecido/deixado de lado...!!!
!     if (GE_min.LT.1000.) then
!        write (*,*) 'FILHADAPUTA! ',GE_min,0.5*Fpar_VFchi2(N_par
!    $        ,Gpar_min,0)
!        write (*,*) '  N_steps=',N_steps , '   Temp=',Temperature
!        z1 = Gpar_min(j_AV)
!        z2 = Gpar_min(j_YAV)
!!       z3 = 0.5 * Fpar_Slowchi2(N_par,Gpar_min)
!        write (*,*) ' AV=' , z1 , ' YAV=',z2, '  slow=',z3
!        write (*,*) ' par= ',(Gpar_min(j),j=1,N_par)
!        write (*,*) 
!!!!!!????????????????????????????????????????????????????????????????!!!!!!!


c     --------------- Update alpha & eps(AV/YAV) (if wanted) ----------------
c     Update alpha based on efficiency using an exponential recipe.
c     Updates can be made for all chains at once (i_UpdateAlpha = 1), in
c     which case the mean efficiency is used.  Using i_UpdateAlpha = 2
c     will update alpha for each chain using its own efficiency. ATT:
c     Alpha-updates are only done if the resulting step size is not too
c     big (> 20% of low==>upp range)!!
      if (i_UpdateAlpha.EQ.1) then

         call avevar(Chain_eff,N_chains,ave_eff,var_eff)
         alp_factor = Falpha**((ave_eff - eff_IDEAL) / eff_IDEAL)
         if (ave_eff.GE.eff_IDEAL) alp_factor = Falpha**((ave_eff -
     $        eff_IDEAL) / (1. - eff_IDEAL))
         test_alpha = Chain_alpha(1) * alp_factor

         do j=1,N_par
            z1 = test_alpha * eps(j)
            z2 = (par_upp(j) - par_low(j)) / 5.
            if (z1.GT.z2) goto 50
         end do

         do i_chain=1,N_chains
            Chain_alpha(i_chain) = test_alpha
         end do

      else if (i_UpdateAlpha.EQ.2) then

         do i_chain=1,N_chains
            alp_factor = Falpha**((Chain_eff(i_chain) - eff_IDEAL) /
     $           eff_IDEAL)
            if (Chain_eff(i_chain).GE.eff_IDEAL) alp_factor = 
     $           Falpha**((Chain_eff(i_chain) - eff_IDEAL) / (1. -
     $           eff_IDEAL))
            test_alpha = Chain_alpha(i_chain) * alp_factor

            do j=1,N_par
               z1 = test_alpha * eps(j)
               z2 = (par_upp(j) - par_low(j)) / 5.
               if (z1.GT.z2) goto 40
            end do
            Chain_alpha(i_chain) = test_alpha

 40         DoNothingOnThisSillyLine = 0.
         end do

      end if
 50   DoNothingOnThisSillyLine = 0.


c     Adjust the step-size for AV & YAV according to how efficient were
c     the moves in these 2 variables in the past N_sim steps of all
c     chains. This is equivalent to the alpha-update above, but here we
c     operate on eps itself instead of on alpha. Again, precautions are
c     taken against too large step-sizes. Also, we consider that the
c     effective step size depends on alpha (updated above).
      if (i_UpdateAVYAVStepSeparately.EQ.1) then
         AVYAV_eff = 1. - float(N_AVYAV_skip) / float(N_AVYAV_steps)

         eps_factor = Falpha**((AVYAV_eff - eff_IDEAL) / eff_IDEAL)
         if (AVYAV_eff.GE.eff_IDEAL) eps_factor = Falpha**((AVYAV_eff -
     $        eff_IDEAL) / (1. - eff_IDEAL))

         alp_ave = Chain_alpha(1)
         if (i_UpdateAlpha.EQ.2) call avevar(Chain_alpha,N_chains
     $        ,alp_ave,alp_var)

         z1 = alp_ave * eps_factor * eps(j_AV)
         z2 = (par_upp(j_AV) - par_low(j_AV)) / 5.
         if (z1.LT.z2) eps(j_AV) = eps_factor * eps(j_AV)

         if (N_YAV_components.GT.0) then
            z1 = alp_ave * eps_factor * eps(j_YAV)
            z2 = (par_upp(j_YAV) - par_low(j_YAV)) / 5.
            if (z1.LT.z2) eps(j_YAV) = eps_factor * eps(j_YAV)
         end if

!     Screen check
!        write (*,'(a,2(1x,i5),3(a,f10.6))') '  *> AVYAV-step-update: '
!    $        ,N_AVYAV_steps , N_AVYAV_skip ,  '  eff%=' , 100.
!    $        * AVYAV_eff, ' eps_AV=',eps(j_AV), ' eps_YAV=',eps(j_YAV)
      end if
c     -----------------------------------------------------------------------


c     ----------------------- GR Convergence Test ---------------------------
c     Perform Gelman & Rubin R-test. Note that we EXCLUDE fn from the
c     test! This is justifiable because fn is already inside the
c     pop-vector... Since fn is the last entry in par(), we just pretend
c     there are N_par - 1 parameters. Also, if no YAV is beeing fit,
c     exclude YAV from the GR test too.
      call Calc_GR_R(N_steps,N_chains,Npar4GRtest,Cpar_ave,Cpar_var,
     $     R_Threshold,GR_R,GR_R_ave,IsAveBurnInOver,IsIndBurnInOver)


c     Verify if GR-R converged. If so, we are close enough to the end of
c     the burn-in to get out of the convergence loop and start a proper
c     mapping of the parameter space with other routines. Otherwise,
c     iterate again adding more points to the chains, until either R <
c     R_Threshold or we exceed our limit on N_steps... (<= Nmax_steps).
      i_aux = IsAveBurnInOver
      if (IsGRTestHard.EQ.1) i_aux = IsIndBurnInOver


c     Burn-In over:) print something & go home.
      if (i_aux.EQ.1) then 

         IsBurInOver = 1
         if (i_verbose.GE.1) write (*,'(a,i8,a,1p,e9.3)')
     $        '[Scubidu] Burn In over after ' ,N_steps ,
     $        ' steps :)    @ T = ' , Temperature
         goto 200

c     GR-convergence was NOT reached after Nmax_steps:( Issue a Warning
c     and give up!
      else if ((i_aux.EQ.0).AND.(N_steps.GT.Nmax_steps)) then

         if (i_verbose.EQ.1) write (*,'(a,i8,1p,2(a,e9.3),a)')
     $        '[Scubidu] ATT: NO CONVERGENCE after ' ,N_steps ,
     $        ' steps:(    @ Temperature = ' , Temperature, '  <R> ='
     $        ,GR_R_ave,' :(   ...but will keep going...'
         goto 200

      end if
c     -----------------------------------------------------------------------


c     ------------------- Burn In NOT over: Iterate More --------------------
c     Burn-In NOT over:(  Do a few things, print stuff & iterate more...
 300  alp_ave = Chain_alpha(1)
      alp_var = 0.
      if (i_UpdateAlpha.EQ.2) call avevar(Chain_alpha,N_chains,alp_ave
     $     ,alp_var)
      call avevar(Chain_eff,N_chains,ave_eff,var_eff)


c     Find out which parameter has the largest R. par(jR_max) will be
c     given an extra help in the next loop through the chains.
      R_max  = GR_R(1)
      jR_max = 1
      do j=2,Npar4GRtest
         if (GR_R(j).GT.R_max) then
            R_max  = GR_R(j)
            jR_max = j
         end if
      end do


c     Increase step size for parameters with large R_j (compared to <R>)
c     NEED TESTS!!! Maybe ABANDON this feature????
      if (i_UpdateEps.EQ.1) then
         do j=1,Npar4GRtest
            if ((GR_R(j)/GR_R_ave).GT.3.0) then
               test_eps = eps(j) * Feps
               z1 = (par_upp(j) - par_low(j)) / 5.
               if ((alp_ave * test_eps).LT.z1) then 
                  write (*,*) ' ????Re-scaled eps_j: ', j , eps(j)
     $                 ,test_eps , GR_R(j) , GR_R_ave
                  eps(j) = test_eps
               end if
            end if
         end do
      end if 


c     Print stuff...
      if (i_verbose.GT.1) then
         min_pRmax = Cpar_last(jR_max,1)
         max_pRmax = Cpar_last(jR_max,1)
         do i_chain=1,N_chains
            z1 = Cpar_last(jR_max,i_chain)
            if (z1.LT.min_pRmax) min_pRmax = z1
            if (z1.GT.max_pRmax) max_pRmax = z1
            aux1(i_chain) = z1
         end do
         call avevar(aux1,N_chains,ave_aux,var_aux)
         write (*,1) '[Scubidu] N_steps=',N_steps ,' 2Emin=',2*GE_min ,
     $        '  T=',Temperature , '  a=',alp_ave , '  a*eps_x='
     $        ,alp_ave*eps(1), '  a*eps_AV=',alp_ave*eps(j_AV),
     $        '  eff%=',100*ave_eff ,'  <R>=' ,GR_R_ave , ' <' ,R_max
     $        ,jR_max, ' d=', max_pRmax - min_pRmax ,  ' sig=',
     $        sqrt(var_aux)
 1       format (a,i9,1p,a,e10.4,4(a,e8.2),0p,3(a,f6.2),1x,i3,2(a,f7.4))
      end if


c     Iterate all chains another N_sim steps
      goto 100
c     -----------------------------------------------------------------------
c ***************************************************************************


c ***************************************************************************
c *                             FINALIZATION...                             *
c ***************************************************************************
c     Just check if <par> yields a better model than the last point for
c     each chain. If so, reset Cpar_last to Cpar_ave (& maybe Cpar_min
c     too) before returning. Take the oportunity to verify if a new
c     global minimum is found on the way.
 200  do i_chain=1,N_chains
         do j=1,N_par
            par(j)  = Cpar_last(j,i_chain)
            aux1(j) = Cpar_ave(j,i_chain)
         end do
         Ene_last = 0.5 * Fpar_VFchi2(N_par,par,0)
         Ene_ave  = 0.5 * Fpar_VFchi2(N_par,aux1,0)

         CEne_last(i_chain) = Ene_last

         if (Ene_ave.LT.Ene_last) then 
            do j=1,N_par
               Cpar_last(j,i_chain) = Cpar_ave(j,i_chain)
            end do
            CEne_last(i_chain) = Ene_ave
         end if

         if (Ene_ave.LT.CEne_min(i_chain)) then
            do j=1,N_par
               Cpar_min(j,i_chain) = Cpar_ave(j,i_chain)
            end do
            CEne_min(i_chain) = Ene_ave
         end if

         if (Ene_ave.LT.GE_min) then 
            do j=1,N_par
               Gpar_min(j) = Cpar_ave(j,i_chain)
            end do
            GE_min = Ene_ave
         end if
      end do


c     Store low-->upp range among last parameters of all chains in
c     Cpar_low & _upp, just for control...
      do j=1,N_par
         Cpar_low(j) = Cpar_last(j,1)
         Cpar_upp(j) = Cpar_last(j,1)
         do i_chain=2,N_chains
            z1 = Cpar_last(j,i_chain)
            if (z1.LT.Cpar_low(j)) Cpar_low(j) = z1
            if (z1.GT.Cpar_upp(j)) Cpar_upp(j) = z1
         end do         
      end do


c     Define mean alpha (to be returned as "alpha")
      alp_ave = Chain_alpha(1)
      alp_var = 0.
      if (i_UpdateAlpha.EQ.2) call avevar(Chain_alpha,N_chains
     $     ,alp_ave,alp_var)
      alpha = alp_ave


c     Define overall efficiency (to be returned)
      eff_Scubi = 1. - eff_Scubi / float(N_steps * N_chains)


!     Screen output <chi2>, <eff>, <alpha>, ... chi2(<p>)...Gpar_ave &
!     p_last-stats...
      if (i_verbose.GT.1) then
         call avevar(Chain_eff,N_chains,ave_eff,var_eff)
         call avevar(CEne_last,N_chains,ave_Ene,var_Ene)
         call avevar(eps,N_par,ave_eps,var_eps)

         do j=1,N_par
            aux1(j) = 0.
            do i_chain=1,N_chains
               aux1(j) = aux1(j) + Cpar_last(j,i_chain)
            end do
            aux1(j) = aux1(j) / float(N_chains)
         end do
         chi2_ave_last = Fpar_VFchi2(N_par,aux1,0)

         write (*,'(1p,3(a,e12.5))')
     $        '[Scubidu] Leaving routine with <chi2> = ',2*ave_Ene,
     $        ' +/- ' , 2*sqrt(var_Ene) , '   |  chi2(<p_last>) = '
     $        , chi2_ave_last
         write (*,'(28x,1p,2(a,e12.5),a)') 'Gchi2_min = ' , 2. * GE_min
     $        , ' (FAST) &  ' ,Fpar_Slowchi2(N_par,Gpar_min)
     $        , ' (SLOW)'

         write (*,'(30x,2(a,f12.5))') '<eff %> = ' , 100* ave_eff ,
     $        ' +/- ', 100 * sqrt(var_eff)
         write (*,'(30x,1p,2(a,e12.5))') '<alpha> = ' , alp_ave,
     $        ' +/- ' ,sqrt(alp_var)
         write (*,'(30x,1p,4(a,e12.5))') '<eps_j> = ' , ave_eps, ' +/- '
     $        , sqrt(var_eps) , '   |  <a> * <eps_j>  = ',alp_ave
     $        * ave_eps , '   |  <a> * eps_AV  = ',alp_ave * eps(j_AV)
         write (*,'(34x,2(a,f12.5),a,i1,a)') '<R> = ' , GR_R_ave ,
     $        '  | R_Threshold = ',R_Threshold , '  (',IsGRTestHard,')'
         write (*,'(30x,a,i8,a,i8)') 'N_steps = ' , N_steps ,
     $        '   <= Nmax_steps = ' , Nmax_steps

         write (*,'(28x,a,1p,e12.5)') 'eff_min % = ' , 100.
     $        * float(N_min) / float(N_steps * N_chains)

         write (*,*)
         write (*,'(a,a)') '[Scubidu] Gpar_ave & par_last(j) ' ,
     $        'chain-stats after Scubi-run'
         do j=1,N_par

c     Define std-dev among the Cpar_ave's for each parameter. Also
c     define std-dev & etc among the Cpar_last's for each parameter
            do i_chain=1,N_chains
               aux1(i_chain) = Cpar_ave(j,i_chain) - Gpar_ave(j)
               aux2(i_chain) = Cpar_last(j,i_chain)
            end do

            call avevar(aux1,N_chains,ave_diff,var_diff)
            Gpar_sig(j) = sqrt(abs(var_diff))
            if (var_diff.LT.0.) Gpar_sig(j) = - Gpar_sig(j)

            call avevar(aux2,N_chains,ave_p,var_p)
            sig_p  = sqrt(var_p)
            min_p  = Cpar_low(j)
            max_p  = Cpar_upp(j)
            p_best = Gpar_min(j)

            z1 = 1.
            if (j.LE.N_base) z1 = 100.
            write (*,'(a,i3,7(a,f6.2),a,f7.3,a,i3,a,f7.3)')
     $           '[Scubidu] j =' ,j , 
     $           '  <Gp>= ',z1*Gpar_ave(j),' +/- ',z1 *Gpar_sig(j) ,
     $           ' | <p_last> = ',z1*ave_p,' +/- ',z1*sig_p , 
     $           '  |  Range=',z1*min_p,' --> ',z1*max_p ,
     $           ' |  Delta= ',z1*max_p - z1*min_p ,
     $           '  |  Gp_min= ',z1*p_best , ' ~ ',nint(z1*p_best) , 
     $           ' | R_j = ' , GR_R(j)
         end do

         write (*,*)
         write (*,'(a)') '[Scubidu] Chains Minima after Scubi-run'
         write (*,'(a,12(1x,f8.2))') '[Scubidu] Chain-chi2_min = '
     $        , (2.* CEne_min(i_chain),i_chain=1,N_chains)
         do j=1,N_par
            z1 = 1.
            if (j.LE.N_base) z1 = 100.
            write (*,'(a,i3,a,12(2x,f7.3))') '[Scubidu] j =' ,j ,
     $           '  |  p_j = ', (z1*Cpar_min(j,i_chain),i_chain=1
     $           ,N_chains)
         end do


c     Prints best chain energy & compare it to global minimum...
         z_min = CEne_min(1)
         i_min = 1
         do i_chain=2,N_chains
            if (CEne_min(i_chain).LT.z_min) then
               z_min = CEne_min(i_chain)
               i_min = i_chain
            end if
         end do
         write (*,'(a,i3,3(a,f8.2))') '[Scubidu] Best Chain = ' , i_min
     $        ,'  2*CEne_min = ', 2.*z_min , ' | 2*GE_min = ',2.*GE_min
     $        ,'  | Diff = ' , 2.*(z_min - GE_min)

         write (*,*)
      end if
c ***************************************************************************



      end
c ###########################################################################




c ###########################################################################
c     Unpack par-vector to (x,fn,AV,YAV) arrays!
c     Cid@Lagoa - 01/Abril/2005

      SUBROUTINE Unpack_par2xAVYAVfn(N_par,par,x,AV,YAV,fn)

      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400,Npar_max = 403)
      real par(Npar_max) , x(Nb_max)

      N_base = N_par - 3

      if (N_base.LE.0) then
         write (*,*) '[Unpack_par2xAVYAVfn] N_base <= 0!! N_base = ' ,
     $        N_base , '  N_par = ' , N_par
         stop
      end if

      j_AV   = N_base + 1
      j_YAV  = N_base + 2
      j_fn   = N_base + 3

      do j=1,N_base
         x(j) = par(j)
      end do
      AV  = par(j_AV)
      YAV = par(j_YAV)
      fn  = par(j_fn)

      end
c ###########################################################################



c ###########################################################################
c #              SAVE STEP IN A FILE (used to sample PDF...)                #
c ###########################################################################


      SUBROUTINE SaveStep(i_sim,i_chain,N_par,par,Ene)


c ***************************************************************************
c *				DEFINITIONS				    *
c ***************************************************************************
      implicit real (a-h,k-m,o-z)
      implicit integer (i,j,n)
      parameter (Nb_max = 400,Npar_max = 403)

c     par-vector (par) & pop-vector array (x) & AV_tot() = population
c     dependent extinction
      real par(Npar_max) , x(Nb_max) , AV_tot(Nb_max)

c     Mstars = array which says what fraction of the INITIAL stellar
c     mass in population # j is still in stars at age(j). mu_cor() are
c     the corrected mass-fractions.
      real Mstars(Nb_max) , mu_cor(Nb_max)

c     L2Mass (@ l_norm), Age & Z in the base
      real fbase_norm(Nb_max) ,age_base(Nb_max) , Z_base(Nb_max)

c     commons...
      common/OutputFileUnits/ i_UnitScubi
      common/Age_and_Met/ age_base , Z_base
      common/CalcMass/ q_norm , fbase_norm , i_FitPowerLaw , fobs_norm
c ***************************************************************************


c ***************************************************************************
c *			COMPUTE QUANTITIES FOR OUTPUT			    *
c ***************************************************************************
c     Define N_base, unpack par => x,AV,YAV,fn & define AV_tot array
      N_base = N_par - 3
      call Unpack_par2xAVYAVfn(N_par,par,x,AV,YAV,fn)
      call Update_AV_array(N_base,AV,YAV,AV_tot)


c     Ignore PL component (if it was included in the base)!
      if (i_FitPowerLaw.EQ.1) N_base = N_base - 1


c     Compute actual (corrected) stellar mass.
      Mcor_tot = 0.
      do j=1,N_base
         flux_j = x(j) * fobs_norm * 10.**(0.4 * AV_tot(j) * q_norm)
         mcor_j = flux_j / fbase_norm(j) * Mstars(j)
         Mcor_tot = Mcor_tot + mcor_j
      end do


c     Compute mass fractions (mu_cor) associated with each base
c     component.
      do j=1,N_base
         flux_j = x(j) * fobs_norm * 10.**(0.4 * AV_tot(j) * q_norm)
         mcor_j = flux_j / fbase_norm(j) * Mstars(j)
         mu_cor(j)  = mcor_j / Mcor_tot
      end do


c     Compute FLUX- & MASS-WEIGHTED mean age & z 
      sum_ft = 0.
      sum_fz = 0.
      sum_mt = 0.
      sum_mz = 0.
      sum_x  = 0.
      sum_m  = 0.
      do j=1,N_base
         sum_ft = sum_ft + x(j) * log10(age_base(j) + 1.)
         sum_fz = sum_fz + x(j) * Z_base(j)
         sum_x  = sum_x  + x(j)

         sum_mt = sum_mt + mu_cor(j) * log10(age_base(j) + 1.)
         sum_mz = sum_mz + mu_cor(j) * Z_base(j)
         sum_m  = sum_m  + mu_cor(j)
      end do
      at_flux = sum_ft / sum_x
      am_flux = sum_fz / sum_x
      at_mass = sum_mt / sum_m
      am_mass = sum_mz / sum_m
c ***************************************************************************


c ***************************************************************************
c *			OUTPUT STEP TO FILE i_UnitScubi			    *
c ***************************************************************************
      write (i_UnitScubi,'(i2,7(1x,f8.4))') i_chain 
     $     , at_flux , log10(am_flux/0.02)
     $     , at_mass , log10(am_mass/0.02)
     $     , AV , log10(Mcor_tot) , log10(2.*Ene)
c ***************************************************************************


      end
c ###########################################################################


!     MISCELANEOUS NOTES...

!     Probelma da enegia muito baixa... como consertar??? Tem a ver com
!     a expansao... uma possib eh sempre usar SlowChi2 qdo se acha um
!     minimo...
c     o prob eh que as vezes da o bug FDP antes do VF_chi2 se dar conta
c     de q a coisa vai mal (ie, antes do chi2 ficar negativo)...
c
c     bug tem q ser resolvido calibrando qdo ele acontece em funcao de
c     AV e YAV (possibly only AV+AYV...) Simulations? William??
c     Talvez o melhor seja incluir elementos avermelhados na base! WILLIAM!

!??   SERAH que nao vale a pena fixar AV, estimar x e refazer o treco AV
!??   a AV? Tipo o q a gente faz pra v0,vd com a Golden-search .,.....

!     tenho a impressao que podemos ser mais relaxed com o GR_R pra AV,
!     pois os valores das cadeias estao BEM proximos (~ centesimos de
!     mag) embora GR_R(AV) > 1.2....

!     have to go ASAP to a condensed base! OR TO Fixed v0 & vd (such
!     that RC terms need not be recomputed often!)

!     se nao mudar kin entao nao precisa recalcular RC sempre! (soh qdo
!     AV crescer muito!) o q apressaria bastante a coisa!  ideia: 1 -
!     convuliar obs e base com vd = 300 km/s; 2 - fazer um fit rapido; 3
!     descnvoluit tudo; 4 - estimar vd com um fit usando o pop-vector
!     rapido; 5 - fixar kinematics thereafter!
c +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

